Package: ribor
Title: An R Interface for Ribo Files
Version: 1.23.0
Authors@R: c(
    person("Michael", "Geng", email = "michaelgeng@utexas.edu", role = c("cre", "aut")),
    person("Hakan", "Ozadam", email = "hakan.ozadam@gmail.com", role = "aut"),
    person("Can", "Cenik", email = "ccenik@austin.utexas.edu", role = "aut"))
Description: The ribor package provides an R Interface for .ribo files. It provides functionality to read the .ribo file, which is of HDF5 format, and performs common analyses on its contents.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 3.6.0)
biocViews: Software, Infrastructure
Imports: dplyr, ggplot2, hash, methods, rhdf5, rlang, stats, S4Vectors,
        tidyr, tools, yaml
Suggests: testthat, knitr, rmarkdown
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Collate: 'annotation_functions.R' 'check_functions.R'
        'coverage_functions.R' 'ribo_class.R' 'ribo_methods.R'
        'create_ribo.R' 'helper_functions.R' 'info_functions.R'
        'metagene_functions.R' 'region_count_functions.R' 'ribor.R'
        'rnaseq_functions.R'
git_url: https://git.bioconductor.org/packages/ribor
git_branch: devel
git_last_commit: 9decab8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 22:41:44 UTC; biocbuild
Author: Michael Geng [cre, aut],
  Hakan Ozadam [aut],
  Can Cenik [aut]
Maintainer: Michael Geng <michaelgeng@utexas.edu>

# Generated by roxygen2: do not edit by hand

export(Ribo)
export(get_coverage)
export(get_experiments)
export(get_info)
export(get_internal_region_coordinates)
export(get_internal_region_lengths)
export(get_length_distribution)
export(get_metadata)
export(get_metagene)
export(get_original_region_coordinates)
export(get_original_region_lengths)
export(get_reference_names)
export(get_region_coordinates)
export(get_region_counts)
export(get_region_lengths)
export(get_rnaseq)
export(get_tidy_metagene)
export(plot_length_distribution)
export(plot_metagene)
export(plot_region_counts)
export(rename_default)
export(rename_transcripts)
export(set_aliases)
exportClasses(Ribo)
exportMethods(alias_hash)
exportMethods(experiment_info)
exportMethods(experiments)
exportMethods(format_version)
exportMethods(has_metadata)
exportMethods(left_span)
exportMethods(length_max)
exportMethods(length_min)
exportMethods(length_offset)
exportMethods(metagene_radius)
exportMethods(original_hash)
exportMethods(path)
exportMethods(reference)
exportMethods(right_span)
exportMethods(show)
exportMethods(transcript_info)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,metadata)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,desc)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_stack)
importFrom(ggplot2,scale_fill_discrete)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(hash,has.key)
importFrom(hash,hash)
importFrom(hash,is.empty)
importFrom(hash,keys)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(methods,validObject)
importFrom(rhdf5,H5Fopen)
importFrom(rhdf5,h5ls)
importFrom(rhdf5,h5read)
importFrom(rhdf5,h5readAttributes)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(tidyr,gather)
importFrom(tools,file_path_as_absolute)
importFrom(yaml,yaml.load)

[![Build Status](https://travis-ci.com/ribosomeprofiling/RiboR.svg?branch=master)](https://travis-ci.com/ribosomeprofiling/RiboR)
[![DOI](https://zenodo.org/badge/200300903.svg)](https://zenodo.org/badge/latestdoi/200300903)


# RiboR
R interface for .ribo files

## Ribo Ecosystem
The paper associated with this package and its larger ecosystem can be found [here](https://academic.oup.com/bioinformatics/advance-article/doi/10.1093/bioinformatics/btaa028/5701654).

## Installation

RiboR requires R version **3.6** or higher.


### Note For Linux Users

Install dependencies for devtools.
For Ubuntu based distributions, you can use the following command.

`sudo apt-get install libxml2-dev libcurl4-openssl-dev libssl-dev build-essential m4 autoconf -y`

### Install Latest Version From GitHub

1) Install devtools

`install.packages("devtools")`

2) Load the devtools package

`library("devtools")`

3) Install RiboR from github

`install_github("ribosomeprofiling/ribor")`

### Note for Bioconductor Installation

As it currently stands, this package lives on the `devel` branch, and it is not on the current release (Bioconductor 3.10). Please use the alternative installation instructions given above.

RiboR will be available via Bioconductor on the next release cycle (Bioconductor 3.11) which is scheduled for the end of April. 

Once it is available, the download instructions will be as follows.

### Install Latest Version from Bioconductor (Not Yet Available)
`if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")`

`BiocManager::install("ribor")`

## Documentation

[Here](https://ribosomeprofiling.github.io/ribor/ribor.html) is a walk-through of RiboR.

## Citing

[RiboFlow, RiboR and RiboPy: an ecosystem for analyzing ribosome profiling data at read length resolution, H. Ozadam, M. Geng, C. Cenik
Bioinformatics 36 (9), 2929-2931](https://academic.oup.com/bioinformatics/article/36/9/2929/5701654)

```bibtex
@article{ozadam2020riboflow,
  title={RiboFlow, RiboR and RiboPy: an ecosystem for analyzing ribosome profiling data at read length resolution},
  author={Ozadam, Hakan and Geng, Michael and Cenik, Can},
  journal={Bioinformatics},
  volume={36},
  number={9},
  pages={2929--2931},
  year={2020},
  publisher={Oxford University Press}
}
```

#' Retrieves a list of reference names
#'
#' Gets a list of reference names by reading directly from the .ribo file
#'
#' @param ribo.object A 'Ribo' object-=09
#' @return a list of the reference names
#' @importFrom rhdf5 h5read
#' @export
#' @examples
#' #generate a ribo object with transcript nicknames/aliases
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #get the reference names
#' names <- get_reference_names(sample)
get_reference_names <- function(ribo.object) {
    # Retrieves the reference transcript names
    return(h5read(path(ribo.object),
                  name = "reference/reference_names"))
}

get_reference_lengths <- function(ribo.object) {
    # Retrieves the reference transcript lengths
    validObject(ribo.object)
    row.names <- h5read(path(ribo.object),
                        name = "reference/reference_names")
    lengths   <- h5read(path(ribo.object),
                        name = "reference/reference_lengths")
    return(data.frame(transcript = row.names, length = lengths))
}


#' Renames the transcripts
#'
#' The function \code{\link{rename_transcripts}} strives to make the transcript names less
#' cumbersome to write and easier to use.
#'
#' Transcript names found in a .ribo file can often be long and inconvenient to use.
#' As a result, this function allows the user to rename the transcripts.
#'
#' Often times, a short function can be used on the ribo file reference names
#' to split and extract a more convenient name, and a function with a similar input and
#' output to {\code{\link{rename_default}}} can be passed in.
#'
#' However, if there is no simple function that takes the original name and renames it into
#' a unique alias, then the user can provide a character vector of the same length as the number of
#' transcripts in the ribo file. This character vector would provide aliases that match the order
#' of the original reference names returned by the {\code{\link{get_reference_names}}} function.
#'
#' @param ribo a path to the ribo file or a 'Ribo' object
#' @param rename A function that renames the original transcript or an already generated
#' character vector of aliases
#' @importFrom rhdf5 h5read
#' @seealso
#' {\code{\link{rename_default}}} to view expected input and output of a 'rename' function
#' {\code{\link{Ribo}}} to generate a ribo object
#' @examples
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path, rename = rename_default)
#'
#' aliases <- rename_transcripts(sample, rename = rename_default)
#' @export
#' @return A character vector denoting the renamed transcript aliases
rename_transcripts <- function(ribo, rename) {
    #ensure that the ribo path is retrieved
    ribo.path <- ribo
    if (is(ribo, "Ribo")) {
        ribo.path <- path(ribo)
    }

    #handle the function case and the vector case
    simplify <- NULL
    original <- h5read(ribo.path, 
                       name = "reference/reference_names")

    if (is.function(rename)) {
        simplify <- vapply(X = original,
                           FUN = rename,
                           FUN.VALUE = "character")
    } else if (is.vector(rename)) {
        #handle the case that rename is a list as well
        simplify <- unlist(rename)
    } else {
        stop("Param 'rename' must be a function or a vector.")
    }

    if (anyDuplicated(simplify)) {
        stop("Invalid param 'rename'. Redundant values found in the alias.",
             call. = FALSE)
    }

    # helper function to rename the long transcript names
    return(simplify)
}

#' Rename function for appris transcriptome naming convention
#'
#' The function {\code{\link{rename_default}}} is the default renaming function for the
#' appris human transcriptome. It takes one single transcript name and returns a simplified
#' alias.
#' @examples
#' original <- paste("ENST00000613283.2|ENSG00000136997.17|",
#'                   "OTTHUMG00000128475.8|-|MYC-206|MYC|1365|protein_coding|",
#'                   sep = "")
#' alias <- rename_default(original)
#' @param x Character denoting original name of the transcript
#' @return Character denoting simplified name of the object
#' @export
rename_default <- function(x) {
    return(unlist(strsplit(x, split = "|", fixed = TRUE))[5])
}

get_content_info <- function(ribo.path) {
    file_info     <- h5ls(ribo.path, recursive = TRUE, all = FALSE)
    experiment   <- file_info[file_info$group == "/experiments", ]$name
    length <- length(experiment)

    #creates the separate lists for reads, coverage, rna.seq, and metadata
    #to eventually put in a data frame
    reads.list    <- vector(mode = "integer", length = length)
    coverage.list <- vector(mode = "logical", length = length)
    rna.seq.list  <- vector(mode = "logical", length = length)
    metadata.list <- vector(mode = "logical", length = length)

    #ls function provides information about the contents of each experiment
    ls <- h5ls(ribo.path)

    #loop over all of the experiments
    for (i in seq(length)) {
        exp <- experiment[i]
        #gathers information on the number of reads for each experiment by looking at
        #the attributes
        name           <-
            paste("/experiments/", exp, sep = "")
        attribute      <- h5readAttributes(ribo.path, name)
        reads.list[i]     <- attribute[["total_reads"]]

        #creates separate logical lists to denote the presence of
        #reads, coverage, RNA-seq, metadata
        metadata.list[i]  <- ("metadata" %in% names(attribute))

        group.contents <- ls[ls$group == name,]
        group.names    <- group.contents$name

        coverage.list[i]  <- ("coverage" %in% group.names)
        rna.seq.list[i]   <- ("rnaseq" %in% group.names)
    }

    experiments.info       <- data.frame(
        experiment  = experiment,
        total.reads = reads.list,
        coverage    = coverage.list,
        rna.seq     = rna.seq.list,
        metadata    = metadata.list,
        stringsAsFactors = FALSE
    )
    return(experiments.info)
}


get_attributes <- function(ribo.object) {
    # Retrieves the attributes of the ribo.object
    path  <- path(ribo.object)
    attribute <- h5readAttributes(path, "/")
    return(attribute[-which(names(attribute) == "time")])
}

get_read_lengths <- function(ribo.object) {
    # Retrieves the minimum and maximum read lengths
    #
    # get_read_lengths finds the minimum and maximum read lengths of the .ribo file
    attributes <- get_attributes(ribo.object)
    result <- c(attributes$length_min, attributes$length_max)
    return(result)
}



make_dataframe <- function(ribo.object,
                           matched.list,
                           range.info,
                           conditions,
                           matrix) {
    alias      <- conditions[["alias"]]
    ref.names <- get_reference_names(ribo.object)
    # helper function that creates a polished dataframe out of a filled in matrix
    if (alias) {
        original <- ref.names
        ref.names <- vector(mode = "character", length = length(original))
        for (i in seq(length(ref.names))) {
            ref.names[i] <- original_hash(ribo.object)[[original[[i]]]]
        }
    }
    return(help_make_dataframe(ref.names,
                               conditions,
                               matched.list,
                               range.info,
                               matrix))
}

help_make_dataframe <- function(ref.names,
                                conditions,
                                matched.list,
                                range.info,
                                matrix) {

    # helper that generates data frame of the correct size
    # the helper method is used for both the get_region_counts
    # and the get_metagene functions,
    range.lower <- range.info['range.lower']
    range.upper <- range.info['range.upper']

    transcript  <- conditions[["transcript"]]
    length      <- conditions[["length"]]

    ref.length  <- length(ref.names)
    num.reads   <- range.upper - range.lower + 1
    total.list  <- length(matched.list)
    
    # get the matrix data and pass create a data frame of the 
    # correct format
    if (transcript & length) {
        experiment.list   <- matched.list
        return (data.frame(experiment = matched.list,
                           matrix,
                           stringsAsFactors = FALSE,
                           check.names = FALSE))
    } else if (transcript) {
        #sum transcripts only
        experiment.column <- rep(matched.list, each = num.reads)
        read.column <- rep(c(range.lower:range.upper), total.list)
        return (data.frame(experiment = experiment.column,
                           length     = read.column,
                           matrix, stringsAsFactors = FALSE,
                           check.names = FALSE))
    } else if (length) {
        #length only
        experiment.column <- rep(matched.list, each = ref.length)
        transcript.column <- rep(ref.names, total.list)
        return (data.frame(experiment = experiment.column,
                           transcript = transcript.column,
                           matrix, stringsAsFactors = FALSE,
                           check.names = FALSE))
    }
    #!transcript and !length
    experiment.column <- rep(matched.list, each = num.reads * ref.length)
    transcripts       <- rep(ref.names, total.list * num.reads)
    ref.read          <- rep(c(range.lower:range.upper), each = ref.length)
    read.column <- rep(ref.read, total.list)
    return (data.frame(experiment  = experiment.column,
                       transcript  = transcripts,
                       length      = read.column,
                       matrix, stringsAsFactors = FALSE,
                       check.names = FALSE))
}


generate_matrix <- function(ribo.object,
                            transcript,
                            length,
                            normalize,
                            ncol,
                            file,
                            path,
                            index){
    #helper method that generates the matrix under different circumstances
    experiment <- strsplit(path, split="/")[[1]][3]
    output <- t(h5read(file=file, index=index, name=path))

    ref.length <- length(get_reference_names(ribo.object))
    single <- (ref.length == nrow(output))

    # handle the different cases of the user's passed in parameters for
    # transcript and length 
    if (transcript & length) {
        output <- matrix(unlist(colSums(output)), ncol=ncol, byrow=TRUE)
    } else if (transcript) {
        # condense all transcripts together
        condense_transcripts <- seq(from = 1 ,
                                    to = nrow(output),
                                    by = ref.length)
        
        output <- lapply(condense_transcripts, sum_lengths, 
                                               ref.length = ref.length,
                                               mat        = output)

        output <- matrix(unlist(output), ncol=ncol, byrow=TRUE)
   } else if (length) {
        #only sum across lengths if there is more than one length 
        if (!single) {
            val <- seq(ref.length)
            condense_lengths <- lapply(val,
                                       seq,
                                       to = nrow(output),
                                       by = ref.length)
            output <- lapply(condense_lengths, sum_transcripts, mat=output)
            output <- matrix(unlist(output), ncol=ncol, byrow=TRUE)
        }
  }
  return(output)
}

sum_transcripts <- function (index, mat) {
  #in the case that there is only one region
  if (ncol(mat) == 1) {
    return(sum(mat[index, ]))
  }
  return (colSums(mat[index, ]))
}

sum_lengths <- function(index, ref.length, mat) {
  #in the case that there is only one region 
  if (ncol(mat) == 1) {
    return(sum(mat[index:(index+ref.length-1), ]))
  }
  return (colSums(mat[index:(index+ref.length-1), ]))
}


prepare_DataFrame <- function(ribo.object, DF) {
  # Helper method that creates factors and Rle the columns of the 
  # metagene and region_count functions
  DF <- as(DF, "DataFrame")
  if (!is.null(DF$region)) DF$region <- Rle(factor(DF$region))
  if (!is.null(DF$transcript)) DF$transcript <- factor(DF$transcript)
  if (!is.null(DF$position)) DF$position <- Rle(DF$position)
  DF$experiment = Rle(factor(DF$experiment))
  
  # add the metadata
  metadata(DF)[[1]] <- get_info(ribo.object)$experiment.info[, c("experiment", 
                                                                "total.reads")]
  return(DF)
}

strip_rlefactor <- function(DF) {
  # Helper method that strips the factors and prepares the DataFrame for plotting
  if (!is.null(DF$region)) DF$region <- as.character(DF$region)
  if (!is.null(DF$transcript)) DF$transcript <- as.character(DF$transcript)
  if (!is.null(DF$position)) DF$position <- as.integer(DF$position)
  if (!is.null(DF$length)) DF$length <- as.integer(DF$length)
  DF$experiment <- as.character(DF$experiment)
  return(DF)
}

#' Returns the overall length of each region with UTR Junctions 
#' 
#' The function {\code{\link{get_internal_region_coordinates}}} retrieves the 
#' lengths for the UTR5, UTR5 Junction, CDS, UTR3 Junction,
#' and UTR3 regions of every transcript. 
#' 
#' @examples 
#' # generate a ribo object 
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path, rename = rename_default)
#' 
#' # get the region lengths
#' region_lengths <- get_internal_region_lengths(sample, alias = TRUE)
#' 
#' @export
#' @importFrom rhdf5 h5read
#' @param ribo.object A 'Ribo' object 
#' @param alias Option to return the transcript names as aliases  
#' @return A data.frame of the region lengths 
get_internal_region_lengths <- function(ribo.object, alias = FALSE) {
    validObject(ribo.object)
    check_alias(ribo.object, alias)
    
    #generate the start and stop indices, get the matrix of positions
    start <- seq(1, 10, by = 2)
    stop  <- seq(2, 10, by = 2)
    result <- compute_boundaries(ribo.object)
    
    #in the case of NA, report the length as 0 
    result[, c(start)][is.na(result[, c(start)])] <- 0
    result[, c(stop)][is.na(result[, c(stop)])] <- -1
    
    #generate the lengths 
    diff <- lapply(start, function(c, result) result[, c+1] - result[, c] + 1, result)
    diff <- matrix(unlist(diff), byrow = FALSE, nrow = nrow(result))
    colnames(diff) <- c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3")
    
    #generate the data.frame 
    references <- change_reference_names(ribo.object, alias)
    return(data.frame(transcript=references, diff))
}


#' Returns the overall length of each region with UTR Junctions 
#' 
#' The function {\code{\link{get_region_lengths}}} retrieves the 
#' lengths for the UTR5, UTR5 Junction, CDS, UTR3 Junction,
#' and UTR3 regions of every transcript. 
#' 
#' This function is deprecated, and we recommend {\code{\link{get_internal_region_lengths}}}.
#' 
#' @export
#' @importFrom rhdf5 h5read
#' @param ribo.object A 'Ribo' object 
#' @param alias Option to return the transcript names as aliases  
#' @return A data.frame of the region lengths 
get_region_lengths <- function(ribo.object, alias = FALSE) {
    .Deprecated("get_internal_region_lengths")
    return(get_internal_region_lengths(ribo.object = ribo.object, alias = alias))
}



#' Returns the overall length of each region
#' 
#' The function {\code{\link{get_original_region_coordinates}}} retrieves the 
#' lengths for the UTR5, CDS, and UTR3 regions of every transcript. 
#' 
#' @examples 
#' # generate a ribo object 
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path, rename = rename_default)
#' 
#' # get the region coordinates
#' region_lengths <- get_original_region_lengths(sample, alias = TRUE)
#' 
#' @export
#' @importFrom rhdf5 h5read
#' @param ribo.object A 'Ribo' object 
#' @param alias Option to return the transcript names as aliases  
#' @return A data.frame of the region lengths 
get_original_region_lengths <- function(ribo.object, alias = FALSE) {
    validObject(ribo.object)
    check_alias(ribo.object, alias)
    
    #generate the start and stop indices, get the matrix of positions
    start <- seq(1, 6, by = 2)
    stop  <- seq(2, 6, by = 2)
    result <- compute_original_boundaries(ribo.object)
    
    #in the case of NA, report the length as 0 
    result[, c(start)][is.na(result[, c(start)])] <- 0
    result[, c(stop)][is.na(result[, c(stop)])] <- -1
    
    #generate the lengths based on the differences in coordinate position 
    diff <- lapply(start, function(c, result) result[, c+1] - result[, c] + 1, result)
    diff <- matrix(unlist(diff), byrow = FALSE, nrow = nrow(result))
    colnames(diff) <- c("UTR5", "CDS", "UTR3")
    
    #generate the data.frame 
    references <- get_reference_names(ribo.object)
    return(data.frame(transcript=references, diff))
}


#' Retrieves the region stop and start coordinates 
#' 
#' The function {\code{\link{get_internal_region_coordinates}}} retrieves the 
#' start and site positions for the UTR5, UTR5 Junction, CDS, UTR3 Junction,
#' and UTR3 regions of every transcript. 
#' 
#' To note, because of the R-specific 1-based indexing, the positions start at
#' 1 instead of 0 in other programming languages. The positions provided in 
#' the returned data.frame will correspond to the positions in the output of 
#' {\code{\link{get_coverage}}}.
#' 
#' Additionally, within the transcripts, there are edge cases. 
#' NA values found in the returned data.frame means that the region has no 
#' start and stop position and a length of zero after computing the boundaries
#' of the UTR5 and UTR3 junction.
#' 
#' @export 
#' @examples 
#' # generate a ribo object 
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path, rename = rename_default)
#' 
#' # get the region coordinates
#' coord <- get_internal_region_coordinates(sample, alias = TRUE)
#' 
#' @inheritParams get_region_lengths
#' @return A data.frame of start and stop coordinates for every region
get_internal_region_coordinates <- function(ribo.object, alias=FALSE) {
    validObject(ribo.object)
    check_alias(ribo.object, alias)
    references <- change_reference_names(ribo.object, alias)
    #generate the data.frame from the boundary positions
    return(data.frame(transcript=references, compute_boundaries(ribo.object)))
}


#' Retrieves the region stop and start coordinates 
#' 
#' The function {\code{\link{get_region_coordinates}}} retrieves the 
#' start and site positions for the UTR5, UTR5 Junction, CDS, UTR3 Junction,
#' and UTR3 regions of every transcript. 
#' 
#' To note, because of the R-specific 1-based indexing, the positions start at
#' 1 instead of 0 in other programming languages. The positions provided in 
#' the returned data.frame will correspond to the positions in the output of 
#' {\code{\link{get_coverage}}}.
#' 
#' Additionally, within the transcripts, there are edge cases. 
#' NA values found in the returned data.frame means that the region has no 
#' start and stop position and a length of zero after computing the boundaries
#' of the UTR5 and UTR3 junction.
#' 
#' @export 
#' 
#' @inheritParams get_region_lengths
#' @return A data.frame of start and stop coordinates for every region
get_region_coordinates <- function(ribo.object, alias=FALSE) {
    .Deprecated("get_internal_region_coordinates")
    return(get_internal_region_coordinates(ribo.object = ribo.object, alias = alias))
}




#' Retrieves the region stop and start coordinates 
#' 
#' The function {\code{\link{get_original_region_coordinates}}} retrieves the 
#' start and site positions for the UTR5, UTR5 Junction, CDS, UTR3 Junction,
#' and UTR3 regions of every transcript. 
#' 
#' To note, because of the R-specific 1-based indexing, the positions start at
#' 1 instead of 0 in other programming languages. The positions provided in 
#' the returned data.frame will correspond to the positions in the output of 
#' {\code{\link{get_coverage}}}.
#' 
#' Additionally, within the transcripts, there are edge cases. 
#' NA values found in the returned data.frame means that the region has no 
#' start and stop position and a length of zero after computing the boundaries
#' of the UTR5 and UTR3 junction.
#' 
#' @export 
#' @examples 
#' # generate a ribo object 
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path, rename = rename_default)
#' 
#' # get the region coordinates
#' coord <- get_original_region_coordinates(sample, alias = TRUE)
#' 
#' @inheritParams get_original_region_lengths
#' @return A data.frame of start and stop coordinates for every region
get_original_region_coordinates <- function(ribo.object, alias=FALSE) {
    validObject(ribo.object)
    check_alias(ribo.object, alias)
    references <- change_reference_names(ribo.object, alias)
    #generate the data.frame from the boundary positions
    return(data.frame(transcript=references, compute_original_boundaries(ribo.object)))
}




compute_boundaries <- function(ribo.object) {
    #helper method that computes the boundaries for each region
    annotation <- t(h5read(path(ribo.object),
                           name = "/reference/annotation"))
    left.span <- left_span(ribo.object)
    right.span <- right_span(ribo.object)
    UTR5_end <- annotation[, 1]
    CDS_end <-  annotation[, 2]
    UTR3_end <- annotation[, 3]
    
    #compute the junction start and stop
    UTR5J_start <- UTR5_end + 1 - left.span 
    UTR5J_start[UTR5J_start <= 0] <- 1 
    UTR5J_stop <- UTR5_end + right.span
    UTR5J_stop[UTR5J_stop > CDS_end] <- CDS_end[UTR5J_stop > CDS_end]
    UTR3J_start <- CDS_end + 1 - left.span
    UTR3J_start[UTR3J_start <= UTR5J_stop] <- 
        UTR5J_stop[UTR3J_start <= UTR5J_stop] + 1
    UTR3J_stop <- CDS_end + right.span
    UTR3J_stop[UTR3J_stop > UTR3_end] <- UTR3_end[UTR3J_stop > UTR3_end]
    UTR3J_start[UTR3J_start > UTR3J_stop] <- NA
    UTR3J_stop[UTR3J_start > UTR3J_stop] <- NA
    
    #compute the CDS start and stop
    CDS_start <- UTR5J_stop + 1 
    CDS_stop <- UTR3J_start - 1
    CDS_start[CDS_start > CDS_stop] <- NA
    CDS_stop[CDS_start > CDS_stop] <- NA
    
    #compute the UTR5 start and stop
    UTR5_start  <- rep(1, times = length(get_reference_names(ribo.object)))
    UTR5_stop   <- UTR5J_start - 1
    UTR5_start[UTR5_stop == 0] <- NA
    UTR5_stop[UTR5_stop == 0] <- NA
    
    #compute the UTR3 start and stop
    UTR3_start <- UTR3J_stop + 1 
    UTR3_stop  <- UTR3_end
    UTR3_start[UTR3_start > UTR3_stop] <- NA
    UTR3_stop[UTR3_start > UTR3_stop] <- NA
    info <- list(UTR5_start, UTR5_stop, UTR5J_start, UTR5J_stop,
                 CDS_start, CDS_stop, UTR3J_start, UTR3J_stop,
                 UTR3_start, UTR3_stop)
    result <- do.call(cbind, info)
    colnames(result) <- c("UTR5_start", "UTR5_stop", "UTR5J_start", 
                          "UTR5J_stop", "CDS_start", "CDS_stop", "UTR3J_start",
                          "UTR3J_stop", "UTR3_start", "UTR3_stop")
    return(result)
}



compute_original_boundaries <- function(ribo.object) {
    #helper method that computes the boundaries for each region
    annotation <- t(h5read(path(ribo.object),
                           name = "/reference/annotation"))
    UTR5_end <- annotation[, 1]
    CDS_end <-  annotation[, 2]
    UTR3_end <- annotation[, 3]
    
    #compute the CDS start and stop
    CDS_start <- UTR5_end + 1 
    CDS_stop <- CDS_end
    CDS_start[CDS_start > CDS_stop] <- NA
    CDS_stop[CDS_start > CDS_stop] <- NA
    
    #compute the UTR5 start and stop
    UTR5_start  <- rep(1, times = length(get_reference_names(ribo.object)))
    UTR5_stop   <- UTR5_end
    UTR5_start[UTR5_stop == 0] <- NA
    UTR5_stop[UTR5_stop == 0] <- NA
    
    #compute the UTR3 start and stop
    UTR3_start <- CDS_end + 1 
    UTR3_stop  <- UTR3_end
    UTR3_start[UTR3_start > UTR3_stop] <- NA
    UTR3_stop[UTR3_start > UTR3_stop] <- NA
    info <- list(UTR5_start, UTR5_stop,
                 CDS_start, CDS_stop, 
                 UTR3_start, UTR3_stop)
    result <- do.call(cbind, info)
    colnames(result) <- c("UTR5_start", "UTR5_stop",
                          "CDS_start", "CDS_stop",
                          "UTR3_start", "UTR3_stop")
    return(result)
}

context("annotation functions")
library(ribor)

file.path <- system.file("extdata", "sample.ribo", package = "ribor")
ribo.object <- Ribo(file.path)


# test the internal region lengths
expect_warning(lengths <- get_region_lengths(ribo.object))

UTR5 <- c(2, 1, 0)
UTR5J <- c(5, 5, 5)
CDS <- c(5, 7, 2)
UTR3J <- c(5, 5, 5)
UTR3 <- c(3,4,5)


actual <- unname(unlist(lengths[, "UTR5"]))
expected <- UTR5
test_that("get_original_region_lengths, UTR5",
          expect_equal(actual, expected))

actual <- unname(unlist(lengths[, "UTR5J"]))
expected <- UTR5J
test_that("get_original_region_lengths, UTR5J",
          expect_equal(actual, expected))

actual <- unname(unlist(lengths[, "CDS"]))
expected <- CDS
test_that("get_original_region_lengths, CDS",
          expect_equal(actual, expected))

actual <- unname(unlist(lengths[, "UTR3J"]))
expected <- UTR3J
test_that("get_original_region_lengths, UTR3J",
          expect_equal(actual, expected))

actual <- unname(unlist(lengths[, "UTR3"]))
expected <- UTR3
test_that("get_original_region_lengths, UTR3",
          expect_equal(actual, expected))


# test the original region lengths
lengths <- get_original_region_lengths(ribo.object)

UTR5 <- c(5, 4, 3)
CDS <- c(10, 12, 7)
UTR3 <- c(5,6,7)

actual <- unname(unlist(lengths[, "UTR5"]))
expected <- UTR5
test_that("get_region_lengths, UTR5",
          expect_equal(actual, expected))


actual <- unname(unlist(lengths[, "CDS"]))
expected <- CDS
test_that("get_region_lengths, CDS",
          expect_equal(actual, expected))


actual <- unname(unlist(lengths[, "UTR3"]))
expected <- UTR3
test_that("get_region_lengths, UTR3",
          expect_equal(actual, expected))

# test the internal region coordinates

UTR5_start <- c(1, 1, NA)
UTR5_stop <- c(2, 1, NA)
UTR5J_start <- c(3, 2, 1)
UTR5J_stop <- c(7, 6, 5)
CDS_start <- c(8, 7, 6)
CDS_stop <- c(12, 13, 7)
UTR3J_start <- c(13, 14, 8)
UTR3J_stop <- c(17, 18, 12)
UTR3_start <- c(18,19,13)
UTR3_stop <- c(20, 22, 17)


expect_warning(coordinates <- get_region_coordinates(ribo.object))

actual <- unname(unlist(coordinates[, c("UTR5_start", "UTR5_stop")]))
expected <- c(UTR5_start, UTR5_stop)
test_that("get_region_coordinates, UTR5", expect_equal(actual, expected))

actual <- unname(unlist(coordinates[, c("UTR5J_start", "UTR5J_stop")]))
expected <- c(UTR5J_start, UTR5J_stop)
test_that("get_region_coordinates, UTR5J", expect_equal(actual, expected))

actual <- unname(unlist(coordinates[, c("CDS_start", "CDS_stop")]))
expected <- c(CDS_start, CDS_stop)
test_that("get_region_coordinates, CDS", expect_equal(actual, expected))

actual <- unname(unlist(coordinates[, c("UTR3J_start", "UTR3J_stop")]))
expected <- c(UTR3J_start, UTR3J_stop)
test_that("get_region_coordinates, UTR3J", expect_equal(actual, expected))

actual <- unname(unlist(coordinates[, c("UTR3_start", "UTR3_stop")]))
expected <- c(UTR3_start, UTR3_stop)
test_that("get_region_coordinates, UTR3", expect_equal(actual, expected))


# test the original region coordinates

UTR5_start <- c(1, 1, 1)
UTR5_stop <- c(5, 4, 3)
CDS_start <- c(6, 5, 4)
CDS_stop <- c(15, 16, 10)
UTR3_start <- c(16,17,11)
UTR3_stop <- c(20, 22, 17)

coordinates <- get_original_region_coordinates(ribo.object)

actual <- unname(unlist(coordinates[, c("UTR5_start", "UTR5_stop")]))
expected <- c(UTR5_start, UTR5_stop)
test_that("get_original_region_coordinates, UTR5", expect_equal(actual, expected))


actual <- unname(unlist(coordinates[, c("CDS_start", "CDS_stop")]))
expected <- c(CDS_start, CDS_stop)
test_that("get_original_region_coordinates, CDS", expect_equal(actual, expected))


actual <- unname(unlist(coordinates[, c("UTR3_start", "UTR3_stop")]))
expected <- c(UTR3_start, UTR3_stop)
test_that("get_original_region_coordinates, UTR3", expect_equal(actual, expected))

#' Retrieves the region counts from a .ribo file
#'
#' \code{\link{get_region_counts}} will return the particular region counts
#' of any subset of regions for a given set of experiments.
#'
#' This function will return a data frane of the counts at each specified region
#' for each specified experiment. The region options are "UTR5", "UTR5J", "CDS",
#' "UTR3J", and "UTR3". The user can specify any subset of regions in the form of a vector,
#' a list, or a single string if only one region is desired.
#'
#' The dimensions of the returned DataFrame depend on the parameters
#' range.lower, range.upper, length, and transcript.
#'
#' The param 'length' condenses the read lengths together.
#' When length is TRUE and transcript is FALSE, the
#' data frame presents information for each transcript across
#' all of the read lengths. That is, each transcript has a value
#' that is the sum of all of the counts across every read length.
#' As a result, information about the transcript at each specific
#' read length is lost.
#'
#' The param 'transcript' condenses the transcripts together.
#' When transcript is TRUE and length is FALSE data
#' frame presents information at each read length between range.lower and
#' range.upper inclusive. That is, each separate read length denotes the
#' sum of counts from every transcript. As a result, information about the
#' counts of each individual transcript is lost.
#'
#' When 'transcript' is set to FALSE, the 'alias' parameter specifies whether
#' or not the returned DataFrame should present each transcript as an alias
#' instead of the original name. If 'alias' is set to TRUE, then the column
#' of the transcript names will contain the aliases rather than the original
#' reference names of the .ribo file.
#'
#' If both 'length' and 'transcript' are TRUE, then the resulting
#' DataFrame prints out one row for each experiment. This provides the metagene
#' information across all transcripts and all reads in a given experiment.
#'
#' If both length' and 'transcript' are FALSE, calculations are done to the data,
#' all information is preserved for both the read length and the transcript.
#' The DataFrame would just present the entire stored raw data
#' from the read length 'range.lower' to the read length 'range.upper' which in most
#' cases would result in a slow run time with a massive DataFrame returned.
#'
#' When 'transcript' is set to FALSE, the 'alias' parameter specifies whether
#' or not the returned DataFrame should present each transcript as an alias
#' instead of the original name. If 'alias' is set to TRUE, then the column
#' of the transcript names will contain the aliases rather than the original
#' reference names of the .ribo file.
#' @examples
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #specify the regions and experiments of interest
#' regions <- c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3")
#' experiments <- c("Hela_1", "Hela_2", "WT_1")
#'
#' #obtains the region counts at each individual read length, summed across every transcript
#' region.counts <- get_region_counts(ribo.object = sample,
#'                                    region = regions,
#'                                    range.lower = 2,
#'                                    range.upper = 5,
#'                                    length = FALSE,
#'                                    transcript = TRUE,
#'                                    tidy = FALSE,
#'                                    alias = FALSE,
#'                                    experiment = experiments)
#'
#' @param ribo.object A 'Ribo' object
#' @param range.lower Lower bound of the read length, inclusive
#' @param range.upper Upper bound of the read length, inclusive
#' @param transcript Logical value that denotes if the region count information should be summed across transcripts
#' @param length Logical value that denotes if the region count information should be summed across read lengths
#' @param experiment List of experiment names
#' @param alias Option to report the transcripts as aliases/nicknames
#' @param compact Option to return a DataFrame with Rle and factor as opposed to a raw data.frame
#' @param tidy Option to return the data frame in a tidy format
#' @param region Specific region of interest
#' @param normalize Option to normalize the counts as counts per million reads
#' @param compact Option to return a DataFrame with Rle and factor as opposed to a raw data.frame
#' @return An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
#' region count information for the regions specified in the 'region' parameter. The returned data frame
#' will have a length column when the 'length' parameter is set to FALSE, indicating that the count
#' information will not be summed across the provided range of read lengths. Similarly, the returned data
#' frame will have a transcript column when the 'transcript' parameter is set to FALSE, indicating that the
#' count information will not be summed across the transcripts. In the case that transcript parameter is
#' 'FALSE', the returned data frame will present the transcripts according to the aliases specified at the
#' creation of the ribo object if the 'alias' parameter is set to TRUE.
#' @importFrom rhdf5 h5read
#' @importFrom methods as 
#' @importFrom S4Vectors DataFrame Rle
#' @importFrom tidyr gather
#' @importFrom dplyr left_join
#' @export
get_region_counts <- function(ribo.object,
                              range.lower = length_min(ribo.object),
                              range.upper = length_max(ribo.object),
                              length = TRUE,
                              transcript = TRUE,
                              tidy = TRUE,
                              alias = FALSE,
                              normalize = FALSE,
                              region = c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3"),
                              compact = TRUE, 
                              experiment = experiments(ribo.object)) {
    if (!is(ribo.object, "Ribo")) stop("Please provide a ribo object.")
    range.info <- c(range.lower = range.lower, range.upper = range.upper)
    region     <- check_rc_input(ribo.object, region, range.info, experiment, alias)
    conditions <- c(transcript = transcript, length = length,
                    normalize  = normalize,   alias = alias)

    path                <- path(ribo.object)
    matched.experiments <- intersect(experiment, experiments(ribo.object))
    total.experiments   <- length(matched.experiments)
    ref.names           <- get_reference_names(ribo.object)
    ref.length          <- length(ref.names)

    values    <- c("UTR5"  = 1, "UTR5J" = 2, "CDS" = 3, "UTR3J" = 4, "UTR3" = 5)
    columns   <- unname(values[region])
    ncol      <- length(region)
    range.min <- length_min(ribo.object)
    row.start <- (range.lower - range.min) * ref.length + 1
    row.stop  <- row.start + ref.length* (range.upper - range.lower + 1) - 1
    rows      <- c(row.start:row.stop)
    matched.experiments <- intersect(experiment, experiments(ribo.object))
    
    # compute the file paths
    file_paths <- vapply(matched.experiments, get_rc_path, FUN.VALUE = "character")
    data  <- lapply(file_paths, generate_matrix, ribo.object = ribo.object,
                                                 transcript  = transcript,
                                                 length      = length,
                                                 normalize   = normalize,
                                                 file        = path,
                                                 index       = list(columns, rows),
                                                 ncol        = ncol)
    data           <- do.call(rbind, data)
    colnames(data) <- region
    
    # make_dataframe is a generic helper that returns a DataFrame so need to cast
    result         <- as.data.frame(make_dataframe(ribo.object, matched.experiments,
                                                   range.info, conditions, data))
    
    if (normalize) {
        # normalize the counts to reads/total reads * 1000000
        info <- get_info(ribo.object)$experiment.info[, c("experiment", "total.reads")]
        result <- left_join(result, info, by = "experiment")
        result[, region] <- result[, region] * 1000000/result$total.reads
        result[, "total.reads"] <- NULL
    }
    
    if (tidy) result <- gather(result, "region", "count", region)
    
    if (compact) {
      result <- as(result, "DataFrame")
      return(prepare_DataFrame(ribo.object, result))
    } 
    return (result)
}


get_rc_path <- function(experiment) {
    return(paste("/experiments/",
                 experiment,
                 "/region_counts/region_counts",
                 sep = ""))
}

check_rc_input <- function(ribo.object,
                           region,
                           range.info,
                           experiment,
                           alias) {
    #helper function that checks for valid parameters given by the user
    #calls error messages on any incorrect parameters
    region <- check_regions(ribo.object, region)

    range.lower <- range.info[["range.lower"]]
    range.upper <- range.info[["range.upper"]]

    check_alias(ribo.object, alias)
    check_lengths(ribo.object, range.lower, range.upper)
    check_experiments(ribo.object, experiment)
    return(region)
}

check_regions <- function(ribo.object,
                          region) {
    region <- toupper(region)
    region.options <- c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3")

    if (typeof(region) != 'list' & typeof(region) != "character") {
        stop("Please specify the regions as a single string, a vector, or a list.",
             call. = FALSE)
    }
    
    if (length(which(!region %in% region.options))) {
        stop("Please indicate the region(s) with ", 
             "the following: 'UTR5', 'UTR5J, 'CDS', 'UTR3J', 'UTR3'",
             call. = FALSE)
    }
    
    region <- factor(region, 
                     levels  = c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3"), 
                     ordered = TRUE)
    region <- as.vector(sort(region))
    return(region)
}


#' Retrieves the length distribution of a given region
#'
#' The function {\code{\link{get_length_distribution}}} retrieves the raw or normalized
#' counts at each read length from 'range.lower' to 'range.upper'.
#'
#' This function is a wrapper function of {\code{\link{get_region_counts}}}, and the
#' returned DataFrame is valid input for {\code{\link{plot_length_distribution}}}.
#'
#' @examples
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #specify the experiments of interest
#' experiments <- c("Hela_1", "Hela_2", "WT_1")
#'
#' #gets the normalized length distribution from read length 2 to 5
#' length.dist <- get_length_distribution(ribo.object = sample,
#'                                        region = "CDS",
#'                                        range.lower = 2,
#'                                        range.upper = 5)
#'
#' @param ribo.object A 'Ribo' object
#' @param range.lower Lower bound of the read length, inclusive
#' @param range.upper Upper bound of the read length, inclusive
#' @param experiment List of experiment names
#' @param compact Option to return a DataFrame with Rle and factor as opposed to a raw data.frame
#' @param region Specific region of interest
#' @importFrom methods as
#' @importFrom S4Vectors DataFrame Rle
#' @export
#' @seealso
#' {\code{\link{plot_length_distribution}}} to plot the output of this function
#' @return An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
#' read-length specific region count information for a single region specified in the 'region' parameter.
#' The returned data frame will have a length column, and it will not contain a transcript column.
get_length_distribution <- function(ribo.object,
                                    region,
                                    range.lower = length_min(ribo.object),
                                    range.upper = length_max(ribo.object),
                                    compact = TRUE,
                                    experiment = experiments(ribo.object)) {
  if (length(region) != 1) {
    stop("Please provide only one region.")
  }
  if (!is(ribo.object, "Ribo")) stop("Please provide a ribo object.")
  result <- get_region_counts(ribo.object,
                              range.lower = range.lower,
                              range.upper = range.upper,
                              region = region,
                              length = FALSE,
                              transcript = TRUE,
                              normalize = FALSE,
                              compact = compact,
                              experiment = experiment)[, -3]
  
  # by not using DataFrame, we will need to add total.reads back
  if (!compact) {
    result %>% 
      left_join(get_info(ribo.object)$experiment.info[, c("experiment", "total.reads")],
                by = "experiment") -> result
  }
  return (result)
}


#' Plots the length distribution
#'
#' The function \code{\link{plot_length_distribution}} can take either a DataFrame
#' or a "Ribo" object to generate a line graph of the length distributions from
#' range.lower to range.upper.
#'
#' The param 'fraction' will plot the fractions of each length relative
#' to the total sum of the read length range provided by param 'range.lower'
#' and 'range.upper'. When fraction is set to FALSE, the total count of each
#' read length is plotted.
#'
#' When given a "Ribo" object, \code{\link{plot_length_distribution}} calls
#' \code{\link{get_region_counts}} to retrieve the necessary information
#' for plotting.
#'
#' The user can instead provide a DataFrame with the same structure as the
#' output of the \code{\link{get_region_counts}} function where the 'transcript'
#' parameter is set to FALSE and 'length' parameters is the default value of
#' TRUE. This also means that the many of the remaining parameters of the
#' \code{\link{plot_length_distribution}} function are not necessary. The run
#' time becomes substantially faster when \code{\link{plot_region_counts}} is
#' given the direct DataFrame to plot. Note that there is no manipulation by
#' this function on the DataFrame. This responsibility is given to the user
#' and allows for more control.
#'
#' @examples
#' #ribo object use case
#'
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #specify experiments of interest
#' experiments <- c("Hela_1", "Hela_2", "WT_1")
#'
#' plot_length_distribution(x = sample,
#'                          region = "CDS",
#'                          range.lower = 2,
#'                          range.upper = 5,
#'                          experiment = experiments,
#'                          fraction = TRUE)
#'
#'
#' #DataFrame use case
#' #obtains the region counts at each individual read length, summed across every transcript
#' region.counts <- get_length_distribution(ribo.object = sample,
#'                                          region      = "CDS",
#'                                          range.lower = 2,
#'                                          range.upper = 5,
#'                                          experiment  = experiments)
#'
#' #the param 'length' must be set to FALSE and param 'transcript' must be set
#' #to TRUE to use a DataFrame
#' plot_length_distribution(region.counts)
#'
#'
#' @seealso \code{\link{get_region_counts}} to generate a DataFrame that can
#' be provided as input,
#' \code{\link{Ribo}} to create a ribo.object that can be provided as input
#' @param x A 'Ribo' object or a DataFrame generated from \code{\link{get_region_counts}}
#' @param region the region of interest
#' @param range.lower a lower bounds for a read length range
#' @param range.upper an upper bounds for a read length range
#' @param experiment a list of experiment names
#' @param fraction logical value that, if TRUE, presents the count as a fraction of the total reads in the given ranges
#' @param title a title for the generated plot
#' @importFrom tidyr gather
#' @importFrom dplyr select
#' @importFrom ggplot2 ggplot geom_line theme_bw theme labs aes_string
#' @importFrom stats aggregate
#' @importFrom rlang .data
#' @export
#' @return A 'ggplot' of the length distribution
plot_length_distribution <- function(x,
                                     region,
                                     experiment,
                                     range.lower,
                                     range.upper,
                                     fraction = FALSE,
                                     title = "Length Distribution") {
    x <- check_ld_input(x, region, range.lower, range.upper, experiment)
    y.axis <- "Count"
    y.value <- "count"
    
    if (fraction) {
        if (is(x, "DataFrame")) {
          info <- metadata(x)[[1]]
          x %>% 
            strip_rlefactor() %>% 
            as.data.frame() %>% 
            left_join(info, by = "experiment") -> x
        } else {
          x %>% 
            left_join(get_info(x)$experiment.info[, c("experiment", "total.reads")],
                      by = "experiment") 
        }
        
        x %>% 
          mutate(fraction = .data$count/.data$total.reads) -> x
        y.axis <- "Fraction"
        y.value <- "fraction"
    } else if (is(x, "DataFrame")) {
        x <- as.data.frame(x)
    }
    ggplot(x, aes_string(x="length", y=y.value, color="experiment")) +
        geom_line() +
        theme_bw() +
        theme(plot.title=element_text(hjust=0.5)) +
        labs(title=title, x="Read Length", y=y.axis, color="Experiment")
}

check_ld_input <- function(x,
                           region,
                           range.lower,
                           range.upper,
                           experiment) {
    #check the plot_length_distribution output
    if (is(x, "Ribo") && validObject(x)) {
        if (missing(region) || length(region) != 1){
            stop("Please indicate a single region.") 
        } 
        if (missing(experiment)) experiment <- experiments(x)
        if (missing(range.lower)) range.lower <- length_min(x)
        if (missing(range.upper)) range.upper <- length_max(x)
        
        x <- strip_rlefactor(get_length_distribution(ribo.object = x,
                                                     region      = region,
                                                     range.lower = range.lower,
                                                     range.upper = range.upper,
                                                     experiment  = experiment))
    } else if (is(x, "DataFrame") || is(x, "DFrame")) {

        x <- strip_rlefactor(x)
        col.names <- c("experiment", "length", "count")
        types <- c("integer", "double")
        mismatch <- !all(names(x) == col.names, 
                         typeof(x[, "experiment"]) == "character",
                         typeof(x[, "length"]) %in% types, 
                         typeof(x[, "count"]) %in% types,
                         ncol(x) == 3,
                         length(metadata(x)) > 0)
        if (mismatch) {
              stop("Please make sure that the DataFrame is of the correct format.
                    It requires a non-empty metadata field.",
                    call.=FALSE)
        }
    } else if (is.data.frame(x)) {
        col.names <- c("experiment", "length", "count", "total.reads")
        types <- c("integer", "double")
        mismatch <-  !all(names(x) == col.names,                    
                          typeof(x[, "experiment"]) == "character",
                          typeof(x[, "length"]) %in% types,
                          typeof(x[, "count"]) %in% types,    
                          typeof(x[, "total.reads"]) %in% types, 
                          ncol(x) == 4)
        if (mismatch) {
          stop("Please make sure that the data frame is of the correct format.",
               " It requires a 'total.reads' column.",
               call.=FALSE)
        }
    } else {
        stop("Please make sure that param 'x' is either", 
             "a DataFrame or a Ribo object.",
             call.=FALSE)
    }
    return(x)
}


#' Plots the region counts of UTR5, CDS, and UTR3
#'
#' The function \code{\link{plot_region_counts}} can take either a DataFrame
#' or a "Ribo" object to generate the a stacked bar plot of proportions that
#' correspond to the "UTR5", "CDS", and "UTR3" regions.
#'
#' When given a 'Ribo' object, \code{\link{plot_region_counts}} calls
#' \code{\link{get_region_counts}} to retrieve the necessary information
#' for plotting. This option is in the case that a DataFrame of the
#' region count information is not required.
#'
#' The user can instead provide a DataFrame with the same structure as the
#' output of the \code{\link{get_region_counts}} function where the 'transcript'
#' and 'length' parameters are the default values of TRUE. This also means that
#' the remaining parameters of the \code{\link{plot_region_counts}} function are not necessary.
#' The run time becomes substantially faster when \code{\link{plot_region_counts}} is given
#' the direct DataFrame to plot. However, the DataFrame needs to follow the format and 
#' types in the output of the reading functions 
#'
#' @examples
#' #ribo object use case
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #specify the regions and experiments of interest
#' regions <- c("UTR5", "CDS", "UTR3")
#' experiments <- c("Hela_1", "Hela_2", "WT_1")
#'
#' plot_region_counts(sample,
#'                    range.lower = 2,
#'                    range.upper = 5,
#'                    experiments)
#'
#' #DataFrame use case
#' #obtains the region counts at each individual read length, summed across every transcript
#' region.counts <- get_region_counts(sample,
#'                                    region = regions,
#'                                    range.lower = 2,
#'                                    range.upper = 5,
#'                                    tidy = TRUE,
#'                                    length = TRUE,
#'                                    transcript = TRUE)
#'
#' #the params 'length' and 'transcript' must be set to true to use a DataFrame
#' plot_region_counts(region.counts)
#'
#' @seealso \code{\link{get_region_counts}} to generate a DataFrame that can be provided as input,
#' \code{\link{Ribo}} to create a ribo.object that can be provided as input
#'
#' @param x A 'Ribo' object or a DataFrame generated from \code{\link{get_region_counts}}
#' @param range.lower a lower bounds for a read length range
#' @param range.upper an upper bounds for a read length range
#' @param experiment a list of experiment names
#' @param title a title for the generated plot
#' @importFrom dplyr left_join mutate %>% group_by summarize arrange desc
#' @importFrom tidyr gather
#' @importFrom rlang .data
#' @importFrom ggplot2 ggplot geom_col theme_bw theme ggtitle coord_flip theme
#' @importFrom ggplot2 labs scale_fill_discrete element_blank geom_text position_stack
#' @export
#' @return A 'ggplot' of the region counts for each of the experiments 
plot_region_counts <- function(x,
                               experiment,
                               range.lower,
                               range.upper,
                               title = "Region Counts") {
    rc <- check_plot_rc_input(x, range.lower, range.upper, experiment)
    rc <- as.data.frame(rc)
    
    #prepare data for visualization
    rc %>%
        group_by(.data$experiment) %>%
        summarize(sum=sum(.data$count)) -> rc.total

    rc.total %>%
        left_join(rc, by = "experiment") -> rc

    rc %>%
        mutate(percentage=round(100 * .data$count/sum, 1)) %>%
        mutate(region=factor(.data$region, 
               levels=c("UTR3", "CDS", "UTR5"))) -> rc

    #text label of percentages only in the "CDS" region
    percentages <- replace(rc$percentage, 
                           rc$region != "CDS", "")

    ggplot(rc, 
           aes(x=.data$experiment, y=.data$percentage, fill=.data$region)) +
        geom_col() +
        coord_flip() +
        geom_text(aes(x=.data$experiment, y=50, label=percentages), size=3) +
        theme_bw() +
        theme(plot.title   = element_text(hjust = 0.5),
              panel.border = element_blank(),
              panel.grid   = element_blank()) +
        scale_fill_discrete(breaks = c("UTR5", "CDS", "UTR3")) +
        labs(title=title, fill="Region", x="Experiment", y="Percentage")
}


check_plot_rc_input <- function(x,
                                range.lower,
                                range.upper,
                                experiment) {
    #helper method that checks the plot_region_counts input and returns data
    #for plotting
    regions <- c("UTR5", "CDS", "UTR3")
    if (is(x, "Ribo") && validObject(x)) {
        if (missing(experiment) || is.null(experiment)) {
            experiment <- experiments(x)
        } 
        if (missing(range.lower)) range.lower <- length_min(x)
        if (missing(range.upper)) range.upper <- length_max(x)
        check_lengths(x, range.lower, range.upper)
        x <- strip_rlefactor(get_region_counts(ribo.object = x,
                                                region      = regions,
                                                range.lower = range.lower,
                                                range.upper = range.upper,
                                                length      = TRUE,
                                                transcript  = TRUE,
                                                experiment = experiment))
    } else if (is(x, "DataFrame") || is.data.frame(x)) {
        if (is(x, "DataFrame")) x <- strip_rlefactor(x)

        col.names <- c("experiment", "region", "count")
        mismatch  <- !all(names(x) == col.names, 
                          typeof(x[, "experiment"]) == "character",
                          typeof(x[, "region"]) == "character",
                          typeof(x[, "count"]) %in% c("double", "integer"),
                          ncol(x) == 3)
        if (mismatch) {
            stop("Please make sure that the DataFrame is of ",  
                 "the correct format.", call. = FALSE)
        } else if (!identical(unique(x$region), regions)) {
            stop("Please make sure that the DataFrame only includes the ", 
                 "'UTR5','CDS', and 'UTR3' regions.", call. = FALSE)
        }
    } else {
        stop("Please provide a ribo object or a DataFrame of the",
             "correct format.", call. = FALSE)
    }
    return(x)
}

context("region count functions")
library(ribor)

file.path <- system.file("extdata", "sample.ribo", package = "ribor")
ribo.object<- Ribo(file.path)

file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
green <- Ribo(file.path, rename = rename_default)
total.reads <- get_info(green)[["experiment.info"]][,2][1]

green_rc_alias <- get_region_counts(green,
                                    region = "CDS",
                                    range.lower = 28,
                                    range.upper = 28,
                                    transcript = FALSE,
                                    alias = TRUE,
                                    normalize = FALSE,
                                    experiment = experiments(green)[1])

norm.test1 <- 22 * 1000000 / total.reads

green_rc_original <- get_region_counts(green,
                                       region = "CDS",
                                       range.lower = 28,
                                       range.upper = 28,
                                       transcript = FALSE,
                                       alias = FALSE,
                                       experiment = experiments(green)[1])


test_that("get_region_counts- alias values preserved",
          expect_true(all(as.vector(as.character(green_rc_alias[, 4])) ==
                          as.vector(green_rc_original[, 4]))))

original.names <- unlist(as.character(green_rc_original[, 2]))
actual   <- unlist(green_rc_alias[, 2])
expected <- sapply(original.names, rename_default)

test_that("get_region_counts- name ordering preserved",
          expect_true(all(actual == expected)))

green_rc_alias <- get_region_counts(green,
                                    region = "CDS",
                                    range.lower = 28,
                                    range.upper = 28,
                                    transcript = FALSE,
                                    alias = FALSE,
                                    normalize = TRUE,
                                    experiment = experiments(green)[1])

actual <- green_rc_alias[5, 4]
expected <- norm.test1

test_that("get_region_counts- normalize function",
          expect_true(all(actual == expected)))


green_rc <- get_region_counts(green,
                              region = "CDS",
                              range.lower = 28,
                              range.upper = 28,
                              transcript = TRUE,
                              alias = FALSE,
                              normalize = FALSE,
                              experiment = experiments(green)[1])

green_rc_norm <- get_region_counts(green,
                                   region = "CDS",
                                   range.lower = 28,
                                   range.upper = 28,
                                   transcript = TRUE,
                                   alias = FALSE,
                                   normalize = TRUE,
                                   experiment = experiments(green)[1])

expected <- unlist(green_rc[, 3] * 1000000/total.reads)

actual <- unlist(green_rc_norm[, 3])
threshold <- 0.001

test_that("get_region_counts- normalize function",
          expect_true(all(abs(expected - actual) < threshold)))

green_rc_norm <- get_region_counts(green,
                                   region = "CDS",
                                   range.lower = 28,
                                   range.upper = 28,
                                   transcript = FALSE,
                                   alias = TRUE,
                                   normalize = TRUE,
                                   experiment = experiments(green)[1])

actual <- sum(green_rc_norm[, 4])

test_that("get_region_counts- normalize function",
          expect_true(abs(actual - expected) < threshold))

green_rc_norm <- get_region_counts(green,
                                   region = "CDS",
                                   range.lower = 28,
                                   range.upper = 28,
                                   transcript = FALSE,
                                   length = FALSE,
                                   alias = FALSE,
                                   normalize = TRUE,
                                   experiment = experiments(green)[1])

actual <- sum(green_rc_norm[, 5])
test_that("get_region_counts- normalize function",
          expect_true(abs(actual - expected) < threshold))


rc_1 <- get_region_counts(ribo.object,
                          region = c("UtR5", "cDs", "utr3"),
                          2,
                          5,
                          experiment = "Hela_1")

actual <- c(nrow(rc_1), ncol(rc_1))
expected <- c(3, 3)

test_that("get_region_counts- size",
          expect_equal(actual, expected))


rc_all <- get_region_counts(ribo.object,
                            region = c("UtR5", "UTR5j", "cDs", "utr3j", "utr3"),
                             2,
                             5,
                             experiment = "Hela_1")

actual <- c(nrow(rc_all), ncol(rc_all))
expected <- c(5, 3)

test_that("get_region_counts- size",
          expect_equal(actual, expected))

actual   <- sum(rc_all[, 3])
expected <- 118

test_that("get_region_counts- total count",
           expect_equal(actual, expected))

rc_3 <- get_region_counts(ribo.object,
                          region = c("CDS"),
                          2,
                          5,
                          length = TRUE,
                          transcript = FALSE,
                          experiment = c("Hela_1"))

actual <- c(nrow(rc_3), ncol(rc_3))
expected <- c(3, 4)

test_that("get_region_counts- size",
          expect_equal(actual, expected))

actual <- rc_3[1, ][["count"]]
expected <- 13

test_that("get_region-count- preserving transcripts",
          expect_equal(actual, expected))

rc_4 <- get_region_counts(ribo.object,
                          region = c("UtR5j"),
                          2,
                          5,
                          length = FALSE,
                          transcript = TRUE)

actual <- c(nrow(rc_4), ncol(rc_4))
expected <- c(20, 4)

test_that("get_region-count- size",
          expect_equal(actual, expected))

actual <- sum(rc_4$count)
expected <- 102
test_that("get_region_count- preserving lengths",
          expect_equal(actual, expected))



rc_1 <- get_region_counts(ribo.object,
                          region = c("UtR5", "cDs", "utr3"),
                          2,
                          5,
                          compact = FALSE,
                          experiment = "Hela_1")

actual <- c(nrow(rc_1), ncol(rc_1))
expected <- c(3, 3)

test_that("get_region_counts- noncompact, size",
          expect_equal(actual, expected))


rc_all <- get_region_counts(ribo.object,
                            region = c("UtR5", "UTR5j", "cDs", "utr3j", "utr3"),
                            2,
                            5,
                            compact = FALSE,
                            experiment = "Hela_1")

actual <- c(nrow(rc_all), ncol(rc_all))
expected <- c(5, 3)

test_that("get_region_counts- noncompact, size",
          expect_equal(actual, expected))

actual   <- sum(rc_all[, 3])
expected <- 118

test_that("get_region_counts- noncompact, total count",
          expect_equal(actual, expected))

rc_3 <- get_region_counts(ribo.object,
                          region = c("CDS"),
                          2,
                          5,
                          length = TRUE,
                          transcript = FALSE,
                          compact = FALSE,
                          experiment = c("Hela_1"))

actual <- c(nrow(rc_3), ncol(rc_3))
expected <- c(3, 4)

test_that("get_region_counts- size",
          expect_equal(actual, expected))

actual <- rc_3[1, ][["count"]]
expected <- 13

test_that("get_region-count- noncompact, preserving transcripts",
          expect_equal(actual, expected))

rc_4 <- get_region_counts(ribo.object,
                          region = c("UtR5j"),
                          2,
                          5,
                          compact = FALSE,
                          length = FALSE,
                          transcript = TRUE)

actual <- c(nrow(rc_4), ncol(rc_4))
expected <- c(20, 4)

test_that("get_region-count- noncompact, size",
          expect_equal(actual, expected))

actual <- sum(rc_4$count)
expected <- 102
test_that("get_region_count- noncompact, preserving lengths",
          expect_equal(actual, expected))

ld <- get_length_distribution(ribo.object, "CDS", compact = FALSE)
rc <- get_region_counts(ribo.object, region = "CDS", compact = FALSE, length = FALSE)

test_that("get length distribution- compact, returns data.frame",
          expect_true(is.data.frame(ld)))

test_that("get length distribution- compact, correctness for counts",
          expect_equal(ld$count, rc$count))

ld <- get_length_distribution(ribo.object, "CDS")
rc <- get_region_counts(ribo.object, region = "CDS", compact = FALSE, length = FALSE)
test_that("get length distribution- compact, correctness for counts",
          expect_equal(ld$count, rc$count))

#' @include ribo_class.R
#' @include ribo_methods.R
NULL

initialize_transcript_info <- function(transcript.names,
                                       transcript.lengths) {
    # helper method that generates a hash of the length offset and length
    # of each transcript
    
    num.transcripts <- length(transcript.names)
    if (num.transcripts <= 0) {
        stop("Please make sure that there is a positive amount of transcripts.")
    }
    
    hash.value <- rep(list(c("offset" = 0, "length" = 0)), 
                      length = num.transcripts)
    names(hash.value)         <- transcript.names
    hash.value[[1]]["length"] <-  transcript.lengths[[1]]
    
    i <- 2
    while (i <= num.transcripts) {
        hash.value[[i]][["offset"]] <- hash.value[[i - 1]][["length"]] + 
            hash.value[[i - 1]][["offset"]]
        hash.value[[i]][["length"]] <- transcript.lengths[[i]]
        i <- i + 1
    }
    
    transcript.info <- hash(hash.value)
    length.offset <- hash.value[[num.transcripts]][["offset"]] +
        hash.value[[num.transcripts]][["length"]]
    names(length.offset) <- NULL
    
    return(list(transcript.info = transcript.info,
                length.offset   = length.offset))
}

#' Set the aliases of a ribo object 
#'
#' The function \code{\link{set_aliases}} allows the user to add aliases to a valid
#' ribo object.
#'
#' If there is a different naming convention from the default appris transcriptome,
#' there may be no simple way to generate convenient aliases from the original reference names.
#' As a result, the user can first generate the ribo object and get the reference names, use custom
#' (and likely more intricate) functions to generate a list of aliases, and then pass in a character
#' vector of these aliases. The character vector should match the order of and correspond to the
#' list of reference names retrieved from \code{\link{get_reference_names}}
#' @param ribo.object A 'ribo' object
#' @param rename A function that renames original transcript name into an alias
#' @export
#' @importFrom rhdf5 h5read
#' @importFrom hash hash is.empty
#' @importFrom methods validObject
#' @return A modified 'ribo' object that contains alias information
#' @examples
#' #generate a ribo object with transcript nicknames/aliases
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#' sample <- set_aliases(ribo.object = sample,
#'                       rename = rename_default)
set_aliases <- function(ribo.object, rename) {
    if (!is.empty(alias_hash(ribo.object)) ||
        !is.empty(original_hash(ribo.object))) {
        warning("Ribo object already has a naming convention. Aliases will be overriden.",
                call. = FALSE)
    }
    path <- path(ribo.object)
    original   <- h5read(path,
                         name = "reference/reference_names")
    
    num.transcripts <- length(original)
    alias <- rename_transcripts(path, rename)
    ribo.object@alias.hash    <- hash(keys = alias, values = original)
    ribo.object@original.hash <- hash(keys = original, values = alias)
    return(ribo.object)
}


#' Ribo Class
#' 
#' The Ribo object serves as the main utility vehicle for the ribor package.
#' Specifically, it allows the user to interface with a .ribo file in the R
#" environment through the use of the ribor package. Almost all functions in
#' ribor rely on the Ribo object to read, visualize, and inspect the
#' contents of the .ribo file. 
#' The information stored in this object include the .ribo file path, the 
#' list of experiments, the format version, the reference model, the minimum
#' read length, maximum read length, the left span, the right span, and 
#' other transcript information.
#' 
#' Note that the path parameter takes in a file path and stores it. While 
#' using the package, be sure to not to move or change the location of the 
#' .ribo file. The default names of the transcripts may be difficult to use depending on the 
#' settings used to generate the .ribo file. As a result, we have provided a 
#' rename parameter that integrates well with the Appris reference transcriptome.
#' Users may also define a simple function that processes a given default 
#' transcript name in a one-to-one manner to another custom alias. 
#' 
#' 
#' @param path The path to the .ribo file
#' @param rename A function that renames the original transcript or an already generated
#' character vector of aliases
#' @return Returns an S4 object of class "Ribo" containing a path to the HDF5 file,
#'         various attributes in the root folder, and information about the transcripts 
#'         such as names and lengths
#' @importFrom rhdf5 H5Fopen h5readAttributes h5ls h5read
#' @importFrom hash hash is.empty
#' @importFrom methods new
#' @importFrom tools file_path_as_absolute
#' @examples
#' #generate a ribo object with transcript nicknames/aliases
#' file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
#' sample <- Ribo(file.path, rename = rename_default )
#' @seealso
#' If a ribo object is already generated but aliases want to be added or updated, use the
#' \code{\link{set_aliases}} function.
#' @rdname Ribo-class
#' @export
#' @aliases experiments alias_hash experiment_info format_version has_metadata
#' @aliases left_span length_max length_min length_offset metagene_radius
#' @aliases length_offset metagene_radius original_hash path reference right_span transcript_info
Ribo <- function(path, rename = NULL) {
    ribo.path   <- file_path_as_absolute(path)
    attributes <- h5readAttributes(ribo.path, name = "/")
    transcript.names   <- h5read(ribo.path,
                                 name = "reference/reference_names")
    
    transcript.lengths <- h5read(ribo.path, 
                                 name = "reference/reference_lengths")
    file_info          <- h5ls(ribo.path, recursive = TRUE, all = FALSE)
    transcript.info    <- initialize_transcript_info(transcript.names,
                                                     transcript.lengths)
    
    has.metadata   <- ("metadata" %in% names(attributes))
    
    ribo.object <- new("Ribo", 
                       path            = ribo.path,
                       experiments     = file_info[file_info$group == "/experiments",]$name,
                       format.version  = as.integer(attributes$format_version),
                       reference       = attributes$reference,
                       length.min      = attributes$length_min,
                       length.max      = attributes$length_max,
                       left.span       = attributes$left_span,
                       right.span      = attributes$right_span,
                       metagene.radius = attributes$metagene_radius,
                       length.offset   = transcript.info[['length.offset']],
                       has.metadata    = has.metadata,
                       experiment.info = get_content_info(ribo.path),
                       transcript.info = transcript.info[['transcript.info']])
    
    if (!is.null(rename)) {
        ribo.object <- set_aliases(ribo.object, rename)
    }
    return(ribo.object)
}

context("coverage functions")
library(ribor)

file.path <- system.file("extdata", "sample.ribo", package = "ribor")
ribo.object <- Ribo(file.path)

tidy_cov_1 <- get_coverage(ribo.object,
                           name = "MYC",
                           range.lower = 2,
                           range.upper = 5,
                           tidy = TRUE,
                           experiment = "Hela_1")

tidy_cov_2   <- get_coverage(ribo.object,
                        name = "VEGFA",
                        range.lower = 2,
                        range.upper = 5,
                        tidy = TRUE,
                        experiment = "Hela_1")

tidy_cov_3   <- get_coverage(ribo.object,
                        name = "GAPDH",
                        range.lower = 2,
                        range.upper = 5,
                        tidy = TRUE,
                        experiment = c("Hela_1"))

actual <- sum(tidy_cov_1[, 3]) +
          sum(tidy_cov_2[, 3]) +
          sum(tidy_cov_3[, 3])

actual <- unname(actual)
expected <- 118

test_that("get_coverage, tidy - total reads of an experiment",
          expect_equal(actual, expected))

tidy_cov_1 <- get_coverage(ribo.object,
                           name = "GAPDH",
                           range.lower = 2,
                           range.upper = 5,
                           tidy = TRUE,
                           length = FALSE,
                           experiment = "Hela_1")

expected <- c(1, 1, 0, 1)
actual   <- unname(unlist(tidy_cov_1[as.integer(tidy_cov_1$position) == 2, 4]))

test_that("get_coverage, tidy - randomly choose a value",
          expect_equal(actual, expected))

tidy_cov_2 <- get_coverage(ribo.object,
                           name = "MYC",
                           range.lower = 2,
                           range.upper = 5,
                           tidy = TRUE,
                           length = FALSE,
                           experiment = "Hela_1")


actual <- unname(unlist(tidy_cov_2[as.integer(tidy_cov_2$position) == 11, 4]))
expected <- c(2, 1, 3, 7)
test_that("get_coverage, tidy - randomly choose a value",
          expect_equal(actual, expected))

cov_1   <- get_coverage(ribo.object,
                        name = "MYC",
                        range.lower = 2,
                        range.upper = 5,
                        experiment = "Hela_1")

cov_2   <- get_coverage(ribo.object,
                        name = "VEGFA",
                        range.lower = 2,
                        range.upper = 5,
                        experiment = "Hela_1")

cov_3   <- get_coverage(ribo.object,
                        name = "GAPDH",
                        range.lower = 2,
                        range.upper = 5,
                        experiment = c("Hela_1"))

actual <- sum(unlist(as.list(cov_1[, -1]))) + 
          sum(unlist(as.list(cov_2[, -1]))) + 
          sum(unlist(as.list(cov_3[, -1])))

expected <- 118

test_that("get_coverage- total reads of an experiment",
          expect_equal(actual, expected))

actual <- unname(unlist(cov_1[, 12]))
expected <- 13

test_that("get_coverage- correct sum across lengths",
          expect_equal(actual, expected))

cov_4   <- get_coverage(ribo.object,
                        name = "GAPDH",
                        range.lower = 2,
                        range.upper = 2,
                        experiment = c("Hela_1"))

actual   <- sum(unlist(as.list(cov_4[, -1])))
expected <- 14 

test_that("get_coverage- test individual read length",
          expect_equal(actual, expected))

cov_5   <- get_coverage(ribo.object,
                        name = "MYC",
                        range.lower = 3,
                        range.upper = 3,
                        experiment = c("Hela_2"))

actual <- sum(unlist(as.list(cov_5[, -1])))
expected <- 3 

test_that("get_coverage- test individual read length",
          expect_equal(actual, expected))

file.path <- system.file("extdata", "CBS.ribo", package = "ribor")
CBS <- Ribo(file.path)

CBS_name <- names(CBS@transcript.info)[1]

CBS_cov <- get_coverage(CBS,
                        name = CBS_name,
                        range.lower = 15,
                        range.upper = 35)
actual <- sum(unlist(as.list(CBS_cov[ , -1])))
expected <- 846
test_that("get_coverage- edge case transcript only", expect_equal(actual, expected))

tidy_cov_1 <- get_coverage(ribo.object,
                           name = "MYC",
                           range.lower = 2,
                           range.upper = 5,
                           tidy = TRUE,
                           compact = FALSE,
                           experiment = "Hela_1")

tidy_cov_2   <- get_coverage(ribo.object,
                             name = "VEGFA",
                             range.lower = 2,
                             range.upper = 5,
                             tidy = TRUE,
                             compact = FALSE,
                             experiment = "Hela_1")

tidy_cov_3   <- get_coverage(ribo.object,
                             name = "GAPDH",
                             range.lower = 2,
                             range.upper = 5,
                             tidy = TRUE,
                             compact = FALSE,
                             experiment = c("Hela_1"))

actual <- sum(tidy_cov_1[, 3]) +
    sum(tidy_cov_2[, 3]) +
    sum(tidy_cov_3[, 3])

actual <- unname(actual)
expected <- 118

test_that("get_coverage, noncompact, tidy - total reads of an experiment",
          expect_equal(actual, expected))

tidy_cov_1 <- get_coverage(ribo.object,
                           name = "GAPDH",
                           range.lower = 2,
                           range.upper = 5,
                           tidy = TRUE,
                           length = FALSE,
                           compact = FALSE,
                           experiment = "Hela_1")

expected <- c(1, 1, 0, 1)
actual   <- unname(unlist(tidy_cov_1[as.integer(tidy_cov_1$position) == 2, 4]))

test_that("get_coverage, tidy - randomly choose a value",
          expect_equal(actual, expected))

tidy_cov_2 <- get_coverage(ribo.object,
                           name = "MYC",
                           range.lower = 2,
                           range.upper = 5,
                           tidy = TRUE,
                           length = FALSE,
                           compact = FALSE,
                           experiment = "Hela_1")


actual <- unname(unlist(tidy_cov_2[as.integer(tidy_cov_2$position) == 11, 4]))
expected <- c(2, 1, 3, 7)
test_that("get_coverage, noncompact, tidy - randomly choose a value",
          expect_equal(actual, expected))

cov_1   <- get_coverage(ribo.object,
                        name = "MYC",
                        range.lower = 2,
                        range.upper = 5,
                        compact = FALSE,
                        experiment = "Hela_1")

cov_2   <- get_coverage(ribo.object,
                        name = "VEGFA",
                        range.lower = 2,
                        range.upper = 5,
                        compact = FALSE,
                        experiment = "Hela_1")

cov_3   <- get_coverage(ribo.object,
                        name = "GAPDH",
                        range.lower = 2,
                        range.upper = 5,
                        compact = FALSE,
                        experiment = c("Hela_1"))

actual <- sum(unlist(as.list(cov_1[, -1]))) + 
    sum(unlist(as.list(cov_2[, -1]))) + 
    sum(unlist(as.list(cov_3[, -1])))

expected <- 118

test_that("get_coverage- noncompact, total reads of an experiment",
          expect_equal(actual, expected))

actual <- unname(unlist(cov_1[, 12]))
expected <- 13

test_that("get_coverage- noncompact, correct sum across lengths",
          expect_equal(actual, expected))

cov_4   <- get_coverage(ribo.object,
                        name = "GAPDH",
                        range.lower = 2,
                        range.upper = 2,
                        compact = FALSE,
                        experiment = c("Hela_1"))

actual   <- sum(unlist(as.list(cov_4[, -1])))
expected <- 14 

test_that("get_coverage- noncompact, test individual read length",
          expect_equal(actual, expected))

cov_5   <- get_coverage(ribo.object,
                        name = "MYC",
                        range.lower = 3,
                        range.upper = 3,
                        compact = FALSE,
                        experiment = c("Hela_2"))

actual <- sum(unlist(as.list(cov_5[, -1])))
expected <- 3 

test_that("get_coverage- noncompact, test individual read length",
          expect_equal(actual, expected))

context("rnaseq functions")
library(ribor)

experiments <- c("Hela_1", "Hela_2", "WT_1")
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
ribo.object <- Ribo(file.path)

rnaseq_tidy <- get_rnaseq(ribo.object,
                          experiment = experiments)

rnaseq <- get_rnaseq(ribo.object,
                     tidy = FALSE,
                     experiment = experiments)

actual <- get_rnaseq(ribo.object,
                     region = c("UTR3j", "cds"),
                     tidy = FALSE,
                     experiment = experiments)


expected <-rnaseq[, -c(3, 4, 7)]

test_that("get_rnaseq = non-tidy CDS and UTR3J",
          expect_true(all(actual$CDS == expected$CDS)))

actual <- get_rnaseq(ribo.object,
                     region = c("UTR3", "cds", "utR5"),
                     tidy = FALSE,
                     experiment = experiments)

expected <-rnaseq[, -c(4, 6)]

test_that("get_rnaseq = non-tidy CDS and UTR3J",
          expect_true(all(actual$CDS == expected$CDS)))

actual   <- get_rnaseq(ribo.object,
                         tidy = TRUE,
                         region = c("UTR5"),
                         experiment = experiments)

expected <- rnaseq_tidy[as.character(rnaseq_tidy$region) == "UTR5", ]


test_that("get_rnaseq- tidy UTR5 only",
          expect_true(all(actual$count == expected$count)))

actual   <- get_rnaseq(ribo.object,
                       tidy = TRUE,
                       region = c("CDS", "utR5j"),
                       experiment = experiments)

expected <- rnaseq_tidy[as.character(rnaseq_tidy$region) %in% c("UTR5J", "CDS"), ]

test_that("get_rnaseq- tidy UTR5 and CDS",
          expect_true(all(actual$count == expected$count)))

actual   <- c(nrow(rnaseq_tidy), ncol(rnaseq_tidy))
expected <- c(45, 4)

test_that("get_rnaseq- tidy version size",
          expect_equal(actual, expected))

hela_1_rnaseq <- get_rnaseq(ribo.object,
                            tidy = FALSE,
                            experiment = "Hela_1")

actual   <- rnaseq[as.character(rnaseq$experiment) == "Hela_1", ]
expected <- hela_1_rnaseq

test_that("get_rnaseq- subsetting experiments",
          expect_equal(actual$UTR5, expected$UTR5))

actual   <- rnaseq_tidy[as.character(rnaseq_tidy$experiment) == "Hela_1" &
                        as.character(rnaseq_tidy$region) == "UTR5", 4]
actual   <- unname(actual)
actual   <- unlist(actual)

expected <- c(2.04, 3.2, 0)
error <- 1e-2

test_that("get_rnaseq- correct values",
          expect_equal(all(abs(expected - actual) <= error), TRUE))

actual   <- rnaseq[as.character(rnaseq$experiment) == "WT_1" &
                   as.character(rnaseq$transcript) == "GAPDH", c(-1,-2)]
actual   <- unlist(actual)

expected <- c(2.04, 5.3, 2.89, 1.78, 3.0)

result <- all(abs(expected - actual) <= error)

test_that("get_rnaseq- correct values",
          expect_equal(all(abs(expected - actual) <= error), expected = TRUE))

actual <- c(nrow(rnaseq), ncol(rnaseq))
expected <- c(9, 7)

test_that("get_rnaseq- non-tidy version size",
          expect_equal(actual, expected))

rnaseq <- get_rnaseq(ribo.object,
                     tidy = FALSE,
                     experiment = experiments,
                     compact = FALSE)

rnaseq_tidy <- get_rnaseq(ribo.object,
                          experiment = experiments,
                          compact = FALSE)

actual <- get_rnaseq(ribo.object,
                     region = c("UTR3j", "cds"),
                     tidy = FALSE,
                     experiment = experiments)


expected <-rnaseq[, -c(3, 4, 7)]

test_that("get_rnaseq = compact CDS and UTR3J",
          expect_true(all(actual$CDS == expected$CDS)))

actual <- get_rnaseq(ribo.object,
                     region = c("UTR3", "cds", "utR5"),
                     tidy = FALSE,
                     experiment = experiments)

expected <-rnaseq[, -c(4, 6)]

test_that("get_rnaseq = non-tidy CDS and UTR3J",
          expect_true(all(actual$CDS == expected$CDS)))

actual   <- get_rnaseq(ribo.object,
                       tidy = TRUE,
                       region = c("UTR5"),
                       experiment = experiments)

expected <- rnaseq_tidy[as.character(rnaseq_tidy$region) == "UTR5", ]


test_that("get_rnaseq- compact tidy UTR5 only",
          expect_true(all(actual$count == expected$count)))

actual   <- get_rnaseq(ribo.object,
                       tidy = TRUE,
                       region = c("CDS", "utR5j"),
                       experiment = experiments)

expected <- rnaseq_tidy[as.character(rnaseq_tidy$region) %in% c("UTR5J", "CDS"), ]

test_that("get_rnaseq- tidy UTR5 and CDS",
          expect_true(all(actual$count == expected$count)))

### =========================================================================
### Factor objects
### -------------------------------------------------------------------------
###
### The Factor class serves a similar role as factor in base R except that
### the levels of a Factor object can be any vector-like object.
###
#' @rdname Ribo-class
#' @export
#' @aliases Ribo-class

setClass(
    "Ribo",
    slots =      c(path            = "character",
                   experiments     = "character",
                   format.version  = "integer",
                   reference       = "character",
                   length.min      = "integer",
                   length.max      = "integer",
                   left.span       = "integer",
                   right.span      = "integer",
                   metagene.radius = "integer",
                   length.offset   = "numeric",
                   has.metadata    = "logical",
                   experiment.info = "data.frame",
                   transcript.info = "hash",
                   alias.hash      = "hash",
                   original.hash   = "hash"))

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Validity
###

setValidity("Ribo", function(object) {
    # The validity method is to protect against function calls with ribo
    # objects that have been modified in a way that could generate incorrect
    # output
    
    # Generate all of the values found on the .ribo file itself 
    attributes         <- h5readAttributes(object@path, name = "/")
    file_info          <- h5ls(object@path, recursive = TRUE, all = FALSE)
    transcript.names   <- h5read(object@path,
                                 name = "reference/reference_names")
    transcript.lengths <- h5read(object@path, 
                                 name = "reference/reference_lengths")
    transcript.info    <- initialize_transcript_info(transcript.names,
                                                     transcript.lengths)
    
    errors <- character()
    expected_radius <- attributes$metagene_radius
    
    if (expected_radius != object@metagene.radius) {
        msg <- paste("Metagene radius is different from the .ribo file.")
        errors <- c(errors, msg)
    }
    
    num_experiments <- length(object@experiments)
    
    if (num_experiments <= 0) {
        msg <- paste("File appears to have no experiments.")
        errors <- c(errors, msg)
    }
    
    if (!all(file_info[file_info$group == "/experiments",]$name == object@experiments)) {
        msg <- paste("Experiments are different from the .ribo file.")
        errors <- c(errors, msg)
    }
    
    
    length.min <- object@length.min
    length.max <- object@length.max 
    
    if (length.min > length.max) {
        msg <- paste("The minimum read length should be less than or equal ",
                     "to the maximum read length.", sep = "")
        errors <- c(errors, msg)
    }
    
    if (length.min != attributes$length_min) {
        msg <- paste("The minimum read length differs from the .ribo file minimum read length.")
        errors < c(errors, msg)
    }
    
    if (length.max != attributes$length_max) {
        msg <- paste("The maximum read length differs from the .ribo file maximum read length.")
        errors <- c(errors, msg)
    }
    
    if (object@length.offset != transcript.info[['length.offset']]) {
        msg <- paste("The length offset differs from the .ribo file read length")
        errors <- c(errors, msg)
    }
    
    if (object@left.span != attributes$left_span) {
        msg <- paste("The left span differs from the .ribo file left span.")
        errors <- c(errors, msg)
    }
    
    if (object@right.span != attributes$right_span) {
        msg <- paste("The right span differs from the .ribo file right span.")
        errors <- c(errors, msg)
    }
    
    if (!all(object@experiment.info == get_content_info(object@path))) {
        msg <- paste("The experiment info differs from the .ribo file experiment info.")
        errors <- c(errors, msg)
    }
    
    if (length(errors) == 0) TRUE else errors
})

check_alias <- function(ribo.object,
                        alias) {
    has.alias <- !is.empty(alias_hash(ribo.object)) &&
                 !is.empty(original_hash(ribo.object))
    if (alias && !has.alias) {
        stop("Transcripts do not have any aliases.", call. = FALSE)
    }
}

check_lengths <- function(ribo.object, range.lower, range.upper) {
    # Helper method that checks for correct lengths
    #
    # check_lengths directly reads the .ribo file for its lowest and highest read
    # length and compares it to the corresponding parameters
    #
    # Args:
    # ribo.object 
    # range.lower lowest read length
    # range.upper highest read length
    #
    # Return:
    # none

    min.length <- length_min(ribo.object)
    max.length <- length_max(ribo.object)

    if ((range.lower < min.length | range.lower > range.upper)) {
        stop(
            "Param range.lower must be greater than or equal to the minimum
            length and less than range.upper.",
            call. = FALSE
        )
    } else if ((range.upper > max.length |
                range.upper < range.lower)) {
        stop(
            "Param range.upper must be less than or equal to the maximum length
            and greater than or equal to range.lower.",
            call. = FALSE
        )
    }
}

check_experiments <- function(ribo.object, experiment) {
    # Helper method that checks if the user-given experiments
    # are present in the current ribo file
    # Args:
    # ribo.object
    # Return:
    # None
    ribo.experiments <- experiments(ribo.object)
    matched.experiments <- intersect(experiment, ribo.experiments)

    if (!length(matched.experiments)) {
        stop("Param 'experiments' contained no valid experiments.",
             call. = FALSE)
    }

    #deals with missing experiments
    check <- setdiff(experiment, matched.experiments)
    if (length(check)) {
        for (exp in check) {
            warning("'", exp, "'", " was not found.", call. = FALSE)
        }
        warning("Param 'experiment' contained experiments that were not ", 
                "found. The returned data frame ignores these experiments.",
                call. = FALSE)
    }
}

#' Retrieves the metagene data from a .ribo file
#'
#' The function \code{\link{get_metagene}} returns a data frame that provides
#' the coverage at the positions surrounding the metagene start or stop site.
#'
#' The dimensions of the returned data frame depend on the parameters
#' range.lower, range.upper, length, and transcript.
#'
#' The param 'length' condenses the read lengths together.
#' When length is TRUE and transcript is FALSE, the
#' data frame presents information for each transcript across
#' all of the read lengths. That is, each transcript has a value
#' that is the sum of all of the counts across every read length.
#' As a result, information about the transcript at each specific
#' read length is lost.
#'
#' The param 'transcripts' condenses the transcripts together.
#' When transcript is TRUE and length is FALSE, the data
#' frame presents information at each read length between range.lower and
#' range.upper inclusive. That is, each separate read length denotes the
#' sum of counts from every transcript. As a result, information about the
#' counts of each individual transcript is lost.
#'
#' If both 'length' and 'transcript' are TRUE, then the resulting
#' data frame prints out one row for each experiment. This provides the metagene
#' information across all transcripts and all reads in a given experiment.
#'
#' If both length' and 'transcript' are FALSE, no calculations are done to the data,
#' all information is preserved for both the read length and the transcript.
#' The data frame would just present the entire stored raw data
#' from the read length 'range.lower' to the read length 'range.upper' which in most
#' cases would result in a slow run time with a massive DataFrame returned.
#'
#' When 'transcript' is set to FALSE, the 'alias' parameter specifies whether
#' or not the returned DataFrame should present each transcript as an alias
#' instead of the original name. If 'alias' is set to TRUE, then the returned
#' data frame will contain the aliases rather than the original
#' reference names of the .ribo file.
#'
#' @param ribo.object A 'Ribo' object
#' @param site "start" or "stop" site coverage
#' @param range.lower Lower bound of the read length, inclusive
#' @param range.upper Upper bound of the read length, inclusive
#' @param transcript Logical value that denotes if the metagene information should be summed across transcripts
#' @param length Logical value that denotes if the metagene information should be summed across read lengths
#' @param experiment List of experiment names
#' @param alias Option to report the transcripts as aliases/nicknames
#' @param compact Option to return a DataFrame with Rle and factor as opposed to a raw data.frame
#' @return An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
#' metagene information for either the 'stop' or 'start' site provided in the 'site' parameter. The
#' returned data frame will have a length column when the 'length' parameter is set to FALSE, indicating
#" that the count information will not be summed across the provided range of read lengths. Similarly,
#' the returned data frame will have a transcript column whe the 'transcript' parameter is set to FALSE,
#' indicating that the count information will not be summed across the transcripts.
#' In the case that transcript parameter is 'FALSE', the returned data frame will present the transcripts according
#' to the aliases specified at the creation of the ribo object if the 'alias' parameter is set to TRUE.
#' @examples
#'
#' #generate the ribo object by providing the file.path to the ribo file
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#'
#' #extract the total metagene information for all experiments
#' #across the read lengths and transcripts of the start site
#' #from read length 2 to 5
#' metagene_info <- get_metagene(ribo.object = sample,
#'                               site = "start",
#'                               range.lower = 2,
#'                               range.upper = 5,
#'                               length = TRUE,
#'                               transcript = TRUE,
#'                               experiment = experiments(sample))
#'
#'
#' #Note that length, transcript, and experiments in this case are the
#' #default values and can be left out. The following generates the same output.
#'
#' metagene_info <- get_metagene(ribo.object = sample,
#'                               site = "start",
#'                               range.lower = 2,
#'                               range.upper = 5)
#'
#' @seealso
#' \code{\link{Ribo}} to generate the necessary 'Ribo' class object,
#' \code{\link{plot_metagene}} to visualize the metagene data,
#' \code{\link{get_tidy_metagene}} to obtain tidy metagene data under certain conditions
#' @importFrom rhdf5 h5read
#' @importFrom methods as 
#' @importFrom S4Vectors DataFrame Rle
#' @importFrom hash keys
#' @export
get_metagene <- function(ribo.object,
                         site,
                         range.lower = length_min(ribo.object),
                         range.upper = length_max(ribo.object),
                         transcript = TRUE,
                         length = TRUE,
                         alias = FALSE,
                         compact = TRUE,
                         experiment = experiments(ribo.object)) {
    range.info <- c(range.lower = range.lower, range.upper = range.upper)
    conditions <- c(transcript = transcript, length = length, alias = alias)
    site <- tolower(site)
    check_metagene_input(ribo.object, site, range.info, experiment, alias)

    path                <- path(ribo.object)
    range.min           <- length_min(ribo.object)
    metagene.radius     <- h5readAttributes(path, "/")[["metagene_radius"]]
    ncol                <- 2 * metagene.radius + 1
    columns             <- seq(ncol)
    ref.length          <- length(get_reference_names(ribo.object))

    row.start <- (range.lower - range.min) * ref.length + 1
    row.stop <- row.start + ref.length*(range.upper - range.lower + 1) - 1
    rows <- c(row.start:row.stop)

    #gather information to use in filling and labeling final data frame
    matched.experiments <- intersect(experiment, experiments(ribo.object))
    exp_paths <- vapply(matched.experiments, get_metagene_path, site = site,
                        FUN.VALUE = "character")
    data  <- lapply(exp_paths, generate_matrix,
                               ribo.object = ribo.object,
                               transcript = transcript,
                               length = length,
                               normalize = FALSE,
                               ncol = ncol,
                               file = path,
                               index = list(columns, rows))

    data <- as.data.frame(do.call(rbind, data))
    colnames(data) <- c(as.character(-metagene.radius:metagene.radius))
    
    result <- make_dataframe(ribo.object,
                             matched.experiments,
                             range.info,
                             conditions,
                             data)
    if(compact) {
      result <- as(result, "DataFrame")
      result <- prepare_DataFrame(ribo.object, result)
    }
    
    return(result)
}


get_metagene_path <- function(experiment, site) {
    # helper method that generates a path within the ribo file
    dataset.name <- paste(site, "_site_coverage", sep = "")
    data.path <- paste("/experiments/", experiment, "/metagene/", sep = "")
    path <- paste(data.path, dataset.name, sep = "")
    return(path)
}
#' Retrieves the metagene data in a tidy format
#'
#' The function \code{\link{get_tidy_metagene}} provides the user with a tidy data format for easier
#' data cleaning and manipulation. In providing this functionality while reducing the returned data frame
#' size, the user must aggregate across the transcripts and is only provided the option to aggregate the 
#' read lengths together.
#'
#' The dimensions of the returned data frame depend on the parameters
#' range.lower, range.upper, and length.
#'
#' The param 'length' condenses the read lengths together.
#' When length is TRUE, then the resulting data frame prints out one row
#' for each experiment. This provides a tidy format of the metagene information
#' across all transcripts and all read lengths in a given experiment. Each row
#' in the data frame represents the total metagene coverage count of a given experiment
#' at a given position.
#'
#' When the param  'length' is FALSE, then the resulting data frame prints out the
#' metagene coverage count at each position of the metagene radius for each read length.
#' This provides a tidy format of the metagene information across the transcripts, preserving
#' the metagene coverage count at each read length.
#' 
#'
#' @param ribo.object A 'Ribo' object
#' @param site "start" or "stop" site coverage
#' @param range.lower Lower bound of the read length, inclusive
#' @param range.upper Upper bound of the read length, inclusive
#' @param length Logical value that denotes if the metagene information should be summed across read lengths
#' @param experiment List of experiment names
#' @param compact Option to return a DataFrame with Rle and factor as opposed to a raw data.frame
#' @return An annotated, tidy DataFrame or data.frame (if the compact parameter is set to FALSE) of the
#' metagene information for either the 'stop' or 'start' site provided in the 'site' parameter. The data frame,
#' as a result of its tidy property, will have a position column.
#' The returned data frame will have a length column when the 'length' parameter is set to FALSE, indicating
#" that the count information will not be summed across the provided range of read lengths. Note that the transcripts
#' will be automatically aggregated to keep the memory footprint of this function reasonable.
#' @examples
#' #generate the ribo object by loading in a ribo function and calling the \code{\link{Ribo}} function
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #extract the total metagene information in a tidy format
#' #for all experiments across the read lengths and transcripts
#' #of the start site from read length 2 to 5
#'
#' metagene_info <- get_tidy_metagene(ribo.object = sample,
#'                                    site = "start",
#'                                    range.lower = 2,
#'                                    range.upper = 5,
#'                                    length = TRUE,
#'                                    experiment = experiments(sample))
#'
#' #Note that length and experiments in this case are the
#' #default values and can be left out. The following generates the same output.
#' metagene_info <- get_tidy_metagene(ribo.object = sample,
#'                                    site = "start",
#'                                    range.lower = 2,
#'                                    range.upper = 5)
#'
#' @seealso
#' \code{\link{Ribo}} to generate the necessary 'Ribo' class object.
#' \code{\link{plot_metagene}} to visualize the metagene data,
#' \code{\link{get_metagene}} to obtain tidy metagene data under certain conditions
#' @importFrom rhdf5 h5read
#' @importFrom tidyr gather
#' @export
get_tidy_metagene <- function(ribo.object,
                              site,
                              range.lower = length_min(ribo.object),
                              range.upper = length_max(ribo.object),
                              length = TRUE,
                              compact = TRUE,
                              experiment = experiments(ribo.object)) {
  site <- tolower(site)
  result <- get_metagene(ribo.object,
                         site,
                         range.lower,
                         range.upper,
                         length,
                         transcript = TRUE,
                         alias = FALSE,
                         experiment = experiment)
  
  result <- strip_rlefactor(result)
  metagene.radius <- as.integer((ncol(result) - 2) / 2)
  result <- data.frame(result, check.names = FALSE)
  tidy.data <- gather(result,
                      key = "position",
                      value = "count",
                      c(as.character(-metagene.radius:metagene.radius)))
  tidy.data$position <- as.integer(tidy.data$position)
  
  if(compact) {
    tidy.data <- as(tidy.data, "DataFrame")
    tidy.data <- prepare_DataFrame(ribo.object, tidy.data)
  } else {
    tidy.data %>%
      left_join(get_info(ribo.object)$experiment.info[, c("experiment", "total.reads")],
                by = "experiment")  -> tidy.data
  }
  return(tidy.data)
}


check_metagene_input <- function(ribo.object,
                                 site,
                                 range.info,
                                 experiment,
                                 alias) {
  #check_metagene_input is a helper function that checks the validity of
  #the metagene function parameters
  #check param validity
  if (!is(ribo.object, "Ribo")) stop("Please provide a ribo object.")
  if (site != "start" & site != "stop") {
    stop("Please type 'start' or 'stop' to indicate the 'site' parameter value.")
  }
  
  range.lower <- range.info[["range.lower"]]
  range.upper <- range.info[["range.upper"]]

  check_alias(ribo.object, alias)
  check_lengths(ribo.object, range.lower, range.upper)
  check_experiments(ribo.object, experiment)
}

#' Plots the metagene coverage data
#'
#' The function \code{\link{plot_metagene}} plots the metagene site coverage,
#' separating by experiment.
#'
#' If a DataFrame is provided as param 'x', then the only additional parameter
#' is the optional title' parameter for the generated plot. If a ribo.object is
#' provided as param 'x', the rest of the parameters listed are necessary.
#'
#' When given a ribo class object, the \code{\link{plot_metagene}} function
#' generates a DataFrame by calling the \code{\link{get_tidy_metagene}}
#' function, so the run times in this case will be mostly comprised of a call
#' to the \code{\link{get_metagene}} function.
#'
#' This function uses ggplot in its underlying implementation.
#'
#' @param x A 'Ribo' object or a data frame generated from \code{\link{get_metagene}}
#' @param site "start" or "stop" site
#' @param range.lower lower bound of the read length, inclusive
#' @param range.upper upper bound of the read length, inclusive
#' @param experiment list of experiments
#' @param normalize When TRUE, normalizes the data by the total reads.
#' @param title title of the generated plot
#' @param tick x-axis labeling increment
#' @examples
#' #a potential use case is to directly pass in the ribo object file as param 'x'
#'
#' #generate the ribo object to directly use
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #specify experiments of interest
#' experiments <- c("Hela_1", "Hela_2", "WT_1")
#'
#' #plot the metagene start site coverage for all experiments in 'sample.ribo'
#' #from read length 2 to 5
#' plot_metagene(x = sample,
#'               site = "start",
#'               range.lower = 2,
#'               range.upper = 5,
#'               experiment = experiments)
#'
#' #Note that the site, range.lower, range.upper, and experiment parameter are only
#' #necessary if a ribo object is being passed in as param 'x'. If a ribo
#' #object is passed in, then the param 'experiments' will be set to all of
#' #the experiments by default.
#'
#' #If a DataFrame is passed in, then the plot_metagene function
#' #does not need any other information. All of the elements of the DataFrame
#' #will be used, assuming that it contains the same column names and number of
#' #columns as the output from get_tidy_metagene()
#'
#' #gets the metagene start site coverage from read length 2 to 5
#' #note that the data must be summed across transcripts and read lengths
#' #for the plot_metagene function
#' data <- get_tidy_metagene(sample,
#'                           site = "start",
#'                           range.lower = 2,
#'                           range.upper = 5)
#'
#' #plot the metagene data
#' plot_metagene(data)
#'
#' @importFrom dplyr left_join mutate
#' @importFrom ggplot2 ggplot geom_line theme_bw ggtitle aes expand_limits
#' @importFrom ggplot2 element_text theme labs scale_x_continuous
#' @importFrom rlang .data
#' @importFrom tidyr gather
#' @export
#' @return
#' A 'ggplot' of the metagene site coverage
plot_metagene <- function(x,
                          site,
                          experiment,
                          range.lower,
                          range.upper,
                          normalize = FALSE,
                          title = "Metagene Site Coverage",
                          tick = 10) {
    x <- check_plot_metagene(x,
                             site,
                             range.lower,
                             range.upper,
                             experiment)

    y.value <- "count"
    y.label <- "Count"
    
    if (normalize) {
      per.million <- 1000000
      if (is(x, "DataFrame")) {
          info <- metadata(x)[[1]]
          x %>% 
            strip_rlefactor() %>% 
            as.data.frame() %>% 
            left_join(info, by = "experiment") %>% 
            mutate(normalize = per.million * .data$count/.data$total.reads) -> x
      } else {
          x <- mutate(x, normalize=(.data$count/.data$total.reads)*per.million)
      }
      y.value <- "normalize"
      y.label <- "Counts per 1M Reads"
    } else {
      x <- as.data.frame(x)
    }

    metagene.radius <- max(x$position)
    ggplot(x,
           aes_string(x     = "position", 
                      y     = y.value, 
                      color = "experiment")) +
    scale_x_continuous(breaks=seq(-metagene.radius, metagene.radius, tick)) +
    expand_limits(y=0) +
    geom_line() +
    theme_bw() +
    theme(plot.title=element_text(hjust = 0.5)) +
    labs(title=title, x="Position", y=y.label, color="Experiment")
}

check_plot_metagene <- function(x,
                                site,
                                range.lower,
                                range.upper,
                                experiment) {
    
    #x is a ribo object
    if (is(x, "Ribo") && validObject(x)) {
        if (missing(site)) {
          stop("Please indicate the 'site' parameter with either 'start' or 'stop'",
               call. = FALSE)
        } 
        if (missing(experiment)) experiment <- experiments(x)
        if (missing(range.lower)) range.lower <- length_min(x)
        if (missing(range.upper)) range.upper <- length_max(x)
        x <- strip_rlefactor(get_tidy_metagene(x,
                                               site,
                                               range.lower,
                                               range.upper,
                                               length = TRUE,
                                               experiment = experiment))
    } else if (is(x, "DataFrame") || is(x, "DFrame")) {

        x <- strip_rlefactor(x)
        col.names <- c("experiment", "position", "count")
        types <- c("integer", "double")
        mismatch <- !all(names(x) == col.names,
                         typeof(x[, "experiment"]) == "character",
                         typeof(x[, "position"]) %in% types,
                         typeof(x[, "count"]) %in% types,
                         ncol(x) == 3)
        if (mismatch) {
            stop("Please make sure that the data frame is of the correct format.",
                 call.=FALSE)
        }
    } else if (is.data.frame(x)){
      col.names <- c("experiment", "position", "count", "total.reads")
      types <- c("integer", "double")
      mismatch <-  !all(names(x) == col.names,                    
                        typeof(x[, "experiment"]) == "character",
                        typeof(x[, "position"]) %in% types,
                        typeof(x[, "count"]) %in% types,    
                        typeof(x[, "total.reads"]) %in% types, 
                        ncol(x) == 4)
      if (mismatch) {
        stop("Please make sure that the data frame is of the correct format.",
             call.=FALSE)
      }
    } else{ 
        #not a data frame
        stop("Please make sure that param 'x' is either ", 
             "a DataFrame, data.frame, or ribo object.")
    }
    return(x)
}

context("metagene functions")
library(ribor)
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
ribo.object <- Ribo(file.path)

file.path <- system.file("extdata", "CBS.ribo", package = "ribor")
CBS         <- Ribo(file.path)

file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")

green <- Ribo(file.path, rename = rename_default)

green_meta_alias <- get_metagene(green,
                                 "start",
                                 range.lower = 28,
                                 range.upper = 28,
                                 transcript = FALSE,
                                 alias = TRUE,
                                 experiment = experiments(green)[1])

green_meta_original <- get_metagene(green,
                                    "start",
                                    range.lower = 28,
                                    range.upper = 28,
                                    transcript = FALSE,
                                    alias = FALSE,
                                    experiment = experiments(green)[1])

test_that("get_metagene- alias values preserved",
          expect_true(all(green_meta_alias[, 4] == 
                          green_meta_original[, 4])))

original.names <- unlist(as.character(green_meta_original[, 2]))
actual   <- unlist(as.character(green_meta_alias[, 2]))

expected <- sapply(original.names, rename_default)

test_that("get_metagene- name ordering preserved",
           expect_true(all(actual == 
                           expected)))

meta_1 <- get_metagene(ribo.object,
                       "start",
                       range.lower = 2,
                       range.upper = 5,
                       length = TRUE,
                       transcript = TRUE,
                       experiment = c("Hela_1", "Hela_2"))

actual   <- c(nrow(meta_1), ncol(meta_1)) 
expected <- c(2, 6)

test_that("get_metagene size",
          expect_equal(actual, expected))

expected <- 17
actual <- meta_1[as.character(meta_1$experiment) == "Hela_1", ][, 3]
test_that("get_metagene, checks the default case",
          expect_equal(actual, 
                       expected))

meta_2 <- get_metagene(ribo.object,
                       "start",
                       range.lower = 2,
                       range.upper = 3,
                       length = TRUE,
                       transcript = FALSE,
                       experiment = c("Hela_1"))

actual   <- c(nrow(meta_2), ncol(meta_2)) 
expected <- c(3, 7)

test_that("get_metagene- size 1",
          expect_equal(actual, expected))

expected <- 4
actual   <- meta_2[as.character(meta_2$transcript) == "GAPDH", ][["-1"]]
test_that("get_metagene, only condense lengths",
          expect_equal(actual, 
                       expected))

meta_3 <- get_metagene(ribo.object,
                       "stop",
                       range.lower = 3,
                       range.upper = 4,
                       length = FALSE,
                       transcript = TRUE)

expected <- 4
test_that("get_metagene, only condense transcripts",
          expect_equal(meta_3[as.character(meta_3$experiment) == "Hela_1" &
                              as.integer(meta_3$length) == 3, ][["-2"]], 
                       expected))

meta_4 <- get_metagene(ribo.object,
                       "stop",
                       range.lower = 4,
                       range.upper = 4,
                       length = FALSE,
                       transcript = FALSE)

actual <- meta_4[as.character(meta_4$experiment) == "Hela_1" &
                 as.character(meta_4$transcript) == "GAPDH", ][["1"]]
expected <- 1
test_that("get_metagene, preserve everything",
          expect_equal(actual, 
                       expected))


actual   <- c(nrow(meta_4), ncol(meta_4)) 
expected <- c(15, 8)

test_that("get_metagene- size 2",
          expect_equal(actual, expected))

tidy_meta_1 <- get_tidy_metagene(ribo.object,
                                 "start",
                                 2,
                                 3,
                                 length = TRUE)
actual   <- c(nrow(tidy_meta_1), ncol(tidy_meta_1)) 
expected <- c(25, 3)

test_that("get_metagene- size 3",
          expect_equal(actual, expected))


actual <- tidy_meta_1[as.character(tidy_meta_1$experiment) == "Hela_1" &
                          as.integer(tidy_meta_1$position) == -1, ][["count"]]
expected <- 9

test_that("get_metagene, preserve everything",
          expect_equal(actual, 
                       expected))



tidy_meta_2 <- get_tidy_metagene(ribo.object,
                                 "start",
                                 2,
                                 3,
                                 length = FALSE)
actual <- c(nrow(tidy_meta_2), ncol(tidy_meta_2))
expected <- c(50, 4)

test_that("get_metagene- size 4",
          expect_equal(actual, 
                       expected))

actual <- tidy_meta_2[as.character(tidy_meta_2$experiment) == "Hela_2" &
                      as.integer(tidy_meta_2$length) == 3 &
                      as.integer(tidy_meta_2$position) == -1, ][["count"]]

expected <- 5
test_that("get_metagene- preserve everything",
          expect_equal(actual, 
                       expected))


tidy_df <- get_tidy_metagene(ribo.object,
                             "start",
                             compact = FALSE)

test_that("get_metagene- noncompact everything",
          expect_equal(tidy_df[1,3], 
                       7))

meta_df <- get_metagene(ribo.object,
                        "start",
                        2,
                        3,
                        compact = FALSE,
                        length = TRUE)
test_that("get_metagene- base",
           expect_equal(sum(meta_df[1, -1]), 21))

meta_df <- get_metagene(ribo.object,
                        "start",
                        2,
                        3,
                        compact = FALSE,
                        length = FALSE)

test_that("get_metagene- base",
          expect_equal(sum(meta_df[4, -c(1,2)]), 13))

meta_df <- get_metagene(ribo.object,
                        "start",
                        2,
                        3,
                        compact = FALSE,
                        transcript = FALSE)

test_that("get_metagene- base",
          expect_equal(sum(meta_df[1, -c(1, 2)]), 12))

meta_df <- get_metagene(ribo.object,
                        "stop",
                        2,
                        3,
                        compact = FALSE,
                        transcript = FALSE,
                        length = FALSE)

test_that("get_metagene- base",
          expect_equal(nrow(meta_df), 30))

CBS_meta <- get_metagene(CBS, "start", 15, 35)
actual <- as.integer(CBS_meta[, 3])
expected <- 1L
test_that("get_metagene- non-tidy random access", 
          expect_equal(actual,
                       expected))

CBS_tidy <- get_tidy_metagene(CBS, "stop", 15, 35)

actual <- CBS_tidy[as.integer(CBS_tidy$position) == -47, 3]
expected <- 1L
test_that("get_tidy_metagene- check random value", expect_equal(actual, expected))

#' Get information about the .ribo file
#'
#' The function \code{\link{get_info}} provides information on the attributes, metadata,
#' and datasets of the ribo file.
#'
#' The \code{\link{get_info}} first provides information on the format version, left_span, right_span,
#' longest read length, shortest read length, metagene_radius, and reference model. The last element of the
#' returned list contains the information about the presence of coverage and RNA-seq data which are
#' optional datasets to include in a .ribo file.
#'
#' @param ribo.object ribo.object is an S4 object of class "Ribo"
#' @examples
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #retrieve information
#' get_info(sample)
#'
#' @return Returns a list containing a nested list of file attributes, a logical
#' value denoting whether the root file has additional metadata, and a
#' data.frame of information on each experiment
#'
#' @seealso \code{\link{Ribo}} to generate the necessary ribo.object parameter
#'
#' @importFrom rhdf5 h5ls h5readAttributes
#' @export
get_info <- function(ribo.object) {
    validObject(ribo.object)
    path   <- path(ribo.object)
    
    #retrieve an experiment list
    exp.list <- experiments(ribo.object)
    result <- get_attributes(ribo.object)
    has.metadata <- ("metadata" %in% names(result))
    
    if (has.metadata) {
        result <- result[-which(names(result) == "metadata")]
    }
    
    experiment.info <- get_content_info(path)
    result <- list(
        "has.metadata"    = has.metadata,
        "attributes"      = result,
        "experiment.info" = experiment.info
    )
    return(result)
}

#' Retrieves the metadata of an experiment
#'
#' \code{\link{get_metadata}} provides information on all of the user-inputted
#' metadata of an experiment. If the experiment is not found, then the
#' attributes of the root .ribo file is returned instead.
#'
#' @param ribo.object object of class 'ribo'
#' @param name The name of the experiment
#' @param print Logical value indicating whether or not to neatly print the output
#' @examples
#' #ribo object use case
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #the ribo file contains an experiment named 'Hela_1'
#' get_metadata(sample, "Hela_1")
#'
#' @return
#' If a valid experiment name is provided, a list of elements providing all of the metadata of the
#' experiment is returned.
#'
#' If the name is not provided and the root file has metadata, then a list of elements
#' providing all of the metadata found in the root file is returnend.
#'
#' @seealso \code{\link{Ribo}} to generate the necessary ribo.object parameter
#' @importFrom rhdf5 h5readAttributes
#' @importFrom yaml yaml.load
#' @export
get_metadata <- function(ribo.object,
                         name = NULL,
                         print = TRUE) {
    path <- path(ribo.object)
    exp.list <- experiments(ribo.object)
    file_path = "/"
    
    if (!is.null(name)) {
        file_path = paste("experiments/", name, sep = "")
        if (!(name %in% exp.list)) {
            stop("'",
                 name,
                 "'",
                 " is not a valid experiment name.",
                 call. = FALSE)
        }
    }
    
    attribute <- h5readAttributes(path, file_path)
    if ("metadata" %in% names(attribute)) {
        result <- yaml.load(string = attribute[["metadata"]])
        if (print) {
            print_metadata(result, 0)
            invisible(result)
        } else {
            return(result)
        }
    } else {
        stop("File does not have metadata.")
    }
}

print_metadata <- function(metadata, index) {
    for (i in seq_along(metadata)) {
        if (length(metadata[[i]]) >= 2) {
            cat(rep(" ", index * 3), paste(names(metadata)[i], ": \n", sep = ""))
            info <- metadata[[i]]
            print_metadata(info, index + 1)
        } else {
            cat(rep(" ", index * 3),
                paste(names(metadata)[i], ": ", metadata[[i]], "\n", sep = ""))
        }
    }
}

#' Provides a list of experiments from a .ribo file
#'
#' The function \code{\link{get_experiments}} provides a list of experiment names in the .ribo file.
#'
#' \code{\link{get_experiments}} returns a list of strings denoting the experiments. It obtains this
#' by reading directly from the .ribo file through the path of the 'ribo.object' parameter. To generate
#' the param 'ribo.object', call the \code{\link{Ribo}} function and provide the path to the .ribo file of interest.
#'
#' The user can then choose to create a subset from this list for any specific experiments of interest
#' for later function calls. Many functions that have the param 'experiment.list'
#'  call \code{\link{get_experiments}} to generate a default list of all experiments in the
#' .ribo file.
#' @examples
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #get a list of the experiments
#' get_experiments(sample)
#'
#' @seealso \code{\link{Ribo}} to generate the necessary ribo.object parameter
#' @param ribo.object S4 object of class "Ribo"
#' @return A list of the experiment names
#' @importFrom rhdf5 h5ls
#' @export
get_experiments <- function(ribo.object) {
    validObject(ribo.object)
    result <- h5ls(ribo.object@path)
    result <- result[result$group == "/experiments",]
    return(result$name)
}

context("file info functions")
library(ribor)
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
ribo.object   <- Ribo(file.path)
expected.exp <- c("Hela_1", "Hela_2", "WT_1", "WT_2", "WT_3")

#test get_experiments
test_that("get_experiments returns the correct output",
          expect_equal(get_experiments(ribo.object = ribo.object), expected.exp))

actual <- get_info(ribo.object)

names.list     <- expected.exp
reads.list     <- list(118, 118, 12, 12, 12)
coverage.list  <- list(TRUE, TRUE, TRUE, TRUE, FALSE)
rna.seq.list   <- list(TRUE, TRUE, TRUE, FALSE, FALSE)
metadata.list  <- list(TRUE, FALSE, TRUE, TRUE, TRUE)
expected.table <- data.frame(names    = names.list,
                             reads    = reads.list,
                             coverage = coverage.list,
                             rna.seq  = rna.seq.list,
                             metadata = metadata.list)

properties     <- list(1, 3, 5, 2, 2, "appris_human", 2)
unnamed.actual <- actual[[2]]
unname(unnamed.actual)

#test get_info
test_that("get_info- returns a list of the correct length",
           expect_equal(length(actual), 3))

test_that("get_info- checks attributes",
           expect_equal(as.character(unnamed.actual),
                        as.character(properties)))

metadata <- actual[[1]]
test_that("get_info- has.metadata",
          expect_equal(metadata,
                       TRUE))

#test get_rnaseq
abundance.list  <- c(2.89, 15.46, 8.0, 6.42, 1.37, 0.06, 2.89, 15.46, 8.0)

experiment.list <- c("Hela_1", "Hela_1", "Hela_1",
                     "Hela_2", "Hela_2", "Hela_2",
                     "WT_1"  , "WT_1"  , "WT_1")

transcript.list <- c("GAPDH", "VEGFA", "MYC",
                     "GAPDH", "VEGFA", "MYC",
                     "GAPDH", "VEGFA", "MYC")

expected.rnaseq <- data.frame(transcript = transcript.list,
                              experiment = experiment.list,
                              abundance  = abundance.list)

hela.only <- expected.rnaseq[expected.rnaseq$experiment == "Hela_1" |
                             expected.rnaseq$experiment == "Hela_2",]

hela.list     <- c("Hela_1", "Hela_2")
hela.no.match <- c("Hela_3", "Hela_4")


#test get_metadata
hela.meta <- c("HeLa", "5 min", "HindIII", "https://www.encodeproject.org/")

wt.meta <- c("WT", "5 min", "HindIII", "https://www.encodeproject.org/")

actual.hela.meta <- get_metadata(ribo.object, "Hela_1", print = FALSE)
actual.hela.meta <- unname(actual.hela.meta)

test_that("get_metadata- checks correct input for Hela_1",
          expect_equal(as.character(actual.hela.meta), as.character(hela.meta)))

actual.wt.meta <- get_metadata(ribo.object, "WT_1", print = FALSE)
actual.wt.meta <- unname(actual.wt.meta)

test_that("get_metadata- checks correct input for WT_1",
          expect_equal(as.character(actual.wt.meta), as.character(wt.meta)))

actual <- unname(unlist(get_metadata(ribo.object, print = FALSE)))
expected <- c("6/24/2018", "riboflow")
test_that("get_metadata- checks output for file with no metadata",
          expect_equal(actual, expected))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----pressure, echo=FALSE, fig.cap="Figure 1: .ribo File Internal Structure", out.width = '90%', fig.align="center"----
knitr::include_graphics("ribo_file_structure.jpg")

## ----install_bioconductor, eval=FALSE-----------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("ribor")

## ----install_from_github, eval = FALSE----------------------------------------
# install.packages("devtools")
# library("devtools")
# install_github("ribosomeprofiling/ribor")

## ----import_lib, eval = TRUE--------------------------------------------------
library(ribor)

## ----createRibo---------------------------------------------------------------
#file path to the example ribo file 
file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")

#generates the 'ribo' class object 
original.ribo <- Ribo(file.path, rename = rename_default )

## -----------------------------------------------------------------------------
original.ribo

## ----plot_length_distribution ribo, fig.width = 7-----------------------------
plot_length_distribution(x           = original.ribo,
                         region      = "CDS",
                         range.lower = 28,
                         range.upper = 32,
                         fraction    = TRUE)

## ----plot_length_distribution_UTR5 ribo, fig.width = 7------------------------
plot_length_distribution(x           = original.ribo,
                         region      = "CDS",
                         range.lower = 28,
                         range.upper = 32,
                         fraction    = FALSE)

## ----get_length_dist_basic----------------------------------------------------
rc <- get_length_distribution(ribo.object      = original.ribo,
                                   region      = "CDS",
                                   range.lower = 28,
                                   range.upper = 32)
rc

## ----plot_length_distribution_from_dataframe, eval = FALSE--------------------
# # further DataFrame manipulation and filtering can ensue here
# # before calling the plot function
# 
# plot_length_distribution(rc, fraction = TRUE)

## ----learn_metagene_radius----------------------------------------------------
get_info(original.ribo)$attributes$metagene_radius

## ----plot_metagene ribo_start, fig.width = 7----------------------------------
plot_metagene(original.ribo,
              site        = "start",
              experiment  = c("GSM1606107"),
              range.lower = 28,
              range.upper = 32)

## ----plot_metagene ribo, fig.width = 7----------------------------------------
plot_metagene(original.ribo,
              site        = "stop",
              normalize   = TRUE,
              title       = "Stop Site Coverage",
              range.lower = 28,
              range.upper = 32)

## -----------------------------------------------------------------------------
#get the start site across read lengths 28 to 32
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 28,
                           range.upper = 32,
                           length      = TRUE,
                           transcript  = TRUE)
#only print first ten columns 
print(meta.start[ , 1:10])

## -----------------------------------------------------------------------------
tidy.meta.start <- get_tidy_metagene(ribo.object = original.ribo,
                                     site        = "start",
                                     range.lower = 28,
                                     range.upper = 32,
                                     length      = TRUE)
head(tidy.meta.start, 2)

## -----------------------------------------------------------------------------
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 30,
                           range.upper = 30,
                           experiment  = c("GSM1606108"),
                           length      = FALSE,
                           transcript  = TRUE)

print(meta.start[, 1:10])

## ----get_metagene_tr_F_len_T--------------------------------------------------
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 28,
                           range.upper = 32,
                           length      = TRUE,
                           transcript  = FALSE,
                           alias       = TRUE)

print(meta.start[1:2, 1:10])

## ----plot_metagene_from_dt, fig.width = 7-------------------------------------
plot_metagene(tidy.meta.start, normalize = TRUE)

## ----region_definitions, echo=FALSE, fig.cap="Figure 2: Extended Region Definitions", out.width = '90%', fig.align="center"----
knitr::include_graphics("region_definition.jpg")

## ----plot_region_counts ribo, fig.width = 7-----------------------------------
plot_region_counts(x           = original.ribo,
                   range.lower = 28,
                   range.upper = 32)

## ----get_region_counts default------------------------------------------------
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        length      = TRUE,
                        transcript  = TRUE,
                        region      = c("CDS"))
rc

## ----get_region_counts transcript---------------------------------------------
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        length      = TRUE,
                        transcript  = FALSE,
                        alias       = TRUE,
                        region      = c("CDS"))
rc

## ----get_region_counts full---------------------------------------------------
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        length      = FALSE,
                        transcript  = FALSE,
                        alias       = TRUE,
                        region      = c("CDS"))
rc

## ----get_region_counts non-tidy-----------------------------------------------
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        tidy        = FALSE)
rc

## ----get_region_counts normalize----------------------------------------------
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        normalize   = TRUE,
                        region      = c("CDS"))
rc

## ----plot_region_counts dt, fig.width = 7-------------------------------------
rc.info <- get_region_counts(ribo.object = original.ribo,
                             region      = c("UTR5", "CDS", "UTR3"),
                             range.lower = 28,
                             range.upper = 32)

# further data frame manipulation and filtering can ensue here 
# before calling the plot function 

plot_region_counts(rc.info)

## ----memory_footprint_example-------------------------------------------------
# default return value with compact = TRUE
compact_rc <- get_region_counts(ribo.object = original.ribo, 
                                range.lower = 28,
                                range.upper = 32,
                                length      = TRUE,
                                transcript  = FALSE,
                                compact     = TRUE,
                                alias       = TRUE)
head(compact_rc)
class(compact_rc)
object.size(compact_rc)

# return value with compact = FALSE
# Note that in this example, it takes up twice as much memory
noncompact_rc <- get_region_counts(ribo.object = original.ribo, 
                                   range.lower = 28,
                                   range.upper = 32,
                                   length      = TRUE,
                                   transcript  = FALSE,
                                   compact     = FALSE,
                                   alias       = TRUE)
head(noncompact_rc)
class(noncompact_rc)
object.size(noncompact_rc)

## ----createRibo_revisited, eval = TRUE----------------------------------------
original.ribo <- Ribo(file.path, rename = rename_default )
original.ribo

## ----alias_highlight_in_metagene----------------------------------------------
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 28,
                           range.upper = 32,
                           length      = FALSE,
                           transcript  = FALSE,
                           alias       = TRUE)
meta.start[1:2 , c(1,2,3,38,39,40)]

## ----cumbersome---------------------------------------------------------------
head(get_reference_names(original.ribo), 2)

## ----def_rename_default-------------------------------------------------------
rename_default <- function(names){
  return(unlist(strsplit(names, split = "[|]"))[5])
}

rename_default("ENST00000335137.4|ENSG00000186092.6|OTTHUMG00000001094.4|-|OR4F5-201|OR4F5|1054|protein_coding|") 

## ----alias--------------------------------------------------------------------
alias.ribo    <- Ribo(file.path, rename = rename_default)
alias.ribo

## ----noalias------------------------------------------------------------------
noalias.ribo    <- Ribo(file.path)
noalias.ribo

## ----set_aliases--------------------------------------------------------------
#create a ribo file 
alias.ribo    <- Ribo(file.path)

#generate the vector of aliases
aliases <- rename_transcripts(ribo = alias.ribo, rename = rename_default)

#add aliases
alias.ribo <- set_aliases(alias.ribo, aliases)

head(aliases, 3)

alias.ribo

## ----get_info-----------------------------------------------------------------
#retrieves the experiments
original.info <- get_info(ribo.object = original.ribo)
original.info

## ----get_internal_region_coordinates------------------------------------------
region_coord <- get_internal_region_coordinates(ribo.object = alias.ribo , alias = TRUE)
head(region_coord, 2)

## ----get_internal_region_lengths----------------------------------------------
region_lengths <- get_internal_region_lengths(ribo.object = alias.ribo, alias = TRUE)
head(region_lengths, 2)

## ----get_original_region_coordinates------------------------------------------
region_coord <- get_original_region_coordinates(ribo.object = alias.ribo , alias = TRUE)
head(region_coord, 2)

## ----get_original_region_lengths----------------------------------------------
region_lengths <- get_original_region_lengths(ribo.object = alias.ribo, alias = TRUE)
head(region_lengths, 2)

## ----define_multiple_file_reader----------------------------------------------
# helper function
read_multiple_files <- function(ribo_list, getter_function, ...) {
  df_list <- lapply(ribo_list, getter_function, ...)
  df_list <- mapply(cbind, df_list, "study" = names(df_list), SIMPLIFY=FALSE)
  df <- dplyr::bind_rows(df_list)
  return (df)
}

## ----create_named_ribo_list---------------------------------------------------
# create a named Ribo object list of length >= 1
ribo_file_list <- list(original.ribo)
names(ribo_file_list) <- c("GSE65778")

# try out a function
read_multiple_files(ribo_list = ribo_file_list, getter_function = get_region_counts, compact = FALSE, tidy = FALSE)

## ----check_info_for_optional_fields-------------------------------------------
original.ribo

## ----get_metadata file, eval = FALSE------------------------------------------
# get_metadata(ribo.object = original.ribo,
#              print = TRUE)
# #The output is omitted due to its length

## ----get_metadata shortened_meta, eval = FALSE--------------------------------
# info <- capture.output(get_metadata(ribo.object = original.ribo,
#                                     print = FALSE))
# info
# #The output is omitted due to its length

## ----get_metadata has.metadata------------------------------------------------
#obtain a list of experiments with metadata 
experiment.info <- get_info(ribo.object = original.ribo)[['experiment.info']]
has.metadata <- experiment.info[experiment.info$metadata == TRUE, "experiment"]
has.metadata

#store the name of the first experiment with metadata and gets its metadata
experiment <- has.metadata[1]

get_metadata(ribo.object = original.ribo,
             name        = experiment,
             print       = TRUE)

## ----get_list_of_exps_with_coverage-------------------------------------------
#get a list of experiments that have coverage data
experiment.info <- get_info(ribo.object = original.ribo)[['experiment.info']]
has.coverage <- experiment.info[experiment.info$coverage == TRUE, "experiment"]
has.coverage

## -----------------------------------------------------------------------------
cov <- get_coverage(ribo.object = original.ribo,
                    name        = "MYC-206",
                    range.lower = 28,
                    range.upper = 32,
                    length      = TRUE,
                    alias       = TRUE,
                    tidy        = TRUE,
                    experiment  = has.coverage)
cov

## -----------------------------------------------------------------------------
#only using one experiment for this 
exp.names <- has.coverage[1]

#length is FALSE, get coverage information
#at each read length
cov <- get_coverage(ribo.object = original.ribo,
                    name        = "MYC-206",
                    range.lower = 28,
                    range.upper = 32,
                    length      = FALSE,
                    alias       = TRUE,
                    tidy        = TRUE,
                    experiment  = exp.names)

cov

## -----------------------------------------------------------------------------
#get a list of experiments that have RNA-Seq data
experiment.info <- get_info(ribo.object = original.ribo)[['experiment.info']]
has.rnaseq <- experiment.info[experiment.info$rna.seq == TRUE, "experiment"]
has.rnaseq

## -----------------------------------------------------------------------------
#get a list of experiments that have RNA-Seq data
rnaseq <- get_rnaseq(ribo.object = original.ribo,
                     tidy        = FALSE,
                     alias       = TRUE,
                     experiment  = has.rnaseq)

#print out the first 2 rows of the DataFrame
head(rnaseq, 2)

## -----------------------------------------------------------------------------
experiment <- has.rnaseq[1]
rnaseq <- get_rnaseq(ribo.object = original.ribo,
                     tidy        = TRUE,
                     alias       = TRUE,
                     region     = c("UTR5J", "CDS", "UTR3J"),
                     experiment  = experiment)
head(rnaseq, 2)

## ----rc_rnaseq_sample---------------------------------------------------------
# obtain the rnaseq and ribosomal occupancy of the CDS region
rnaseq_CDS <- get_rnaseq(ribo.object = original.ribo,
                         tidy        = TRUE,
                         alias       = TRUE,
                         region      = "CDS",
                         compact     = FALSE,
                         experiment  = "GSM1606108")


rc_CDS <- get_region_counts(ribo.object    = original.ribo,
                                tidy       = TRUE,
                                alias      = TRUE,
                                transcript = FALSE,
                                region     = "CDS",
                                compact    = FALSE,
                                experiment = "GSM1606108")

# filter out the lowly expressed transcripts  
plot(density(log2(rnaseq_CDS$count)), xlab = "Log 2 of RNA-Seq Count", main = "RNA-Seq Density")

## ----filter_and_plot----------------------------------------------------------
# filter out transcripts with count value less than 2 for either ribosome profiling region counts or rnaseq
filtered_transcripts <- which(rnaseq_CDS$count > 2 & rc_CDS$count > 2)
rnaseq_CDS           <- rnaseq_CDS[filtered_transcripts, ]
rc_CDS               <- rc_CDS[filtered_transcripts, ]
plot(log2(rnaseq_CDS$count), log2(rc_CDS$count), xlab = "RNA-Seq", ylab = "Ribosome Profiling", main = "log2 CDS Read Counts", pch = 19, cex = 0.5)

## ----translational_efficiency-------------------------------------------------
# get the translational efficiency of CDS 
te_CDS    <- rc_CDS$count/rnaseq_CDS$count
plot(density(log2(te_CDS)), xlab = "Log 2 of Translation Efficiency", main = "")

## ----filter_te----------------------------------------------------------------
# filter for translational efficiency of the observed elbow at -2 (on the log2 scale, untransformed value of .25) 
high_te <- rc_CDS$transcript[which(te_CDS > .25)]

# found 229 transcripts, first 20 shown here 
head(high_te, n = 20)

#'@include create_ribo.R
#'@include ribo_class.R
NULL

#' @rdname Ribo-class
#' @aliases Ribo-class
#' @examples 
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#' 
#' show(sample)
#' 
#' @export
setMethod(f = "show",
          signature = "Ribo",
          definition = function(object) {
              file.values <- list("format version"   = format_version(object),
                                  "reference"        = reference(object),
                                  "min read length"  = length_min(object),
                                  "max read length"  = length_max(object),
                                  "left span"        = left_span(object),
                                  "right span"       = right_span(object),
                                  "transcript count" = length(transcript_info(object)),
                                  "has.metadata"     = has_metadata(object),
                                  "metagene radius"  = metagene_radius(object),
                                  "has.alias"        = !is.empty(alias_hash(object)))
              file.info <- data.frame(info = names(file.values),
                                      " " = unlist(unname(file.values)), check.names = FALSE,
                                                         stringsAsFactors = FALSE,
                                                         fix.empty.names = FALSE)
              
              #center by reformatting the output of the data tables
              file.info   <- format_file_info(file.info)
              experiment.info <- format_experiment_info(object@experiment.info)

              print_output(file.info, experiment.info)
              invisible(object)
          })

format_experiment_info <- function(experiment.info) {
    #helper method to format the experiment info
    experiment.info <- lapply(as.list(experiment.info), as.character)
    experiment.info <- as.data.frame(experiment.info, stringsAsFactors = FALSE)

    #format in the case that the experiment names are really long
    exp.val.name <- unlist(names(experiment.info)[-1])
    exp.val      <- unlist(experiment.info[, -1])
    name.size    <- max(nchar(exp.val))
    name.size    <- max(name.size,
                        max(nchar(exp.val.name)))


    names(experiment.info)[-1] <- format(exp.val.name,
                                         width = name.size,
                                         justify = "right")

    experiment.info <-  as.data.frame(format(experiment.info,
                                             width   = name.size,
                                             justify = "right"))

    exp.title <- unlist(names(experiment.info[1]))
    exp.name  <- unlist(experiment.info[, 1])
    name.size <- max(nchar(exp.name), nchar(exp.title))

    names(experiment.info)[1] <- format(names(experiment.info)[1],
                                        width = name.size,
                                        justify = "right")

    experiment.info[, 1]      <-  format(experiment.info[, 1],
                                         width = name.size,
                                         justify = "right")
    return(experiment.info)
}

format_file_info <- function(file.info) {
    #helper method to format the file info
    name.width <- max(nchar(unname(unlist(file.info))))
    file.info   <- format(file.info,
                          justify = "right",
                          width = name.width)

    return(file.info)
}

print_output <- function (file.info,
                          experiment.info) {
    # Final helper method that prints the output of the formatted attributes 
    cat("General File Information:\n")
    print(data.frame(file.info, check.names = FALSE),
          row.names = FALSE,
          quote = FALSE)
    cat("\n")
    cat("Dataset Information:\n")
    print(data.frame(experiment.info, check.names = FALSE),
          row.names = FALSE,
          quote = FALSE)
}


setGeneric("path", function(object) standardGeneric("path"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("path", "Ribo", function(object) object@path)

setGeneric("experiments", function(object) standardGeneric("experiments"))

#' @rdname Ribo-class
#' @aliases Ribo-class
#' @param object Ribo object
#' @docType methods
#' @export
setMethod("experiments", "Ribo", function(object) object@experiments)


setGeneric("format_version", function(object) standardGeneric("format_version"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("format_version", "Ribo", function(object) object@format.version)


setGeneric("reference", function(object) standardGeneric("reference"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("reference", "Ribo", function(object) object@reference)


setGeneric("length_min", function(object) standardGeneric("length_min"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("length_min", "Ribo", function(object) object@length.min)

setGeneric("length_max", function(object) standardGeneric("length_max"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("length_max", "Ribo", function(object) object@length.max)

setGeneric("left_span", function(object) standardGeneric("left_span"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("left_span", "Ribo", function(object) object@left.span)

setGeneric("right_span", function(object) standardGeneric("right_span"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("right_span", "Ribo", function(object) object@right.span)

setGeneric("metagene_radius", function(object) standardGeneric("metagene_radius"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("metagene_radius", "Ribo", function(object) object@metagene.radius)


setGeneric("length_offset", function(object) standardGeneric("length_offset"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("length_offset", "Ribo", function(object) object@length.offset)

setGeneric("has_metadata", function(object) standardGeneric("has_metadata"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("has_metadata", "Ribo", function(object) object@has.metadata)

setGeneric("experiment_info", function(object) standardGeneric("experiment_info"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("experiment_info", "Ribo", function(object) object@experiment.info)


setGeneric("transcript_info", function(object) standardGeneric("transcript_info"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("transcript_info", "Ribo", function(object) object@transcript.info)

setGeneric("alias_hash", function(object) standardGeneric("alias_hash"))
 
#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("alias_hash", "Ribo", function(object) object@alias.hash)


setGeneric("original_hash", function(object) standardGeneric("original_hash"))

#' @rdname Ribo-class
#' @param object Ribo object
#' @aliases Ribo-class
#' @docType methods
#' @export
setMethod("original_hash", "Ribo", function(object) object@original.hash)

#' Information on the RNA-Seq data of the experiments, if any
#'
#' \code{\link{get_rnaseq}} returns a data frame containing information on the transcript name, experiment, and
#' sequence abundance
#'
#' As a default value, experiment.list is presumed to include all of the
#' experiments within a ribo file. RNA-Seq data is an optional dataset to
#' include in a .ribo file. The experiments in experiment.list are checked
#' for experiment existence in the ribo file and then checked for RNA-seq data.
#'
#' The returned DataFrame can either be in the tidy format for easier data
#' cleaning or in a condensed non-tidy format. The data will present RNA-seq counts
#' for each transcript in each valid experiment in experiment.list.
#'
#' The 'alias' parameter specifies whether or not the returned DataFrame
#' should present each transcript as an alias instead of the original name.
#' If 'alias' is set to TRUE, then the column of the transcript names will
#' contain the aliases rather than the original reference names of the .ribo
#' file.
#'
#' @examples
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #list out the experiments of interest that have RNA-Seq data
#' experiments <- c("Hela_1", "Hela_2", "WT_1")
#' regions <- c("UTR5", "CDS", "UTR3")
#' rnaseq.data <- get_rnaseq(ribo.object = sample,
#'                           tidy = TRUE,
#'                           region = regions,
#'                           experiment = experiments)
#'
#'
#' @param ribo.object A 'Ribo' object
#' @param experiment List of experiment names
#' @param alias Option to report the transcripts as aliases/nicknames
#' @param region Specific region(s) of interest
#' @param compact Option to return a DataFrame with Rle and factor as opposed to a raw data.frame
#' @param tidy Option to return the data frame in a tidy format
#' @seealso \code{\link{Ribo}} to generate the necessary ribo.object parameter
#' @return An annotated data frame containing the RNA-Seq counts for the regions in specified in the `region` parameter with the option of
#' presenting the data in a tidy format. Additionally, the function returns a DataFrame with Rle and factor applied if the `compact` parameter
#' is set to TRUE and a data.frame without any Rle or factor if the `compact` parameter is set to FALSE
#' @importFrom rhdf5 h5ls h5read
#' @importFrom S4Vectors DataFrame Rle
#' @importFrom tidyr gather
#' @export
get_rnaseq <- function(ribo.object,
                       tidy = TRUE,
                       region = c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3"),
                       experiment = experiments(ribo.object),
                       compact = TRUE,
                       alias = FALSE) {
    if (!is(ribo.object, "Ribo")) stop("Please provide a ribo object.")
    rnaseq.experiments <- check_rnaseq(ribo.object, experiment)
    check_alias(ribo.object, alias)
    regions <- check_regions(ribo.object, region)
    ribo.experiments <- experiments(ribo.object)
    
    #get just the experiments that exist
    ref.names <- change_reference_names(ribo.object, alias)
    ref.length <- length(ref.names)
    total.experiments <- length(rnaseq.experiments)
    num.regions <- length(regions)
    path <- path(ribo.object)
    
    result <- matrix(nrow = ref.length * total.experiments, ncol = num.regions)
    colnames(result) <- regions
    
    values <- c("UTR5" = 1, "UTR5J" = 2, "CDS" = 3, "UTR3J" = 4, "UTR3" = 5)
    cols   <- unname(values[regions])
    rows   <- seq_len(ref.length)
    
    #generate the untidy version
    for (index in seq(total.experiments)) {
        experiment <- rnaseq.experiments[index]
        exp_path <- paste("/experiments/", experiment, "/rnaseq/rnaseq", sep = "")
        row.start <- (index - 1) * ref.length + 1
        row.stop <- row.start + ref.length - 1
        result[row.start:row.stop,] <- t(h5read(path, exp_path, index = list(cols, rows)))
    }
    
    rnaseq <- rep(rnaseq.experiments, each = ref.length)
    transcripts <- rep(ref.names, total.experiments)
    
    result <- data.frame(experiment = rnaseq,
                         transcript = factor(transcripts),
                         result,
                         stringsAsFactors = FALSE)
    
    if (tidy) result <- gather(result, "region", "count", regions)
    if (!compact) return (result)
    
    return(prepare_DataFrame(ribo.object, as(result, "DataFrame")))
}

change_reference_names <- function(ribo.object,
                                   alias) {
    #generate appropriate ref.names in the untidy version
    ref.names <- get_reference_names(ribo.object)
    if (alias) {
        original <- ref.names
        ref.names <- vector(mode = "character", length = length(original))
        for (i in seq_len(length(original))) {
            ref.names[i] <- original_hash(ribo.object)[[original[i]]]
        }
    }
    return(ref.names)
}

check_rnaseq <- function(ribo.object, experiments) {
    #check the experiments for validity and RNA-seq presence
    check_experiments(ribo.object, experiments)
    
    #obtain the rnaseq data
    path <- path(ribo.object)
    table <- get_content_info(path)
    has.rnaseq <- table[table$rna.seq == TRUE,]$experiment
    
    #find the experiments in the experiment.list that do not have coverage and print warnings
    check <- setdiff(experiments, has.rnaseq)
    if (length(check)) {
        for (experiment in check) {
            warning("'",
                    experiment,
                    "'",
                    " did not have RNA-Seq data.",
                    call. = FALSE)
        }
        
        if (length(check) == length(experiments)) {
            stop("No valid experiments with RNA-Seq.", call. = FALSE)
        } else {
            warning("Param 'experiments' contains experiments that did not",
                    "have RNA-Seq data. The return value ignores these", 
                    "experiments.",
                    call. = FALSE)
        }
    }
    return(intersect(experiments, has.rnaseq))
}

#' Retrieves the coverage data for a given transcript
#'
#' The function \code{\link{get_coverage}} generates a DataFrame of coverage
#' data over the length of a given transcript.
#'
#' The function \code{\link{get_coverage}} first checks the experiments in the
#' 'experiments' parameter to see if they are present in the .ribo file.
#' It will then check these experiments for coverage data which is an optional
#' dataset. As a result, this function safe guards against experiments that
#' do not have coverage data, but it also, by default, includes all of the
#' experiments in a file in the experiments' parameter.
#'
#' The function checks the coverage of one transcript at a time at
#' each read length from 'range.lower' to 'range.upper', inclusive. However,
#' the parameter 'length' allows the user to obtain the coverage
#' information of a transcript across the range of read lengths indicated by
#' 'range.lower' and 'range.upper'.
#'
#' If the ribo.object is generated with aliases, the 'alias' parameter, if set to TRUE,
#' allows the user to use the alias of the transcript as the 'name' parameter instead of
#' the original transcript name.
#'
#' @examples
#' #generate the ribo object
#' file.path <- system.file("extdata", "sample.ribo", package = "ribor")
#' sample <- Ribo(file.path)
#'
#' #get the experiments of interest that also contain coverage data
#' experiments <- c("Hela_1", "Hela_2", "Hela_3", "WT_1")
#'
#' #the ribo file contains a transcript named 'MYC'
#' coverage.data <- get_coverage(ribo.object = sample,
#'                               name = "MYC",
#'                               range.lower = 2,
#'                               range.upper = 5,
#'                               length = TRUE,
#'                               experiment = experiments)
#' @param ribo.object A 'Ribo' object
#' @param name Transcript Name
#' @param range.lower Lower bound of the read length, inclusive
#' @param range.upper Upper bound of the read length, inclusive
#' @param length Logical value that denotes if the coverage should be summed across read lengths
#' @param experiment List of experiments to obtain coverage information on
#' @param tidy Logical value denoting whether or not the user wants a tidy format
#' @param alias Option to accept the transcript input as aliases/nicknames
#' @param compact Option to return a DataFrame with Rle and factor as opposed to a raw data.frame
#' @return An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
#' coverage information for the provided list of 'experiments' in the 'experiment' parameter. The
#' returned object will have a length column when the 'length' parameter is set to FALSE, indicating
#' that the user does not want to sum the count information across the range of read lengths. The
#' returned data frame has the option of being tidy, and if the 'tidy' parameter is set to TRUE,
#' a position column will be added. Finally, if the 'alias' parameter is set to TRUE, the alias transcript
#' name must have been provided at the generation of the ribo object, and the function
#' will accept this aliased name in the 'transcript' parameter.
#' @seealso \code{\link{Ribo}} to generate the necessary ribo.object parameter
#' @importFrom rhdf5 h5read
#' @importFrom tidyr gather
#' @importFrom S4Vectors Rle DataFrame
#' @importFrom methods as
#' @importFrom hash has.key
#' @export
get_coverage <- function(ribo.object,
                         name,
                         range.lower = length_min(ribo.object),
                         range.upper = length_max(ribo.object),
                         length = TRUE,
                         tidy = FALSE,
                         alias = FALSE,
                         compact = TRUE,
                         experiment = experiments(ribo.object)) {
    # check the parameters and prepare parameters for reading from ribo file
    if (missing(name)) stop("Please provide a transcript name.")
    if (!is(ribo.object, "Ribo")) stop("Please provide a ribo object.")
  
    matched.experiments <- initialize_coverage(ribo.object,
                                               alias,
                                               range.lower,
                                               range.upper,
                                               experiment)
    
    total.experiments   <- length(matched.experiments)
    info <- retrieve_transcript_info(ribo.object,
                                     name,
                                     alias)

    # read from the ribo file and generate a data.frame
    result <- fill_coverage(ribo.object,
                            total.experiments,
                            matched.experiments,
                            range.lower,
                            range.upper,
                            length,
                            alias,
                            info)
    
    # gather the data to make it tidy 
    if (tidy) {
      tidy.columns <- "experiment"
      if (!length) tidy.columns <- c("experiment", "length")
      
      result <- gather(result,
                       key = "position",
                       value = "count", 
                       -tidy.columns)
    }
    
    # convert to DataFrame with Rle encoding if necessary 
    if (compact) return(prepare_DataFrame(ribo.object, result))
    return(result)
}


fill_coverage <- function(ribo.object,
                          total.experiments,
                          matched.experiments,
                          range.lower,
                          range.upper,
                          length,
                          alias,
                          info) {
    current.offset      <- info[1]
    transcript.length   <- info[2]
    length.offset       <- length_offset(ribo.object)
    min.length          <- length_min(ribo.object)
    read.range          <- range.upper - range.lower + 1

    result <- initialize_matrix_coverage(total.experiments,
                                         transcript.length,
                                         read.range,
                                         length)
    #fills in the entries of the matrix
    for (i in seq(total.experiments)) {
        #for the given experiment, get its coverage
        experiment <- matched.experiments[i]
        path <- get_dataset_path(experiment, "coverage")
        #get the coverage information for the specific transcript at each read length
        for (current.length in range.lower:range.upper) {
            correct.length <- 1 + (current.length - min.length) * length.offset
            coverage.start <- correct.length + current.offset
            coverage.stop  <- coverage.start + transcript.length - 1
            coverage <- t(h5read(path(ribo.object),
                                 path,
                                 index = list(coverage.start:coverage.stop)))
            coverage <- as.integer(coverage)

            # compute the correct offset to store the coverage information in each case
            if (length) {
                result[i,] <- result[i,] + coverage
            } else {
                current.experiment     <- (i - 1) * read.range
                current.read           <- current.length - range.lower
                current.index          <- current.experiment + current.read + 1
                result[current.index,] <- coverage
            }
        }
    }
    matched.size <- length(matched.experiments)
    if (length) {
      return (data.frame(experiment = matched.experiments,
                         result, check.names = FALSE))
    }
    range <- range.upper - range.lower + 1
    return (data.frame(experiment  = rep(matched.experiments, each = range),
                       length      = rep(c(range.lower:range.upper), matched.size),
                       result, 
                       check.names = FALSE))
    
}

get_dataset_path <- function(experiment,
                             dataset) {
  return(paste("/experiments/",
               experiment,
               "/", dataset, "/", dataset, sep =""))
}

initialize_coverage <- function(ribo.object,
                                alias,
                                range.lower,
                                range.upper,
                                experiment) {
    #perform checks on the parameters
    check_alias(ribo.object, alias)
    check_lengths(ribo.object, range.lower, range.upper)
    # check_experiments(ribo.object, experiments)
    ribo.experiments    <- experiments(ribo.object)
    
    #generate list of experiments also present in the ribo file that have coverage data
    filter.experiments  <- intersect(experiment, ribo.experiments)
    matched.experiments <- check_coverage(ribo.object, filter.experiments)
    matched.experiments <- intersect(matched.experiments, filter.experiments)

    return(matched.experiments)
}


retrieve_transcript_info <- function(ribo.object,
                                     name,
                                     alias) {
    #helper method that checks the alias parameter and retrieves the corresponding information
    if (alias) {
        if (!has.key(name, alias_hash(ribo.object))) {
            stop("Alias name was not found.", call. = FALSE)
        }
        name <- alias_hash(ribo.object)[[name]]
    }

    #generate offsets
    info <- as.vector(unlist(transcript_info(ribo.object)[[name]]))
    if (is.null(info)) {
        stop("Transcript name was not found. Check name and 'alias' parameter.",
             call. = FALSE)
    }
    return(info)
}

initialize_matrix_coverage <- function(total.experiments,
                                       transcript.length,
                                       read.range,
                                       length) {
    #create matrix of correct size based on param transcript
    result <- matrix()
    if (length) {
        result <- matrix(0L,
                         nrow = 1 * total.experiments,
                         ncol = transcript.length)
    } else {
        result <- matrix(nrow = read.range * total.experiments,
                         ncol = transcript.length)
    }
    colnames(result) <- as.character(seq_len(transcript.length))
    return(result)
}


create_dataframe <- function (length,
                              range.lower,
                              range.upper,
                              matched.experiments,
                              compact,
                              matrix) {
    # Given a matrix of coverage data, create_dataframe generates the correct
    # DataFrame based on a set of parameters
    #
    # Returns:
    # Data table that wraps the matrix with the correct and appropriate labels

    matched.size <- length(matched.experiments)
    if (length) {
        return (DataFrame(experiment = Rle(factor(matched.experiments)),
                          matrix, check.names = FALSE))
    }
    range <- range.upper - range.lower + 1
    return (DataFrame(
        experiment  = Rle(factor(rep(matched.experiments, each = range))),
        length = Rle(factor(rep(c(range.lower:range.upper), matched.size))),
        matrix, check.names = FALSE
    ))
}


check_coverage <- function(ribo.object, experiment) {
    # helper function that both generates a list of experiments with coverage
    # data using get_info and produces a warning message for each experiment
    # (provided by the user) that does not have coverage data
    #
    # Args:
    # ribo.object: S3 object of class "Ribo"
    # experiment.list: list of experiments inputted by the user
    #
    # Returns:
    # A list of experiments in the ribo.object that have coverage data

    path <- path(ribo.object)
    #obtain the coverage data
    table <- get_content_info(path)
    has.coverage <- table[table$coverage == TRUE,]
    has.coverage <- has.coverage$experiment

    #find the experiments in the experiment.list that do not have
    #coverage and print warnings
    check <- setdiff(experiment, has.coverage)
    if (length(check)) {
        for (exp in check) {
            warning("'", exp, "'",
                    " did not have coverage data.",
                    call. = FALSE)
        }
        warning(
            "Param 'experiments' contains experiment names that did not have coverage data.",
            "The return value ignores these experiments.",
            call. = FALSE
        )
    }

    #return a list of experiments with coverage
    return(has.coverage)
}

---
title: "A Walk-through of RiboR"
author: "Michael Geng, Hakan Ozadam, Can Cenik"
output:
  html_document:
    theme: lumen
    toc: yes
    toc_depth: 4
    toc_float: yes
  pdf_document:
    toc: yes
    toc_depth: 4
  md_document:
    variant: markdown_github
  rmarkdown:: html_vignette
  word_document:
    toc: yes
    toc_depth: '4'
vignette: |
  %\VignetteIndexEntry{A Walkthrough of RiboR}    
  %\VignetteEngine{knitr::rmarkdown}  
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```
## Introduction

[Ribosome profiling](https://www.nature.com/articles/nrg3645) 
is a sequencing based method to study protein synthesis transcriptome-wide.
Actively translating mRNAs are engaged with ribosomes and protein synthesis rates can be approximated by the number of ribosomes that are translating a given mRNA. 
Ribosome profiling employs an RNase digestion step to recover fragments of RNA protected by ribosomes which are called __Ribosome Protected Footprints (RPFs)__.

Ribosome profiling data analyses involve several quantifications for each transcript. Specifically, the lengths of the RPFs provide valuable biological information (see, for example, [Lareau et al.](https://elifesciences.org/articles/01257) and [Wu et al.](https://www.sciencedirect.com/science/article/pii/S1097276518310633?via%3Dihub)). To facilitate ribosome profiling data analyses as a function of RPF length in a highly efficient manner, we implemented a new data format, called [ribo](https://ribopy.readthedocs.io/en/latest/ribo_file_format.html). 
Files in ribo format are called __.ribo__ files. The associated paper for the .ribo file and its larger ecosystem can be found [here](https://academic.oup.com/bioinformatics/advance-article/doi/10.1093/bioinformatics/btaa028/5701654).

RiboR package is an R interface for .ribo files. The package offers a suite of reading functions for .ribo files, and provides plotting functions that are most often employed in ribosome profiling analyses. Using RiboR, one can import .ribo files into the R environment, read ribosome profiling data into data frames and generate essential plots in a few lines of R code. 

This document is structured into several sections. First, we give an overview of the 
[Ribo File format](#ribofileformat) and define [transcript regions](#transcriptregions).
Second, we provide instructions and requirements for the [installation](installation) of RiboR.
Third, we describe how to import a .ribo file to R environment and demonstrate essential ribosome profiling data analyses in three sections:

   * [Length Distribution](#lengthdistribution)
   * [Metagene Analysis](#metageneanalysis)
   * [Region Counts](#regioncounts)
   
In the following two sections, we describe some [advanced features](#advancedfeatures)
including renaming transcripts using [aliases](#alias), accessing .ribo [file attributes](#fileattributes), and getting [region boundaries](#annotation) that define transcript regions.  
In the [last section](#optionaldata), we explain the three optional types of data, which may exist in a .ribo file.

* [Metadata](#metadata): A .ribo file may contain metadata for each experiment and for the .ribo file itself.
* [Coverage](#coverage): A .ribo file can keep the nucleotide level transcriptome coverage.
* [RNA-Seq](#rnaseq): A ribosome profiling experiment can be paired with an RNA-Seq experiment to study ribosome occupancy together with transcript abundance. 

In the final section, we outline a simple analysis to study translational efficiency.

### .ribo File Format {#ribofileformat}

.ribo files are built on an [HDF5](https://www.hdfgroup.org/solutions/hdf5/) architecture and has a predefined internal structure (Figure 1). For a more detailed explanation of the ribo format, we refer to 
[readthedocs page of ribo](https://ribopy.readthedocs.io/en/latest/ribo_file_format.html).

```{r pressure, echo=FALSE, fig.cap="Figure 1: .ribo File Internal Structure", out.width = '90%', fig.align="center"}
knitr::include_graphics("ribo_file_structure.jpg")
```

While many features are required in .ribo files, quantification from paired RNA-Seq data and nucleotide-level coverage are optional. 

### Transcript Regions {#transcriptregions}

The main protein coding region of a transcript is called the coding sequence (CDS). Its boundaries are called transcription start / stop sites. The region consisting of the nucleotides, between the 5’ end of the transcript and the start site, not translated to protein, is called 5’ untranslated region (5’UTR). Similarly, the region having the nucleotides between the stop site and the 3’ end of the transcript, is called 3’ untranslated region (3’UTR). To avoid strings and variable names starting with a number and containing an apostrophe character, we use the names UTR5 and UTR3 instead of 5’UTR and 3’UTR respectively.

## Installation {#installation}

### Availability
The source code of RiboR package is available in a [public Github repository](https://github.com/ribosomeprofiling/ribor).

### Bioconductor
RiboR is a [Bioconductor](https://www.bioconductor.org) package. 
The following R code installs RiboR via Bioconductor.
```{r install_bioconductor, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("ribor")
```

### Installation From Source Code

RiboR can also be installed directly using the source code.
For this, the following are required.

* **R:** RiboR requires **version 3.6 or later**. Previous R versions are known to cause installation and run time problems.

* **Devtools:** Installing the package requires `devtools`, a CRAN package that provides a wide suite of development tools in the R environment. 

To install the package, run the following lines of code.
```{r install_from_github, eval = FALSE}
install.packages("devtools")
library("devtools")
install_github("ribosomeprofiling/ribor")
```

For Linux distributions, there are some dependencies, required by devtools. 
Installation of these dependencies is necessary to install devtools.
For Ubuntu-based distributions, the following command installs these dependencies.

```{bash install_devtools_dep, eval = FALSE}
sudo apt-get install libxml2-dev libcurl4-openssl-dev libssl-dev -y
```

## Getting Started {#gettingstarted}
We start with loading the RiboR package.
``` {r import_lib, eval = TRUE}
library(ribor)
```
#### Generating a ribo object
To interact with a .ribo file, it is necessary to create a ribo object that provides a direct handle to the file and displays its various attributes. As an example, we processed publicly available ribosome profiling data (GEO accession number [GSE65778](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE65778)) from HEK293 cells using [RiboFlow](https://github.com/ribosomeprofiling/riboflow) pipeline to generate the .ribo file in this document. More precisely, we picked two ribosome profiling experiments coming from untreated HEK293 cells (accession numbers: GSM1606107 and GSM1606108) and two RNA-Seq experiments coming from the same line of untreated cells (accession numbers: GSM1606099 and GSM1606100). Each ribo experiment can be paired with a single RNA-Seq experiment. So we arbitrarily paired the ribosome profiling experiment GSM1606107 with RNA-Seq experiment GSM1606099 and GSM1606108 with GSM1606100, when making the example .ribo file in this document. 

```{r createRibo}
#file path to the example ribo file 
file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")

#generates the 'ribo' class object 
original.ribo <- Ribo(file.path, rename = rename_default )
```

**Do not modify the values of the returned ribo object.**
Certain values in the `ribo` object's list are used by later functions, so manually changing these values can alter the correctness of later function calls.

Once the ribo object is created, we can inquire about the contents of the .ribo file.

```{r}
original.ribo
```

The above output provides information about the individual experiments that are contained in the given ribo object. In addition, this output displays some of the parameters, that were used in generating the .ribo file, such as left span, right span and metagene radius.

For a detailed explanation of the contents of this output, we refer to the 
[online documentation of the ribo format](https://ribopy.readthedocs.io/en/latest/ribo_file_format.html). 

In what follows, we demonstrate a typical exploration of ribosome profiling data.
We start with the length distribution of RPFs.

## Length Distribution {#lengthdistribution}
Several experimental decisions including the choice of RNase can have a significant impact on the RPF length distribution. In addition, this information is generally informative about the quality of the ribosome profiling data.  

We use the function `plot_length_distribution` to generate the distribution of the reads mapping to a specific region. This function has also a boolean argument called `fraction`. When fraction is `FALSE`, the y-axis displays the total number of reads mapping to the specified region. When fraction is `TRUE`, the y-axis displays the quotient of the same number as above divided by the total number of reads reported in the experiment.

The following code plots the coding region mapping RPF length distribution in the range of 28 to 32 nucleotides. 

``` {r plot_length_distribution ribo, fig.width = 7}
plot_length_distribution(x           = original.ribo,
                         region      = "CDS",
                         range.lower = 28,
                         range.upper = 32,
                         fraction    = TRUE)
```

We can also plot the absolute number of reads instead of the fraction of total reads by changing the argument `fraction = FALSE`.

``` {r plot_length_distribution_UTR5 ribo, fig.width = 7}
plot_length_distribution(x           = original.ribo,
                         region      = "CDS",
                         range.lower = 28,
                         range.upper = 32,
                         fraction    = FALSE)
```

We can extract the numerical values used to produce the above plot using the function `get_length_distribution` as follows.
The parameters of this function will be explained in more detail later in this document.

``` {r get_length_dist_basic}
rc <- get_length_distribution(ribo.object      = original.ribo,
                                   region      = "CDS",
                                   range.lower = 28,
                                   range.upper = 32)
rc
```

It is also possible to generate the above plot from the DataFrame returned by `get_region_counts`.
This can be useful if the user wants to further process the data prior to plotting.

```{r plot_length_distribution_from_dataframe, eval = FALSE}
# further DataFrame manipulation and filtering can ensue here 
# before calling the plot function

plot_length_distribution(rc, fraction = TRUE)
```


## Metagene Analysis {#metageneanalysis}
A common quality control step in ribosome profiling analyses is the inspection of the pileup of sequencing reads with respect to the start and stop site of annotated coding regions. Given that ribosomes are predominantly translating annotated coding regions, these plots are informative about the enrichment at the boundaries of coding regions and also provide information regarding the periodicity of aligned sequencing reads. This type of plot is called a metagene plot as the reads are aggregated around translation start and stop sites across all transcripts.

The parameter “metagene radius” is the number of nucleotides surrounding the start/stop site and hence defines the region of analysis. For each position, read counts are aggregated across transcripts. This cumulative read coverage (y-axis) is plotted as a function of the position relative to the start/stop site (x-axis).

We can identify the metagene radius of .ribo file using the `get_info` function.
``` {r learn_metagene_radius}
get_info(original.ribo)$attributes$metagene_radius
```

We can plot the ribosome occupancy around the start or stop sites using `plot_metagene`. The following code produces the metagene plot at the start site for the experiment GSM1606107. The values on the y-axis are the raw read counts.

```{r plot_metagene ribo_start, fig.width = 7}
plot_metagene(original.ribo,
              site        = "start",
              experiment  = c("GSM1606107"),
              range.lower = 28,
              range.upper = 32)
```

If we don’t provide a specific value for the the experiments parameter, all available experiments will be plotted. 
To better compare these experiments, we can normalize the coverage by setting `normalize = TRUE`. 
In the following example, in addition to these parameters, we set `site = "stop"` to see the stop site coverage. 

```{r plot_metagene ribo, fig.width = 7}
plot_metagene(original.ribo,
              site        = "stop",
              normalize   = TRUE,
              title       = "Stop Site Coverage",
              range.lower = 28,
              range.upper = 32)
```

There are two functions that allow the user to obtain metagene coverage data. 

* `get_metagene`
     * This function returns a DataFrame of metagene coverage.
     
* `get_tidy_metagene`
     * This function returns a tidy DataFrame of metagene coverage. It is a wrapper of the `get_metagene` function, and in order to keep manageable DataFrame dimensions, it will always sum across the transcripts.

When using `length = TRUE` and `transcript = TRUE`, 
the user will obtain the most concise data corresponding to the sum of reads 
surrounding the start / stop site across all read lengths and transcripts. 
As a result, one row of values will be reported for each experiment.

```{r}
#get the start site across read lengths 28 to 32
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 28,
                           range.upper = 32,
                           length      = TRUE,
                           transcript  = TRUE)
#only print first ten columns 
print(meta.start[ , 1:10])
```

To obtain the tidy version of the above DataFrame,

```{r}
tidy.meta.start <- get_tidy_metagene(ribo.object = original.ribo,
                                     site        = "start",
                                     range.lower = 28,
                                     range.upper = 32,
                                     length      = TRUE)
head(tidy.meta.start, 2)
```

To maintain the counts at each individual read length summed across transcripts, use `length = FALSE` and 
`transcript = TRUE`. If the metagene data of a single read length, say 30, is needed, set `range.lower` and `range.upper` to 30. By default, data retrieval functions return data from all available experiments. To obtain data from a particular subset of experiments, provide a list of experiment names in the `experiments` parameter.

```{r}
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 30,
                           range.upper = 30,
                           experiment  = c("GSM1606108"),
                           length      = FALSE,
                           transcript  = TRUE)

print(meta.start[, 1:10])
```

If we wish to preserve the read counts for individual transcript sum across a range of read lengths, we set `transcript = FALSE` and `length = TRUE`. While it is possible to set `transcript = FALSE` and `length = FALSE`,
run times might be slower for this option, 
and running the function with these options requires a substantial amount of memory.

```{r get_metagene_tr_F_len_T}
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 28,
                           range.upper = 32,
                           length      = TRUE,
                           transcript  = FALSE,
                           alias       = TRUE)

print(meta.start[1:2, 1:10])
```


As we discussed above for read length analysis, we also provide the option to generate metagene plots using the DataFrames returned by `get_metagene`.
``` {r plot_metagene_from_dt, fig.width = 7}
plot_metagene(tidy.meta.start, normalize = TRUE)
```


### A Note on Aggregating the Output

In the above function calls, we used two parameters `length` and `transcript`, 
which are used, in general, by reading and plotting functions in RiboR. 
These parameters determine how data is aggregated. 

More precisely:

* __length__:
    * If `TRUE`, the counts will be summed up across the given read length interval. Otherwise, if length is `FALSE`, the counts at each individual read length will be included separately in the output.
    
* __transcript__:
    * If `TRUE`, the counts will be summed up across all the transcripts. Otherwise, if transcript is `FALSE`, the counts at each individual transcript will be reported separately.


## Region Counts {#regioncounts}

Another important aspect of ribosome profiling data is the number of reads mapping to the different regions of the transcripts, namely, 5'UTR, CDS and 3'UTR. A large number of reads mapping to UTR5 or UTR3 regions might indicate a poor quality ribosome profiling data since ribosomes occupy CDS. Furthermore, the distribution of reads across these regions can be associated with the RNase choice in the experiment. For example in [Miettinen and Bjorklund](https://academic.oup.com/nar/article/43/2/1019/2414322), it was shown that ribosome profiling experiments are dependent on digestion conditions.

Before going into the R functions, we briefly explain how region counts are computed, introduce our naming convention and define the regions used in ribo format.

For each read mapped to the transcriptome, we take the first nucleotide on the 5' end of the read and determine
the corresponding region. After doing this for all reads, the accumulated values give us the region counts. 

As mentioned earlier, a messenger RNA transcript is partitioned into three regions: 5'UTR, CDS and 3'UTR.
For technical reasons, we rename 5'UTR as UTR5 and 3'UTR as UTR3.

It is well-known that ribosomes pause, or move slower, around start and stop sites. As a result, we observe peaks around start and stop sites in metagene plots. This behavior of ribosome makes it harder to perform certain analyses such as coverage, translation efficiency, periodicity and uniformity analysis with accuracy. 
To tackle this problem, we introduce two additional regions called __UTR5 junction__ and __UTR3 junction__,
and modify the definition of the regions UTR5, CDS and UTR3 as shown in Figure 2. 
This way, we keep regions around start and stop sites separate when doing such analyses.

```{r region_definitions, echo=FALSE, fig.cap="Figure 2: Extended Region Definitions", out.width = '90%', fig.align="center"}
knitr::include_graphics("region_definition.jpg")
```

More precisely, first, we fix two integers: __left span__ (__l__) and
__right span__ (__r__) and define the __junction__ regions as follows.

* __UTR5 junction:__ This region consists of __l__ nucleotides to the left of the **start** site
   , and __r__ nucleotides to the right of the **start** site.
   
* __UTR3 junction:__ This region consists of __l__ nucleotides to the left of the **stop** site
   , and __r__ nucleotides to the right of the **stop** site.
   
Using these junction regions, we re-define the conventional regions as follows.   
   
* __UTR5__: This region is the set of nucleotides between the 5' end of the transcript and the UTR5 junction.

* __CDS__:  This region is the set of nucleotides between the UTR5 junction and UTR3 junction.

* __UTR3__: This region is the set of nucleotides between the UTR3 junction and the 3' end of the transcript.

Similar to the `get_metagene` function, the `get_region_counts` function has the `transcript` and `length` parameters. As previously mentioned, `transcript` specifies whether or not to sum across the transcripts, and `length` specifies whether or not to sum across the read lengths. 

The following code will plot the number of sequencing reads whose 5’ ends map to the UTR5, CDS, and UTR3 as a stacked bar plot. To facilitate comparison between experiments, the percentage of the regions counts are plotted and the percentage of reads mapping to CDS are printed on the plot.

``` {r plot_region_counts ribo, fig.width = 7}
plot_region_counts(x           = original.ribo,
                   range.lower = 28,
                   range.upper = 32)
```

To get the region counts summed across both lengths and transcripts, set `length = TRUE` and `transcript = TRUE`.

``` {r get_region_counts default}
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        length      = TRUE,
                        transcript  = TRUE,
                        region      = c("CDS"))
rc
```

When presented with the option of preserving the region counts at each individual read length, it may be preferable to present the transcript names as their shortened aliases.

To get the data only summed across the read lengths, set `length = TRUE` and `transcript = FALSE`. Note that the `alias = TRUE` in this case, and instead of `original.ribo`, we are using `alias.ribo`.

``` {r get_region_counts transcript}
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        length      = TRUE,
                        transcript  = FALSE,
                        alias       = TRUE,
                        region      = c("CDS"))
rc
```

To get the data in its full form, preserving the information each individual read length and transcript, set `length = FALSE` and `transcript = FALSE`.

``` {r get_region_counts full}
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        length      = FALSE,
                        transcript  = FALSE,
                        alias       = TRUE,
                        region      = c("CDS"))
rc
```

By default, the DataFrame output is in a tidy form. If non-tidy data set is desired, then set `tidy = FALSE`.

``` {r get_region_counts non-tidy}
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        tidy        = FALSE)
rc
```

Also, by default, the counts are not normalized, but there is an option to normalize the region counts. Set `normalize = TRUE` to instead obtain the normalized counts per million reads.

``` {r get_region_counts normalize}
rc <- get_region_counts(original.ribo,
                        range.lower = 28,
                        range.upper = 32,
                        normalize   = TRUE,
                        region      = c("CDS"))
rc
```

Similar to the case of metagene and length distribution, the bar plot above can be generated directly from a DataFrame.

``` {r plot_region_counts dt, fig.width = 7}
rc.info <- get_region_counts(ribo.object = original.ribo,
                             region      = c("UTR5", "CDS", "UTR3"),
                             range.lower = 28,
                             range.upper = 32)

# further data frame manipulation and filtering can ensue here 
# before calling the plot function 

plot_region_counts(rc.info)
```


### A Note on the Memory Footprint

In the above function calls, the `compact` parameter is set to TRUE by default, and this parameter is available for all of the offered reading functions. When set to TRUE, it applies Rle (run-length encoding) and factor to several of the columns (notably the columns with repeated values such as experiment, length, and region), and the purpose of this is to reduce the memory footprint of the return values and better accomodate any range of devices using the package.

``` {r memory_footprint_example}
# default return value with compact = TRUE
compact_rc <- get_region_counts(ribo.object = original.ribo, 
                                range.lower = 28,
                                range.upper = 32,
                                length      = TRUE,
                                transcript  = FALSE,
                                compact     = TRUE,
                                alias       = TRUE)
head(compact_rc)
class(compact_rc)
object.size(compact_rc)

# return value with compact = FALSE
# Note that in this example, it takes up twice as much memory
noncompact_rc <- get_region_counts(ribo.object = original.ribo, 
                                   range.lower = 28,
                                   range.upper = 32,
                                   length      = TRUE,
                                   transcript  = FALSE,
                                   compact     = FALSE,
                                   alias       = TRUE)
head(noncompact_rc)
class(noncompact_rc)
object.size(noncompact_rc)
```

If `compact` is FALSE, the called function will return a data.frame instead of a DataFrame, and it will strip the Rle and factor that is applied to several of columns, resulting in a notably larger memory footprint. However, this makes it easier to manipulate and filter the data frame for any downstream analyses, and the data.frame is more readily compatible with other popular packages (i.e. dplyr, tidyr, and ggplot2).


## Advanced Features {#advancedfeatures}

### Alias {#alias}

In the beginning, when we created the ribo object, we used an optional parameter `rename = rename_default`.

```{r createRibo_revisited, eval = TRUE}
original.ribo <- Ribo(file.path, rename = rename_default )
original.ribo
```

Note that `has.alias` attribute is `TRUE` in the above output. 
When the ribo object is created with the `rename` parameter,
providing alias allows us to rename transcripts, by explicitly setting `alias = TRUE`.

```{r alias_highlight_in_metagene}
meta.start <- get_metagene(ribo.object = original.ribo, 
                           site        = "start",
                           range.lower = 28,
                           range.upper = 32,
                           length      = FALSE,
                           transcript  = FALSE,
                           alias       = TRUE)
meta.start[1:2 , c(1,2,3,38,39,40)]
```

The transcriptome reference, that we used to generate this .ribo file, is from [GENCODE](https://www.gencodegenes.org). In particular, transcript names in this reference are long for the sake of completeness. For example, they include Ensemble transcript and gene ids in addition to standard gene symbols (HGNC).

``` {r cumbersome}
head(get_reference_names(original.ribo), 2)
```

A user might want to work with simplified ids for convenience. To provide this functionality, an optional parameter called `rename` can be provided when initializing a ribo object as shown above.

`rename` parameter can be used in two major ways. The first option is providing a renaming function. This is often a short piece of code that parses the original name and returns a shorter gene, or transcript, name. The RiboR package provides a default renaming function, named `rename_default` for the appris human transcriptome. However, an alternative custom renaming function can also be supplied for example if using a different transcriptome annotation. 

The default renaming function extracts the fifth entry in the transcript name, separated by “|”.


```{r def_rename_default}
rename_default <- function(names){
  return(unlist(strsplit(names, split = "[|]"))[5])
}

rename_default("ENST00000335137.4|ENSG00000186092.6|OTTHUMG00000001094.4|-|OR4F5-201|OR4F5|1054|protein_coding|") 
```

Using the above function, the following code will generate a ribo object and assign an alias to all of the transcripts in a .ribo file.

``` {r alias}
alias.ribo    <- Ribo(file.path, rename = rename_default)
alias.ribo
```

Notice that when `rename` is not provided, the `alias` attribute will be false.


``` {r noalias}
noalias.ribo    <- Ribo(file.path)
noalias.ribo
```

We also provide a second option of supplying a string **vector** containing the alias names directly. The string vector should correspond to the order of reference names in the output of get_reference_names. A subsequent call to `set_aliases` can be used to add aliases to an already existing ribo object. Similar to the above use of the rename parameter, `set_aliases` can be invoked with an appropriate renaming function. 

``` {r set_aliases}
#create a ribo file 
alias.ribo    <- Ribo(file.path)

#generate the vector of aliases
aliases <- rename_transcripts(ribo = alias.ribo, rename = rename_default)

#add aliases
alias.ribo <- set_aliases(alias.ribo, aliases)

head(aliases, 3)

alias.ribo
```



### File Attributes {#fileattributes}
If we want a returned list of previously printed attributes, then the function `get_info` will return all of the attributes found in the root of the .ribo file as well as information on each of the experiments. For a detailed explanation, see [the readthedocs page of RiboPy](https://ribopy.readthedocs.io).

Note that this simply returns a named list of many of the ribo object contents. The returned list organizes the information into three separate values, has.metadata, attributes, and experiment.info. This is a more efficient alternative to simply referencing the values in the ribo object since the method reads directly from the .ribo file handle and not the downstream declared ribo object.

```{r get_info}
#retrieves the experiments
original.info <- get_info(ribo.object = original.ribo)
original.info
```


### Region Boundaries {#annotation}

A .ribo file contains the region boundary information. 
This information can be useful to compare CDS lengths of different transcripts or perform region specific analysis using coverage data.

Some transcripts might have very short regions,
compared to left or right span, in their original annotation.
In this case, when attempting to set the junction boundaries, the UTR5 and/or UTR3 junctions may take the entirety of any combination of the transcript’s original regions (5’ UTR, CDS, and 3’UTR).
Then, UTR5 or UTR3 junctions will take over their nucleotides. 
Thus, in our modified region definitions, 
the coordinates of these regions are marked as `NA` and their lengths will be 0. 

Region coordinates that include the UTR5 and/or UTR3 junctions can be obtained using `get_internal_region_coordinates`.

```{r get_internal_region_coordinates}
region_coord <- get_internal_region_coordinates(ribo.object = alias.ribo , alias = TRUE)
head(region_coord, 2)
```

Length of regions can be useful to normalize data, say by CDS length, or to compare different transcripts.
The function `get_internal_region_lengths` gives region lengths, including the UTR5 and/or UTR3 junctions, for each transcript.

```{r get_internal_region_lengths}
region_lengths <- get_internal_region_lengths(ribo.object = alias.ribo, alias = TRUE)
head(region_lengths, 2)
```


However, there is also additional functionality provided by providing the original region lengths and coordinates without the UTR5 and/or UTR3 junctions .
The functions `get_original_region_lengths` and `get_original_region_coordinates` follow similar use cases to `get_internal_region_lengths` and `get_internal_region_coordinates`, respectively, but instead exclude any UTR5 and/or UTR3 junction information.

The original region coordinates that exclude the UTR5 and/or UTR3 junctions can be obtained using `get_internal_region_coordinates`.

```{r get_original_region_coordinates}
region_coord <- get_original_region_coordinates(ribo.object = alias.ribo , alias = TRUE)
head(region_coord, 2)
```

The function `get_original_region_lengths` gives region lengths, excluding the UTR5 and/or UTR3 junctions, for each transcript.

```{r get_original_region_lengths}
region_lengths <- get_original_region_lengths(ribo.object = alias.ribo, alias = TRUE)
head(region_lengths, 2)
```


### Functionality for Multiple Ribo Files

The functions within RiboR take in a single Ribo object. However, the following helper function, `read_multiple_files`, returns an aggregated data frame, given a named list of Ribo objects and any `RiboR` reader function of interest (`get_metagene`, `get_region_counts`, `get_length_distribution`, `get_coverage`, or `get_rnaseq`) with its associated parameters.


```{r define_multiple_file_reader}
# helper function
read_multiple_files <- function(ribo_list, getter_function, ...) {
  df_list <- lapply(ribo_list, getter_function, ...)
  df_list <- mapply(cbind, df_list, "study" = names(df_list), SIMPLIFY=FALSE)
  df <- dplyr::bind_rows(df_list)
  return (df)
}
```

Note that the `compact` parameter should be set to `FALSE` and that the parameters given will be consistently used across all of the Ribo objects in the given list. This means that lists of Ribo objects containing Ribo files with different properties (e.g. Transcript Aliases, RNA-Seq, Coverage) may need to be filtered for consistency before use.

```{r create_named_ribo_list}
# create a named Ribo object list of length >= 1
ribo_file_list <- list(original.ribo)
names(ribo_file_list) <- c("GSE65778")

# try out a function
read_multiple_files(ribo_list = ribo_file_list, getter_function = get_region_counts, compact = FALSE, tidy = FALSE)
```

As seen in the above output, the names of the Ribo object list allow for a new `study` column in the resulting data frame to allow for easier filtering and processing in downstream analyses.

## Optional Data {#optionaldata}

Length distribution, metagene coverage and region counts are essential to ribosome profiling data analysis and these data exist in every .ribo file. However, for certain types of analysis, additional data might be required.

For example, periodicity and uniformity analyses require the knowledge of number of reads at each nucleotide position, aka coverage data. Another analysis, called translational efficiency, can be done when transcript abundance information is present. An example of this is seen below in the section, `Sample Analysis`. For these types of analyses, .ribo files offer two types of optional data: coverage data and RNA-Seq data.

It might be helpful to have data explaining how ribosome profiling data is collected, prepared and processed. For this, .ribo files has an additional field, called metadata, to store such data for each experiment and for the .ribo file itself.

Optional data don't necessarily exist in every .ribo file. Their existence can be checked as follows.
```{r check_info_for_optional_fields }
original.ribo
```

In the above output, we see that both of the experiments have all optional data as the values in the columns
'coverage', 'rna.seq' and 'metadata' are `TRUE`. Also the .ribo file has metadata as the 'has.metadata' attribute is TRUE. 



### Metadata {#metadata}

A .ribo file can contain metadata for each individual experiment as well as the ribo file itself. If we want to see the metadata of a given experiment, then we can use the `get_metadata` function and specify the experiment of interest. 

To view the metadata of the .ribo file, we use the `get_metadata` function.

```{r get_metadata file, eval = FALSE}
get_metadata(ribo.object = original.ribo, 
             print = TRUE)
#The output is omitted due to its length
```

Metadata can be saved in a variable for later use by setting `print = FALSE`.

```{r get_metadata shortened_meta, eval = FALSE}
info <- capture.output(get_metadata(ribo.object = original.ribo, 
                                    print = FALSE))
info
#The output is omitted due to its length
```

To find the experiments that have metadata and subsequently retrieve it from one of the experiments, consider the following.

```{r get_metadata has.metadata}
#obtain a list of experiments with metadata 
experiment.info <- get_info(ribo.object = original.ribo)[['experiment.info']]
has.metadata <- experiment.info[experiment.info$metadata == TRUE, "experiment"]
has.metadata

#store the name of the first experiment with metadata and gets its metadata
experiment <- has.metadata[1]

get_metadata(ribo.object = original.ribo,
             name        = experiment,
             print       = TRUE)
```
 
### Coverage {#coverage}

For all quantifications, we first map the sequencing reads to the transcriptome and use the  5’ most nucleotide of each mapped read. Coverage data is the total number of reads whose 5’ends map to each nucleotide position in the transcriptome.

Within a .ribo file, the coverage data, if exists, is typically the largest data set in terms of storage, and it accounts for a substantial portion of a .ribo file’s size, when present. The `get_coverage` function returns the coverage information for one specific transcript at a time.

Since coverage data is an optional field of .ribo files, it is helpful to keep track of the experiment names with coverage data. Once the list is obtained, the experiments of interest can easily be chosen and extracted.


``` {r get_list_of_exps_with_coverage}
#get a list of experiments that have coverage data
experiment.info <- get_info(ribo.object = original.ribo)[['experiment.info']]
has.coverage <- experiment.info[experiment.info$coverage == TRUE, "experiment"]
has.coverage
```

Because `get_coverage` takes the transcript name, for annotations with long transcriptome names, it is **recommended** to use aliases and generate a ribo object with a valid `rename` parameter.

The `get_coverage` function retrieves  the coverage across any subset of the experiments. If `length` parameter is `TRUE`, then the coverage counts are summed up across the read lengths.


```{r}
cov <- get_coverage(ribo.object = original.ribo,
                    name        = "MYC-206",
                    range.lower = 28,
                    range.upper = 32,
                    length      = TRUE,
                    alias       = TRUE,
                    tidy        = TRUE,
                    experiment  = has.coverage)
cov
```

To preserve the information at ribosome foot print length, set `length = FALSE`.
```{r}
#only using one experiment for this 
exp.names <- has.coverage[1]

#length is FALSE, get coverage information
#at each read length
cov <- get_coverage(ribo.object = original.ribo,
                    name        = "MYC-206",
                    range.lower = 28,
                    range.upper = 32,
                    length      = FALSE,
                    alias       = TRUE,
                    tidy        = TRUE,
                    experiment  = exp.names)

cov
```

### RNA-Seq {#rnaseq}

Most ribosome profiling experiments generate matched RNA-Seq data to enable analyses of translation efficiency. We provide the ability to store RNA-Seq quantification in .ribo files to facilitate these analyses. We store RNA-seq quantifications in a manner that parallel the region counts for the ribosome profiling experiment. Specifically, the RNA-Seq data sets contain information on the relative abundance of each transcript at each of the following transcript regions.

* 5' Untranslated Region (UTR5)
* 5' Untranslated Region Junction (UTR5J)
* Coding Sequence (CDS)
* 3' Untranslated Region Junction (UTR3J)
* 3' Untranslated Region (UTR3)

To get started, it might be helpful to store a character vector of the experiment names with RNA-Seq data.

``` {r}
#get a list of experiments that have RNA-Seq data
experiment.info <- get_info(ribo.object = original.ribo)[['experiment.info']]
has.rnaseq <- experiment.info[experiment.info$rna.seq == TRUE, "experiment"]
has.rnaseq
```

To get all of the RNA-Seq data in the sample .ribo file in a non-tidy format, set `tidy = FALSE`.  

``` {r}
#get a list of experiments that have RNA-Seq data
rnaseq <- get_rnaseq(ribo.object = original.ribo,
                     tidy        = FALSE,
                     alias       = TRUE,
                     experiment  = has.rnaseq)

#print out the first 2 rows of the DataFrame
head(rnaseq, 2)
```

To get the the RNA-Seq data of the experiment at the `UTR5J`, `CDS`, and `UTR3J` regions only, in a tidy format, set `regions = c("UTR5J", "CDS", "UTR3J")` and `tidy = TRUE`.

```{r}
experiment <- has.rnaseq[1]
rnaseq <- get_rnaseq(ribo.object = original.ribo,
                     tidy        = TRUE,
                     alias       = TRUE,
                     region     = c("UTR5J", "CDS", "UTR3J"),
                     experiment  = experiment)
head(rnaseq, 2)
```

## Sample Analysis  

Here, we outline a  simple analysis highlighting some capabilities of our package.

A common use case for the RNA-Seq data (example seen in the `RNA-Seq`section in `Optional Data`) is to measure translational efficiency. We first filter out poorly expressed transcripts, and visualize the relationship between the RNA-Seq and ribosome profiling region counts.

``` {r rc_rnaseq_sample}
# obtain the rnaseq and ribosomal occupancy of the CDS region
rnaseq_CDS <- get_rnaseq(ribo.object = original.ribo,
                         tidy        = TRUE,
                         alias       = TRUE,
                         region      = "CDS",
                         compact     = FALSE,
                         experiment  = "GSM1606108")


rc_CDS <- get_region_counts(ribo.object    = original.ribo,
                                tidy       = TRUE,
                                alias      = TRUE,
                                transcript = FALSE,
                                region     = "CDS",
                                compact    = FALSE,
                                experiment = "GSM1606108")

# filter out the lowly expressed transcripts  
plot(density(log2(rnaseq_CDS$count)), xlab = "Log 2 of RNA-Seq Count", main = "RNA-Seq Density")
```

From the above plot, we decided to use a read count threshold of 2 for both the RNA-Seq and ribosome profiling region counts. We then plot the relationship between ribosome profiling and RNA-Seq counts.

``` {r filter_and_plot}
# filter out transcripts with count value less than 2 for either ribosome profiling region counts or rnaseq
filtered_transcripts <- which(rnaseq_CDS$count > 2 & rc_CDS$count > 2)
rnaseq_CDS           <- rnaseq_CDS[filtered_transcripts, ]
rc_CDS               <- rc_CDS[filtered_transcripts, ]
plot(log2(rnaseq_CDS$count), log2(rc_CDS$count), xlab = "RNA-Seq", ylab = "Ribosome Profiling", main = "log2 CDS Read Counts", pch = 19, cex = 0.5)
```

Next, we calculate the ratio of ribosome profiling to RNA-Seq counts. This ratio has been used as a rough proxy for translational efficiency in the literature. We then visually inspect the distribution of this ratio.

``` {r translational_efficiency}
# get the translational efficiency of CDS 
te_CDS    <- rc_CDS$count/rnaseq_CDS$count
plot(density(log2(te_CDS)), xlab = "Log 2 of Translation Efficiency", main = "")
```

Using the above density distribution, we next select genes with a log2 ratio of greater than -2. From this threshold value, we identify 229 transcripts that exhibit relatively high translational efficiency.

```{r filter_te}
# filter for translational efficiency of the observed elbow at -2 (on the log2 scale, untransformed value of .25) 
high_te <- rc_CDS$transcript[which(te_CDS > .25)]

# found 229 transcripts, first 20 shown here 
head(high_te, n = 20)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info_functions.R
\name{get_metadata}
\alias{get_metadata}
\title{Retrieves the metadata of an experiment}
\usage{
get_metadata(ribo.object, name = NULL, print = TRUE)
}
\arguments{
\item{ribo.object}{object of class 'ribo'}

\item{name}{The name of the experiment}

\item{print}{Logical value indicating whether or not to neatly print the output}
}
\value{
If a valid experiment name is provided, a list of elements providing all of the metadata of the
experiment is returned.

If the name is not provided and the root file has metadata, then a list of elements
providing all of the metadata found in the root file is returnend.
}
\description{
\code{\link{get_metadata}} provides information on all of the user-inputted
metadata of an experiment. If the experiment is not found, then the
attributes of the root .ribo file is returned instead.
}
\examples{
#ribo object use case
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#the ribo file contains an experiment named 'Hela_1'
get_metadata(sample, "Hela_1")

}
\seealso{
\code{\link{Ribo}} to generate the necessary ribo.object parameter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq_functions.R
\name{get_rnaseq}
\alias{get_rnaseq}
\title{Information on the RNA-Seq data of the experiments, if any}
\usage{
get_rnaseq(
  ribo.object,
  tidy = TRUE,
  region = c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3"),
  experiment = experiments(ribo.object),
  compact = TRUE,
  alias = FALSE
)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{tidy}{Option to return the data frame in a tidy format}

\item{region}{Specific region(s) of interest}

\item{experiment}{List of experiment names}

\item{compact}{Option to return a DataFrame with Rle and factor as opposed to a raw data.frame}

\item{alias}{Option to report the transcripts as aliases/nicknames}
}
\value{
An annotated data frame containing the RNA-Seq counts for the regions in specified in the `region` parameter with the option of
presenting the data in a tidy format. Additionally, the function returns a DataFrame with Rle and factor applied if the `compact` parameter
is set to TRUE and a data.frame without any Rle or factor if the `compact` parameter is set to FALSE
}
\description{
\code{\link{get_rnaseq}} returns a data frame containing information on the transcript name, experiment, and
sequence abundance
}
\details{
As a default value, experiment.list is presumed to include all of the
experiments within a ribo file. RNA-Seq data is an optional dataset to
include in a .ribo file. The experiments in experiment.list are checked
for experiment existence in the ribo file and then checked for RNA-seq data.

The returned DataFrame can either be in the tidy format for easier data
cleaning or in a condensed non-tidy format. The data will present RNA-seq counts
for each transcript in each valid experiment in experiment.list.

The 'alias' parameter specifies whether or not the returned DataFrame
should present each transcript as an alias instead of the original name.
If 'alias' is set to TRUE, then the column of the transcript names will
contain the aliases rather than the original reference names of the .ribo
file.
}
\examples{
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#list out the experiments of interest that have RNA-Seq data
experiments <- c("Hela_1", "Hela_2", "WT_1")
regions <- c("UTR5", "CDS", "UTR3")
rnaseq.data <- get_rnaseq(ribo.object = sample,
                          tidy = TRUE,
                          region = regions,
                          experiment = experiments)


}
\seealso{
\code{\link{Ribo}} to generate the necessary ribo.object parameter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotation_functions.R
\name{get_region_lengths}
\alias{get_region_lengths}
\title{Returns the overall length of each region with UTR Junctions}
\usage{
get_region_lengths(ribo.object, alias = FALSE)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{alias}{Option to return the transcript names as aliases}
}
\value{
A data.frame of the region lengths
}
\description{
The function {\code{\link{get_region_lengths}}} retrieves the 
lengths for the UTR5, UTR5 Junction, CDS, UTR3 Junction,
and UTR3 regions of every transcript.
}
\details{
This function is deprecated, and we recommend {\code{\link{get_internal_region_lengths}}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/region_count_functions.R
\name{get_region_counts}
\alias{get_region_counts}
\title{Retrieves the region counts from a .ribo file}
\usage{
get_region_counts(
  ribo.object,
  range.lower = length_min(ribo.object),
  range.upper = length_max(ribo.object),
  length = TRUE,
  transcript = TRUE,
  tidy = TRUE,
  alias = FALSE,
  normalize = FALSE,
  region = c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3"),
  compact = TRUE,
  experiment = experiments(ribo.object)
)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{range.lower}{Lower bound of the read length, inclusive}

\item{range.upper}{Upper bound of the read length, inclusive}

\item{length}{Logical value that denotes if the region count information should be summed across read lengths}

\item{transcript}{Logical value that denotes if the region count information should be summed across transcripts}

\item{tidy}{Option to return the data frame in a tidy format}

\item{alias}{Option to report the transcripts as aliases/nicknames}

\item{normalize}{Option to normalize the counts as counts per million reads}

\item{region}{Specific region of interest}

\item{compact}{Option to return a DataFrame with Rle and factor as opposed to a raw data.frame}

\item{experiment}{List of experiment names}
}
\value{
An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
region count information for the regions specified in the 'region' parameter. The returned data frame
will have a length column when the 'length' parameter is set to FALSE, indicating that the count
information will not be summed across the provided range of read lengths. Similarly, the returned data
frame will have a transcript column when the 'transcript' parameter is set to FALSE, indicating that the
count information will not be summed across the transcripts. In the case that transcript parameter is
'FALSE', the returned data frame will present the transcripts according to the aliases specified at the
creation of the ribo object if the 'alias' parameter is set to TRUE.
}
\description{
\code{\link{get_region_counts}} will return the particular region counts
of any subset of regions for a given set of experiments.
}
\details{
This function will return a data frane of the counts at each specified region
for each specified experiment. The region options are "UTR5", "UTR5J", "CDS",
"UTR3J", and "UTR3". The user can specify any subset of regions in the form of a vector,
a list, or a single string if only one region is desired.

The dimensions of the returned DataFrame depend on the parameters
range.lower, range.upper, length, and transcript.

The param 'length' condenses the read lengths together.
When length is TRUE and transcript is FALSE, the
data frame presents information for each transcript across
all of the read lengths. That is, each transcript has a value
that is the sum of all of the counts across every read length.
As a result, information about the transcript at each specific
read length is lost.

The param 'transcript' condenses the transcripts together.
When transcript is TRUE and length is FALSE data
frame presents information at each read length between range.lower and
range.upper inclusive. That is, each separate read length denotes the
sum of counts from every transcript. As a result, information about the
counts of each individual transcript is lost.

When 'transcript' is set to FALSE, the 'alias' parameter specifies whether
or not the returned DataFrame should present each transcript as an alias
instead of the original name. If 'alias' is set to TRUE, then the column
of the transcript names will contain the aliases rather than the original
reference names of the .ribo file.

If both 'length' and 'transcript' are TRUE, then the resulting
DataFrame prints out one row for each experiment. This provides the metagene
information across all transcripts and all reads in a given experiment.

If both length' and 'transcript' are FALSE, calculations are done to the data,
all information is preserved for both the read length and the transcript.
The DataFrame would just present the entire stored raw data
from the read length 'range.lower' to the read length 'range.upper' which in most
cases would result in a slow run time with a massive DataFrame returned.

When 'transcript' is set to FALSE, the 'alias' parameter specifies whether
or not the returned DataFrame should present each transcript as an alias
instead of the original name. If 'alias' is set to TRUE, then the column
of the transcript names will contain the aliases rather than the original
reference names of the .ribo file.
}
\examples{
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#specify the regions and experiments of interest
regions <- c("UTR5", "UTR5J", "CDS", "UTR3J", "UTR3")
experiments <- c("Hela_1", "Hela_2", "WT_1")

#obtains the region counts at each individual read length, summed across every transcript
region.counts <- get_region_counts(ribo.object = sample,
                                   region = regions,
                                   range.lower = 2,
                                   range.upper = 5,
                                   length = FALSE,
                                   transcript = TRUE,
                                   tidy = FALSE,
                                   alias = FALSE,
                                   experiment = experiments)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_ribo.R
\name{set_aliases}
\alias{set_aliases}
\title{Set the aliases of a ribo object}
\usage{
set_aliases(ribo.object, rename)
}
\arguments{
\item{ribo.object}{A 'ribo' object}

\item{rename}{A function that renames original transcript name into an alias}
}
\value{
A modified 'ribo' object that contains alias information
}
\description{
The function \code{\link{set_aliases}} allows the user to add aliases to a valid
ribo object.
}
\details{
If there is a different naming convention from the default appris transcriptome,
there may be no simple way to generate convenient aliases from the original reference names.
As a result, the user can first generate the ribo object and get the reference names, use custom
(and likely more intricate) functions to generate a list of aliases, and then pass in a character
vector of these aliases. The character vector should match the order of and correspond to the
list of reference names retrieved from \code{\link{get_reference_names}}
}
\examples{
#generate a ribo object with transcript nicknames/aliases
file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
sample <- Ribo(file.path)
sample <- set_aliases(ribo.object = sample,
                      rename = rename_default)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/region_count_functions.R
\name{plot_length_distribution}
\alias{plot_length_distribution}
\title{Plots the length distribution}
\usage{
plot_length_distribution(
  x,
  region,
  experiment,
  range.lower,
  range.upper,
  fraction = FALSE,
  title = "Length Distribution"
)
}
\arguments{
\item{x}{A 'Ribo' object or a DataFrame generated from \code{\link{get_region_counts}}}

\item{region}{the region of interest}

\item{experiment}{a list of experiment names}

\item{range.lower}{a lower bounds for a read length range}

\item{range.upper}{an upper bounds for a read length range}

\item{fraction}{logical value that, if TRUE, presents the count as a fraction of the total reads in the given ranges}

\item{title}{a title for the generated plot}
}
\value{
A 'ggplot' of the length distribution
}
\description{
The function \code{\link{plot_length_distribution}} can take either a DataFrame
or a "Ribo" object to generate a line graph of the length distributions from
range.lower to range.upper.
}
\details{
The param 'fraction' will plot the fractions of each length relative
to the total sum of the read length range provided by param 'range.lower'
and 'range.upper'. When fraction is set to FALSE, the total count of each
read length is plotted.

When given a "Ribo" object, \code{\link{plot_length_distribution}} calls
\code{\link{get_region_counts}} to retrieve the necessary information
for plotting.

The user can instead provide a DataFrame with the same structure as the
output of the \code{\link{get_region_counts}} function where the 'transcript'
parameter is set to FALSE and 'length' parameters is the default value of
TRUE. This also means that the many of the remaining parameters of the
\code{\link{plot_length_distribution}} function are not necessary. The run
time becomes substantially faster when \code{\link{plot_region_counts}} is
given the direct DataFrame to plot. Note that there is no manipulation by
this function on the DataFrame. This responsibility is given to the user
and allows for more control.
}
\examples{
#ribo object use case

#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#specify experiments of interest
experiments <- c("Hela_1", "Hela_2", "WT_1")

plot_length_distribution(x = sample,
                         region = "CDS",
                         range.lower = 2,
                         range.upper = 5,
                         experiment = experiments,
                         fraction = TRUE)


#DataFrame use case
#obtains the region counts at each individual read length, summed across every transcript
region.counts <- get_length_distribution(ribo.object = sample,
                                         region      = "CDS",
                                         range.lower = 2,
                                         range.upper = 5,
                                         experiment  = experiments)

#the param 'length' must be set to FALSE and param 'transcript' must be set
#to TRUE to use a DataFrame
plot_length_distribution(region.counts)


}
\seealso{
\code{\link{get_region_counts}} to generate a DataFrame that can
be provided as input,
\code{\link{Ribo}} to create a ribo.object that can be provided as input
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info_functions.R
\name{get_info}
\alias{get_info}
\title{Get information about the .ribo file}
\usage{
get_info(ribo.object)
}
\arguments{
\item{ribo.object}{ribo.object is an S4 object of class "Ribo"}
}
\value{
Returns a list containing a nested list of file attributes, a logical
value denoting whether the root file has additional metadata, and a
data.frame of information on each experiment
}
\description{
The function \code{\link{get_info}} provides information on the attributes, metadata,
and datasets of the ribo file.
}
\details{
The \code{\link{get_info}} first provides information on the format version, left_span, right_span,
longest read length, shortest read length, metagene_radius, and reference model. The last element of the
returned list contains the information about the presence of coverage and RNA-seq data which are
optional datasets to include in a .ribo file.
}
\examples{
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#retrieve information
get_info(sample)

}
\seealso{
\code{\link{Ribo}} to generate the necessary ribo.object parameter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ribo_class.R, R/ribo_methods.R, R/create_ribo.R
\docType{class}
\name{Ribo-class}
\alias{Ribo-class}
\alias{show,Ribo-method}
\alias{path,Ribo-method}
\alias{experiments,Ribo-method}
\alias{format_version,Ribo-method}
\alias{reference,Ribo-method}
\alias{length_min,Ribo-method}
\alias{length_max,Ribo-method}
\alias{left_span,Ribo-method}
\alias{right_span,Ribo-method}
\alias{metagene_radius,Ribo-method}
\alias{length_offset,Ribo-method}
\alias{has_metadata,Ribo-method}
\alias{experiment_info,Ribo-method}
\alias{transcript_info,Ribo-method}
\alias{alias_hash,Ribo-method}
\alias{original_hash,Ribo-method}
\alias{Ribo}
\alias{experiments}
\alias{alias_hash}
\alias{experiment_info}
\alias{format_version}
\alias{has_metadata}
\alias{left_span}
\alias{length_max}
\alias{length_min}
\alias{length_offset}
\alias{metagene_radius}
\alias{original_hash}
\alias{path}
\alias{reference}
\alias{right_span}
\alias{transcript_info}
\title{Ribo Class}
\usage{
\S4method{show}{Ribo}(object)

\S4method{path}{Ribo}(object)

\S4method{experiments}{Ribo}(object)

\S4method{format_version}{Ribo}(object)

\S4method{reference}{Ribo}(object)

\S4method{length_min}{Ribo}(object)

\S4method{length_max}{Ribo}(object)

\S4method{left_span}{Ribo}(object)

\S4method{right_span}{Ribo}(object)

\S4method{metagene_radius}{Ribo}(object)

\S4method{length_offset}{Ribo}(object)

\S4method{has_metadata}{Ribo}(object)

\S4method{experiment_info}{Ribo}(object)

\S4method{transcript_info}{Ribo}(object)

\S4method{alias_hash}{Ribo}(object)

\S4method{original_hash}{Ribo}(object)

Ribo(path, rename = NULL)
}
\arguments{
\item{object}{Ribo object}

\item{path}{The path to the .ribo file}

\item{rename}{A function that renames the original transcript or an already generated
character vector of aliases}
}
\value{
Returns an S4 object of class "Ribo" containing a path to the HDF5 file,
        various attributes in the root folder, and information about the transcripts 
        such as names and lengths
}
\description{
The Ribo object serves as the main utility vehicle for the ribor package.
Specifically, it allows the user to interface with a .ribo file in the R
ribor rely on the Ribo object to read, visualize, and inspect the
contents of the .ribo file. 
The information stored in this object include the .ribo file path, the 
list of experiments, the format version, the reference model, the minimum
read length, maximum read length, the left span, the right span, and 
other transcript information.
}
\details{
Note that the path parameter takes in a file path and stores it. While 
using the package, be sure to not to move or change the location of the 
.ribo file. The default names of the transcripts may be difficult to use depending on the 
settings used to generate the .ribo file. As a result, we have provided a 
rename parameter that integrates well with the Appris reference transcriptome.
Users may also define a simple function that processes a given default 
transcript name in a one-to-one manner to another custom alias.
}
\examples{
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

show(sample)

#generate a ribo object with transcript nicknames/aliases
file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
sample <- Ribo(file.path, rename = rename_default )
}
\seealso{
If a ribo object is already generated but aliases want to be added or updated, use the
\code{\link{set_aliases}} function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{get_reference_names}
\alias{get_reference_names}
\title{Retrieves a list of reference names}
\usage{
get_reference_names(ribo.object)
}
\arguments{
\item{ribo.object}{A 'Ribo' object-=09}
}
\value{
a list of the reference names
}
\description{
Gets a list of reference names by reading directly from the .ribo file
}
\examples{
#generate a ribo object with transcript nicknames/aliases
file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
sample <- Ribo(file.path)

#get the reference names
names <- get_reference_names(sample)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metagene_functions.R
\name{plot_metagene}
\alias{plot_metagene}
\title{Plots the metagene coverage data}
\usage{
plot_metagene(
  x,
  site,
  experiment,
  range.lower,
  range.upper,
  normalize = FALSE,
  title = "Metagene Site Coverage",
  tick = 10
)
}
\arguments{
\item{x}{A 'Ribo' object or a data frame generated from \code{\link{get_metagene}}}

\item{site}{"start" or "stop" site}

\item{experiment}{list of experiments}

\item{range.lower}{lower bound of the read length, inclusive}

\item{range.upper}{upper bound of the read length, inclusive}

\item{normalize}{When TRUE, normalizes the data by the total reads.}

\item{title}{title of the generated plot}

\item{tick}{x-axis labeling increment}
}
\value{
A 'ggplot' of the metagene site coverage
}
\description{
The function \code{\link{plot_metagene}} plots the metagene site coverage,
separating by experiment.
}
\details{
If a DataFrame is provided as param 'x', then the only additional parameter
is the optional title' parameter for the generated plot. If a ribo.object is
provided as param 'x', the rest of the parameters listed are necessary.

When given a ribo class object, the \code{\link{plot_metagene}} function
generates a DataFrame by calling the \code{\link{get_tidy_metagene}}
function, so the run times in this case will be mostly comprised of a call
to the \code{\link{get_metagene}} function.

This function uses ggplot in its underlying implementation.
}
\examples{
#a potential use case is to directly pass in the ribo object file as param 'x'

#generate the ribo object to directly use
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#specify experiments of interest
experiments <- c("Hela_1", "Hela_2", "WT_1")

#plot the metagene start site coverage for all experiments in 'sample.ribo'
#from read length 2 to 5
plot_metagene(x = sample,
              site = "start",
              range.lower = 2,
              range.upper = 5,
              experiment = experiments)

#Note that the site, range.lower, range.upper, and experiment parameter are only
#necessary if a ribo object is being passed in as param 'x'. If a ribo
#object is passed in, then the param 'experiments' will be set to all of
#the experiments by default.

#If a DataFrame is passed in, then the plot_metagene function
#does not need any other information. All of the elements of the DataFrame
#will be used, assuming that it contains the same column names and number of
#columns as the output from get_tidy_metagene()

#gets the metagene start site coverage from read length 2 to 5
#note that the data must be summed across transcripts and read lengths
#for the plot_metagene function
data <- get_tidy_metagene(sample,
                          site = "start",
                          range.lower = 2,
                          range.upper = 5)

#plot the metagene data
plot_metagene(data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage_functions.R
\name{get_coverage}
\alias{get_coverage}
\title{Retrieves the coverage data for a given transcript}
\usage{
get_coverage(
  ribo.object,
  name,
  range.lower = length_min(ribo.object),
  range.upper = length_max(ribo.object),
  length = TRUE,
  tidy = FALSE,
  alias = FALSE,
  compact = TRUE,
  experiment = experiments(ribo.object)
)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{name}{Transcript Name}

\item{range.lower}{Lower bound of the read length, inclusive}

\item{range.upper}{Upper bound of the read length, inclusive}

\item{length}{Logical value that denotes if the coverage should be summed across read lengths}

\item{tidy}{Logical value denoting whether or not the user wants a tidy format}

\item{alias}{Option to accept the transcript input as aliases/nicknames}

\item{compact}{Option to return a DataFrame with Rle and factor as opposed to a raw data.frame}

\item{experiment}{List of experiments to obtain coverage information on}
}
\value{
An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
coverage information for the provided list of 'experiments' in the 'experiment' parameter. The
returned object will have a length column when the 'length' parameter is set to FALSE, indicating
that the user does not want to sum the count information across the range of read lengths. The
returned data frame has the option of being tidy, and if the 'tidy' parameter is set to TRUE,
a position column will be added. Finally, if the 'alias' parameter is set to TRUE, the alias transcript
name must have been provided at the generation of the ribo object, and the function
will accept this aliased name in the 'transcript' parameter.
}
\description{
The function \code{\link{get_coverage}} generates a DataFrame of coverage
data over the length of a given transcript.
}
\details{
The function \code{\link{get_coverage}} first checks the experiments in the
'experiments' parameter to see if they are present in the .ribo file.
It will then check these experiments for coverage data which is an optional
dataset. As a result, this function safe guards against experiments that
do not have coverage data, but it also, by default, includes all of the
experiments in a file in the experiments' parameter.

The function checks the coverage of one transcript at a time at
each read length from 'range.lower' to 'range.upper', inclusive. However,
the parameter 'length' allows the user to obtain the coverage
information of a transcript across the range of read lengths indicated by
'range.lower' and 'range.upper'.

If the ribo.object is generated with aliases, the 'alias' parameter, if set to TRUE,
allows the user to use the alias of the transcript as the 'name' parameter instead of
the original transcript name.
}
\examples{
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#get the experiments of interest that also contain coverage data
experiments <- c("Hela_1", "Hela_2", "Hela_3", "WT_1")

#the ribo file contains a transcript named 'MYC'
coverage.data <- get_coverage(ribo.object = sample,
                              name = "MYC",
                              range.lower = 2,
                              range.upper = 5,
                              length = TRUE,
                              experiment = experiments)
}
\seealso{
\code{\link{Ribo}} to generate the necessary ribo.object parameter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotation_functions.R
\name{get_original_region_coordinates}
\alias{get_original_region_coordinates}
\title{Retrieves the region stop and start coordinates}
\usage{
get_original_region_coordinates(ribo.object, alias = FALSE)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{alias}{Option to return the transcript names as aliases}
}
\value{
A data.frame of start and stop coordinates for every region
}
\description{
The function {\code{\link{get_original_region_coordinates}}} retrieves the 
start and site positions for the UTR5, UTR5 Junction, CDS, UTR3 Junction,
and UTR3 regions of every transcript.
}
\details{
To note, because of the R-specific 1-based indexing, the positions start at
1 instead of 0 in other programming languages. The positions provided in 
the returned data.frame will correspond to the positions in the output of 
{\code{\link{get_coverage}}}.

Additionally, within the transcripts, there are edge cases. 
NA values found in the returned data.frame means that the region has no 
start and stop position and a length of zero after computing the boundaries
of the UTR5 and UTR3 junction.
}
\examples{
# generate a ribo object 
file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
sample <- Ribo(file.path, rename = rename_default)

# get the region coordinates
coord <- get_original_region_coordinates(sample, alias = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ribor.R
\docType{package}
\name{ribor}
\alias{ribor}
\title{ribor: A package for reading .ribo files}
\description{
The 'ribor' package offers a suite of reading functions for the datasets
present in a .ribo file and also provides some rudimentary plotting 
functions.
}
\section{Vignette}{

To get started with the ribor package, please see the vignette page at
\url{https://ribosomeprofiling.github.io/ribor/ribor.html}.
}

\section{Related Tools}{


The paper associated with the Ribo ecosystem can be found at
\url{https://academic.oup.com/bioinformatics/advance-article/doi/10.1093/bioinformatics/btaa028/5701654}.

For more information on the preprocessing pipeline, please see the link to
the source code at \url{https://github.com/ribosomeprofiling/riboflow}.

For more information on the .ribo file format, please see its documentation
page at \url{https://ribopy.readthedocs.io/en/latest/ribo_file_format.html}.

For an alternative to ribor, please see a link to source code of ribopy, 
a python interface, at \url{https://github.com/ribosomeprofiling/ribopy}.
}

\section{Package Content}{

\subsection{Generating a ribo object}{
 \code{\link{Ribo}} to get started 
}

\subsection{Length Distribution}{
 \code{\link{get_length_distribution}} to get length distribution counts
 
 \code{\link{plot_length_distribution}} to plot the length distribution
}

\subsection{Region Counts}{
  \code{\link{get_region_counts}} to get region counts
   
  \code{\link{plot_region_counts}} to plot the region counts 
}

\subsection{Metagene Coverage}{
  \code{\link{get_metagene}} to get metagene site coverage
  
  \code{\link{get_tidy_metagene}} to get a tidy format of the metagene site coverage
  
  \code{\link{plot_metagene}} to plot the metagene site coverage
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info_functions.R
\name{get_experiments}
\alias{get_experiments}
\title{Provides a list of experiments from a .ribo file}
\usage{
get_experiments(ribo.object)
}
\arguments{
\item{ribo.object}{S4 object of class "Ribo"}
}
\value{
A list of the experiment names
}
\description{
The function \code{\link{get_experiments}} provides a list of experiment names in the .ribo file.
}
\details{
\code{\link{get_experiments}} returns a list of strings denoting the experiments. It obtains this
by reading directly from the .ribo file through the path of the 'ribo.object' parameter. To generate
the param 'ribo.object', call the \code{\link{Ribo}} function and provide the path to the .ribo file of interest.

The user can then choose to create a subset from this list for any specific experiments of interest
for later function calls. Many functions that have the param 'experiment.list'
 call \code{\link{get_experiments}} to generate a default list of all experiments in the
.ribo file.
}
\examples{
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#get a list of the experiments
get_experiments(sample)

}
\seealso{
\code{\link{Ribo}} to generate the necessary ribo.object parameter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metagene_functions.R
\name{get_tidy_metagene}
\alias{get_tidy_metagene}
\title{Retrieves the metagene data in a tidy format}
\usage{
get_tidy_metagene(
  ribo.object,
  site,
  range.lower = length_min(ribo.object),
  range.upper = length_max(ribo.object),
  length = TRUE,
  compact = TRUE,
  experiment = experiments(ribo.object)
)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{site}{"start" or "stop" site coverage}

\item{range.lower}{Lower bound of the read length, inclusive}

\item{range.upper}{Upper bound of the read length, inclusive}

\item{length}{Logical value that denotes if the metagene information should be summed across read lengths}

\item{compact}{Option to return a DataFrame with Rle and factor as opposed to a raw data.frame}

\item{experiment}{List of experiment names}
}
\value{
An annotated, tidy DataFrame or data.frame (if the compact parameter is set to FALSE) of the
metagene information for either the 'stop' or 'start' site provided in the 'site' parameter. The data frame,
as a result of its tidy property, will have a position column.
The returned data frame will have a length column when the 'length' parameter is set to FALSE, indicating
will be automatically aggregated to keep the memory footprint of this function reasonable.
}
\description{
The function \code{\link{get_tidy_metagene}} provides the user with a tidy data format for easier
data cleaning and manipulation. In providing this functionality while reducing the returned data frame
size, the user must aggregate across the transcripts and is only provided the option to aggregate the 
read lengths together.
}
\details{
The dimensions of the returned data frame depend on the parameters
range.lower, range.upper, and length.

The param 'length' condenses the read lengths together.
When length is TRUE, then the resulting data frame prints out one row
for each experiment. This provides a tidy format of the metagene information
across all transcripts and all read lengths in a given experiment. Each row
in the data frame represents the total metagene coverage count of a given experiment
at a given position.

When the param  'length' is FALSE, then the resulting data frame prints out the
metagene coverage count at each position of the metagene radius for each read length.
This provides a tidy format of the metagene information across the transcripts, preserving
the metagene coverage count at each read length.
}
\examples{
#generate the ribo object by loading in a ribo function and calling the \code{\link{Ribo}} function
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#extract the total metagene information in a tidy format
#for all experiments across the read lengths and transcripts
#of the start site from read length 2 to 5

metagene_info <- get_tidy_metagene(ribo.object = sample,
                                   site = "start",
                                   range.lower = 2,
                                   range.upper = 5,
                                   length = TRUE,
                                   experiment = experiments(sample))

#Note that length and experiments in this case are the
#default values and can be left out. The following generates the same output.
metagene_info <- get_tidy_metagene(ribo.object = sample,
                                   site = "start",
                                   range.lower = 2,
                                   range.upper = 5)

}
\seealso{
\code{\link{Ribo}} to generate the necessary 'Ribo' class object.
\code{\link{plot_metagene}} to visualize the metagene data,
\code{\link{get_metagene}} to obtain tidy metagene data under certain conditions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{rename_default}
\alias{rename_default}
\title{Rename function for appris transcriptome naming convention}
\usage{
rename_default(x)
}
\arguments{
\item{x}{Character denoting original name of the transcript}
}
\value{
Character denoting simplified name of the object
}
\description{
The function {\code{\link{rename_default}}} is the default renaming function for the
appris human transcriptome. It takes one single transcript name and returns a simplified
alias.
}
\examples{
original <- paste("ENST00000613283.2|ENSG00000136997.17|",
                  "OTTHUMG00000128475.8|-|MYC-206|MYC|1365|protein_coding|",
                  sep = "")
alias <- rename_default(original)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{rename_transcripts}
\alias{rename_transcripts}
\title{Renames the transcripts}
\usage{
rename_transcripts(ribo, rename)
}
\arguments{
\item{ribo}{a path to the ribo file or a 'Ribo' object}

\item{rename}{A function that renames the original transcript or an already generated
character vector of aliases}
}
\value{
A character vector denoting the renamed transcript aliases
}
\description{
The function \code{\link{rename_transcripts}} strives to make the transcript names less
cumbersome to write and easier to use.
}
\details{
Transcript names found in a .ribo file can often be long and inconvenient to use.
As a result, this function allows the user to rename the transcripts.

Often times, a short function can be used on the ribo file reference names
to split and extract a more convenient name, and a function with a similar input and
output to {\code{\link{rename_default}}} can be passed in.

However, if there is no simple function that takes the original name and renames it into
a unique alias, then the user can provide a character vector of the same length as the number of
transcripts in the ribo file. This character vector would provide aliases that match the order
of the original reference names returned by the {\code{\link{get_reference_names}}} function.
}
\examples{
file.path <- system.file("extdata", "HEK293_ingolia.ribo", package = "ribor")
sample <- Ribo(file.path, rename = rename_default)

aliases <- rename_transcripts(sample, rename = rename_default)
}
\seealso{
{\code{\link{rename_default}}} to view expected input and output of a 'rename' function
{\code{\link{Ribo}}} to generate a ribo object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metagene_functions.R
\name{get_metagene}
\alias{get_metagene}
\title{Retrieves the metagene data from a .ribo file}
\usage{
get_metagene(
  ribo.object,
  site,
  range.lower = length_min(ribo.object),
  range.upper = length_max(ribo.object),
  transcript = TRUE,
  length = TRUE,
  alias = FALSE,
  compact = TRUE,
  experiment = experiments(ribo.object)
)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{site}{"start" or "stop" site coverage}

\item{range.lower}{Lower bound of the read length, inclusive}

\item{range.upper}{Upper bound of the read length, inclusive}

\item{transcript}{Logical value that denotes if the metagene information should be summed across transcripts}

\item{length}{Logical value that denotes if the metagene information should be summed across read lengths}

\item{alias}{Option to report the transcripts as aliases/nicknames}

\item{compact}{Option to return a DataFrame with Rle and factor as opposed to a raw data.frame}

\item{experiment}{List of experiment names}
}
\value{
An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
metagene information for either the 'stop' or 'start' site provided in the 'site' parameter. The
returned data frame will have a length column when the 'length' parameter is set to FALSE, indicating
the returned data frame will have a transcript column whe the 'transcript' parameter is set to FALSE,
indicating that the count information will not be summed across the transcripts.
In the case that transcript parameter is 'FALSE', the returned data frame will present the transcripts according
to the aliases specified at the creation of the ribo object if the 'alias' parameter is set to TRUE.
}
\description{
The function \code{\link{get_metagene}} returns a data frame that provides
the coverage at the positions surrounding the metagene start or stop site.
}
\details{
The dimensions of the returned data frame depend on the parameters
range.lower, range.upper, length, and transcript.

The param 'length' condenses the read lengths together.
When length is TRUE and transcript is FALSE, the
data frame presents information for each transcript across
all of the read lengths. That is, each transcript has a value
that is the sum of all of the counts across every read length.
As a result, information about the transcript at each specific
read length is lost.

The param 'transcripts' condenses the transcripts together.
When transcript is TRUE and length is FALSE, the data
frame presents information at each read length between range.lower and
range.upper inclusive. That is, each separate read length denotes the
sum of counts from every transcript. As a result, information about the
counts of each individual transcript is lost.

If both 'length' and 'transcript' are TRUE, then the resulting
data frame prints out one row for each experiment. This provides the metagene
information across all transcripts and all reads in a given experiment.

If both length' and 'transcript' are FALSE, no calculations are done to the data,
all information is preserved for both the read length and the transcript.
The data frame would just present the entire stored raw data
from the read length 'range.lower' to the read length 'range.upper' which in most
cases would result in a slow run time with a massive DataFrame returned.

When 'transcript' is set to FALSE, the 'alias' parameter specifies whether
or not the returned DataFrame should present each transcript as an alias
instead of the original name. If 'alias' is set to TRUE, then the returned
data frame will contain the aliases rather than the original
reference names of the .ribo file.
}
\examples{

#generate the ribo object by providing the file.path to the ribo file
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)


#extract the total metagene information for all experiments
#across the read lengths and transcripts of the start site
#from read length 2 to 5
metagene_info <- get_metagene(ribo.object = sample,
                              site = "start",
                              range.lower = 2,
                              range.upper = 5,
                              length = TRUE,
                              transcript = TRUE,
                              experiment = experiments(sample))


#Note that length, transcript, and experiments in this case are the
#default values and can be left out. The following generates the same output.

metagene_info <- get_metagene(ribo.object = sample,
                              site = "start",
                              range.lower = 2,
                              range.upper = 5)

}
\seealso{
\code{\link{Ribo}} to generate the necessary 'Ribo' class object,
\code{\link{plot_metagene}} to visualize the metagene data,
\code{\link{get_tidy_metagene}} to obtain tidy metagene data under certain conditions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/region_count_functions.R
\name{plot_region_counts}
\alias{plot_region_counts}
\title{Plots the region counts of UTR5, CDS, and UTR3}
\usage{
plot_region_counts(
  x,
  experiment,
  range.lower,
  range.upper,
  title = "Region Counts"
)
}
\arguments{
\item{x}{A 'Ribo' object or a DataFrame generated from \code{\link{get_region_counts}}}

\item{experiment}{a list of experiment names}

\item{range.lower}{a lower bounds for a read length range}

\item{range.upper}{an upper bounds for a read length range}

\item{title}{a title for the generated plot}
}
\value{
A 'ggplot' of the region counts for each of the experiments
}
\description{
The function \code{\link{plot_region_counts}} can take either a DataFrame
or a "Ribo" object to generate the a stacked bar plot of proportions that
correspond to the "UTR5", "CDS", and "UTR3" regions.
}
\details{
When given a 'Ribo' object, \code{\link{plot_region_counts}} calls
\code{\link{get_region_counts}} to retrieve the necessary information
for plotting. This option is in the case that a DataFrame of the
region count information is not required.

The user can instead provide a DataFrame with the same structure as the
output of the \code{\link{get_region_counts}} function where the 'transcript'
and 'length' parameters are the default values of TRUE. This also means that
the remaining parameters of the \code{\link{plot_region_counts}} function are not necessary.
The run time becomes substantially faster when \code{\link{plot_region_counts}} is given
the direct DataFrame to plot. However, the DataFrame needs to follow the format and 
types in the output of the reading functions
}
\examples{
#ribo object use case
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#specify the regions and experiments of interest
regions <- c("UTR5", "CDS", "UTR3")
experiments <- c("Hela_1", "Hela_2", "WT_1")

plot_region_counts(sample,
                   range.lower = 2,
                   range.upper = 5,
                   experiments)

#DataFrame use case
#obtains the region counts at each individual read length, summed across every transcript
region.counts <- get_region_counts(sample,
                                   region = regions,
                                   range.lower = 2,
                                   range.upper = 5,
                                   tidy = TRUE,
                                   length = TRUE,
                                   transcript = TRUE)

#the params 'length' and 'transcript' must be set to true to use a DataFrame
plot_region_counts(region.counts)

}
\seealso{
\code{\link{get_region_counts}} to generate a DataFrame that can be provided as input,
\code{\link{Ribo}} to create a ribo.object that can be provided as input
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/region_count_functions.R
\name{get_length_distribution}
\alias{get_length_distribution}
\title{Retrieves the length distribution of a given region}
\usage{
get_length_distribution(
  ribo.object,
  region,
  range.lower = length_min(ribo.object),
  range.upper = length_max(ribo.object),
  compact = TRUE,
  experiment = experiments(ribo.object)
)
}
\arguments{
\item{ribo.object}{A 'Ribo' object}

\item{region}{Specific region of interest}

\item{range.lower}{Lower bound of the read length, inclusive}

\item{range.upper}{Upper bound of the read length, inclusive}

\item{compact}{Option to return a DataFrame with Rle and factor as opposed to a raw data.frame}

\item{experiment}{List of experiment names}
}
\value{
An annotated DataFrame or data.frame (if the compact parameter is set to FALSE) of the
read-length specific region count information for a single region specified in the 'region' parameter.
The returned data frame will have a length column, and it will not contain a transcript column.
}
\description{
The function {\code{\link{get_length_distribution}}} retrieves the raw or normalized
counts at each read length from 'range.lower' to 'range.upper'.
}
\details{
This function is a wrapper function of {\code{\link{get_region_counts}}}, and the
returned DataFrame is valid input for {\code{\link{plot_length_distribution}}}.
}
\examples{
#generate the ribo object
file.path <- system.file("extdata", "sample.ribo", package = "ribor")
sample <- Ribo(file.path)

#specify the experiments of interest
experiments <- c("Hela_1", "Hela_2", "WT_1")

#gets the normalized length distribution from read length 2 to 5
length.dist <- get_length_distribution(ribo.object = sample,
                                       region = "CDS",
                                       range.lower = 2,
                                       range.upper = 5)

}
\seealso{
{\code{\link{plot_length_distribution}}} to plot the output of this function
}
